// FILE: ./bower_components/zh-preferences/src/ZhPreferences.js
/**
 * Service to handle preferences storage
 *
 * @class
 * @param {RepositoryFactory} RepositoryFactory Service to factory repositories
 * @param {RequestEngine}     requestEngine     Service to make http requests
 * @param {RequestFactory}    requestFactory    Service to factory requests
 * @param {ZHPromise}         ZHPromise         Service to manipulate promises
 * @param {Query}             Query             Service to build queries
 */
function ZhPreferences(RepositoryFactory, requestEngine, requestFactory, ZHPromise, Query){

    var repositoryMap = {};

    var userWidgetConfiguration = {};

    /**
     * Register new repository to save preferences
     *
     * @param  {String} name Repository name used to save a preference
     */
    this.registerRepository = function(name){
        var repositoryName = name.split('#').pop();
        repositoryMap[repositoryName] = RepositoryFactory.factory(name, 'MEMORY');
    };


    /**
     * Persist preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to persist
     *
     * @return {Promise}                 Preferences saved
     */
    this.savePreferences = function(preferenceType, preferences){
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/save',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response){
            throw new Error(response.data.error);
        }).then(function(response){
            var returnedIds = response.dataset[preferenceType];
            return preferences.map(function(preference, index){
                preference.__is_new = false;
                preference.changed = false;
                preference.ID = returnedIds[index].ID;
                return preference;
            });
        }).then(function(preferences){
            return repositoryMap[preferenceType].save(preferences);
        });
    };

    /**
     * Delete preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to delete
     *
     * @return {Promise}                 Preferences deleted
     */
    this.deletePreferences = function(preferenceType, preferences){
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/remove',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response){
            throw new Error(response.data.error);
        }).then(function(response){
            var deletedIds = response.dataset[preferenceType].map(function(preference){
                return preference.ID;
            });
            var query = Query.build().where('ID').in(deletedIds);

            return repositoryMap[preferenceType].remove(query);
        });
    };

    /**
     * Get local saved preferences from repository
     *
     * @param  {String} preferenceType Repository name
     * @param  {Query}  query          Conditions to filter the preferences
     *
     * @return {Promise}               Array of preferences
     */
    this.getPreferences = function(preferenceType, query){
        var filterRequest = requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType,
            'requestType': 'filterData',
            'filter': query.where()
        });
        return requestEngine.doRequest(filterRequest)
            .then(function(response){
                var preferences = response.dataset[preferenceType];
                var removePreferences = repositoryMap[preferenceType].remove(query);
                var savePreferences = repositoryMap[preferenceType].save(preferences);
                return ZHPromise.all([removePreferences, savePreferences]).then(function(){
                    return preferences;
                });
            })
            .catch(function(response){
                throw new Error(response.data.error);
            });
    };

    /**
     * Get local saved preferences from repository filtering by widget and logged user
     *
     * @param  {String} preferenceType Repository name
     * @param  {String} widgetID       Widget identified
     *
     * @return {Promise}               Array of preferences
     */
    this.getWidgetPreferences = function(preferenceType, widgetID){
        var query = Query.build().where('WIDGET_ID').equals(widgetID);
        return this.getPreferences(preferenceType, query);
    };

    /**
     * Get default preferences for user on specified widget
     *
     * @param  {String} widgetID Widget to get default preferences
     *
     * @return {Promise}         Widget default preferences
     */
    this.getDefaultWidgetPreferences = function(widgetID){
        if(userWidgetConfiguration[widgetID] === undefined){
            var query = Query.build().where('WIDGET_ID').equals(widgetID);

            userWidgetConfiguration[widgetID] = requestEngine.doRequest(requestFactory.factory({
                'serviceName': 'zh-preferences#/default-preferences',
                'requestType': 'filterData',
                'filter': query.where()
            })).then(function(response){
                var dataset = response.dataset;
                var preference = dataset.user_widget.pop()|| {
                    '__is_new': true,
                    'ID': null,
                    'WIDGET_ID': widgetID,
                    'DEFAULT_FILTER': null,
                    'DEFAULT_LAYOUT': null,
                    'DEFAULT_VIEW': null,
                    'LAST_MODIFIED_DATE': null
                };
                if(dataset.defaultFilter && dataset.defaultFilter.length){
                    preference.defaultFilter = dataset.defaultFilter.pop();
                    preference.defaultFilter.FILTER_ROW = JSON.parse(preference.defaultFilter.FILTER_ROW);
                }
                if(dataset.defaultLayout && dataset.defaultLayout.length){
                    preference.defaultLayout = dataset.defaultLayout.pop();
                }
                if(dataset.defaultView && dataset.defaultView.length){
                    preference.defaultView = dataset.defaultView.pop();
                }
                return preference;
            });
        }

        return userWidgetConfiguration[widgetID];
    };

    /**
     * Change default preference for user on specified widget
     *
     * @param  {String}  configWidget        Preference configuration widget
     * @param  {String}  property            Property that contains the default preference ID
     * @param  {Integer} preference          Preference configuration widget changed row
     *
     * @return {Promise}                     Widget default preferences
     */
    this.changeDefaultPreferences = function(configWidget, property, preference){
        var widgetID = configWidget.listWidget.id;
        var preferenceID = preference[property] ? preference.ID : null;

        userWidgetConfiguration[widgetID] = this.getDefaultWidgetPreferences(widgetID)
            .then(function(defaultPreferences){
                defaultPreferences[property] = preferenceID;
                defaultPreferences.__is_new = defaultPreferences.__is_new || false;
                return ZHPromise.all([
                    defaultPreferences,
                    requestEngine.doRequest(requestFactory.factory({
                        'serviceName': 'zh-preferences#/default-preferences/save',
                        'requestType': 'dataSource',
                        'dataSource': [defaultPreferences]
                    }))
                ]);
            }).then(function(args){
                var defaultPreferences = args[0];
                var response = args[1];
                defaultPreferences.ID = Util.getNestedProperty(response, ['dataset', 'user_widget', '0', 'ID']);

                configWidget.dataSource.data.forEach(function(row){
                    row.__is_new = false;
                    row[property] = row.ID == defaultPreferences[property];
                });
                return defaultPreferences;
            });

        return userWidgetConfiguration[widgetID];
    };

    function isMasterDetail(template){
        return template.indexOf('master_detail/grid.html') !== -1 ||
               template.indexOf('master_detail/list.html') !== -1 ||
               template.indexOf('master_detail/tree/grid.html') !== -1;
    }

    function isMasterDetailView(widget){
        return isMasterDetail(widget.template) && widget.mode == ViewMode.VIEW;
    }

    this.validateWidget = function(widget){
        if(!widget.id){
            throw new Error('Widget\'s must have "id" to use Preferences. '+
                'Widget with name "'+widget.name+'" does not have an id.');
        }
        if(!widget.container.id){
            throw new Error('Container\'s must have "id" to use Preferences.');
        }

        return !isMasterDetailView(widget);
    };

    var userPromise = ZHPromise.defer();

    /**
     * Invalidate logged user and clear preferences loaded
     */
    this.unsetUser = function(){
        userPromise = ZHPromise.defer();
        userWidgetConfiguration = {};

        Object.keys(repositoryMap).forEach(function(repositoryName){
            repositoryMap[repositoryName].clearAll();
        });
    };

    /**
     * Set logged user
     *
     * @param {String} userID Logged user
     */
    this.setUserID = function(userID){
        userPromise.resolve(userID);
    };

    /**
     * Get logged user ID
     *
     * @return {Promise} User identifier
     */
    this.getUserID = function(){
        return userPromise.promise;
    };


    /**
     * Generate preference file to download
     *
     * @param {Object} preference       Preference to be exported
     * @param {String} fileName         File name
     *
     * @return {Promise<Object>}        Preference exported
     */
    this.downloadPreferenceFile = function(preference, fileName){
        var dataStr = "data:text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(preference,false,4));

        var linkElement = document.createElement('a');
        linkElement.setAttribute("href",     dataStr);
        linkElement.setAttribute("download", fileName);
        linkElement.click();
        linkElement.remove();

        return ZHPromise.when(preference);
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhPreferences', ZhPreferences);
});


// FILE: ./bower_components/zh-customizations-service/dist/zhCustomizationsService.js

// FILE: src/config/config.js
(function(){
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-customizations-service';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}
})();

// FILE: src/registers/registerEvents.js
Configuration(function(eventEngine) {
  eventEngine.registerEvent("FilterPreferenceOnClick", "filterOnClick");
  eventEngine.registerEvent("VisualizationPreferenceOnClick", "visualizationOnClick");
  eventEngine.registerEvent("LayoutPreferenceOnClick", "layoutOnClick");
});

// FILE: src/PreferencesUser.js
function PreferencesUser() {

    var userId = localStorage.getItem('userId');

    this.registerUser = function(primaryKey){
        userId = primaryKey;
    };

    this.getPreferenceUserRegistered = function(){
        return userId;
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("preferencesUserService", PreferencesUser);
});

// FILE: src/CustomizationActionInjector.js
(function() {

	function CustomizationActionInjector(templateManager, eventAggregator, ApplicationContext) {

		eventAggregator.subscribe("beforeProjectStartup", pushCustomizationAction);

		//------Dependencies
		var zhLayoutPreferences, zhFilterPreferences, zhCustomizationPreferences;

		function getDependencyByCustomizationType(type) {
			var dependency;
			if (type == "layout"){
				dependency = zhLayoutPreferences;
			} else if (type == "filter"){
				dependency = zhFilterPreferences;
			} else if (type =="view"){
				dependency = zhCustomizationPreferences;
			}
			return dependency;
		}

		// function getDefaultVisibilityByTemplate(widget, type) {
		// 	var defaults = [
		// 		{ template: 'widget/list.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/grid.html', 		options: { search: true, filter: true, layout: true, view: true },
		// 		{ template: 'widget/tree/grid.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/kanban.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/crosstab.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/form.html', 		options: { search: false, filter: false, layout: false, view: false },
		// 		{ template: 'widget/dashboard.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'container/dashboard.html', options: { search: true, filter: true, layout: false, view: false }
		// 	];
		// }

		function isCustomizationActionVisible(widget, type) {
			var dependency = getDependencyByCustomizationType(type);
			return dependency && (Util.isUndefined(widget.floatingControl) || 
								  Util.isUndefined(widget.floatingControl.customizationAction) ||
								  widget.floatingControl.customizationAction[type] !== false);
		}

		function pushCustomizationAction() {
			zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
			zhFilterPreferences = ApplicationContext.zhFilterPreferences;
			zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;

			var customFloatingActions = templateManager.project.customFloatingActions;
			var actionExistence = customFloatingActions.filter(function (action) {
				return action.id == "#hashReportsAction#";
			}).shift();
			if (!actionExistence) {
				templateManager.project.customFloatingActions.push({
					"id": "#hashFilterAction#",
					"name": "FilterAction",
					"label": "Search",
					"type": "filter",
					"order": 20,
					"icon": "search",
					"template": "zh-customizations-service#floating-filter-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'filter') || Util.isUndefined(widget.floatingControl) || widget.floatingControl.searchAction !== false; }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashLayoutsAction#",
					"name": "LayoutAction",
					"label": "Layouts",
					"type": "layout",
					"order": 30,
					"icon": "layout",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'layout'); }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashReportsAction#",
					"name": "ReportAction",
					"label": "Views",
					"type": "view",
					"order": 40,
					"icon": "view",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'view'); }
				});
			}
		}
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationActionInjector", CustomizationActionInjector);
	});

})();

// FILE: src/CustomizationsFloatingCardController.js
function CustomizationsFloatingCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {
	
	var boundCustomizationUpdateEvent = eventAggregator.subscribe("OnUpdateCustomizationsRepositories", fillFloatingCardRepositories);

	$scope.$on("destroy", function(){
		eventAggregator.unSubscribe("OnUpdateCustomizationsRepositories", boundCustomizationUpdateEvent);
	});
	
	//------Dependencies
	var zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
	var zhFilterPreferences = ApplicationContext.zhFilterPreferences;
	var zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;
	var zhCustomizationsService = ApplicationContext.zhCustomizationsService;
	
	//------ Control viariables
	var editingWidget = $scope.$parent.widget;
	var customizationIndex = 0;
	var floatingControl = editingWidget.floatingControl || {};
	var customizationsWidgetOptions = floatingControl.customizationAction;
	$scope.settingsCardLabel = ScreenService.i18n("Configurations");

	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			} else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}				
			}
		}
	}

	$scope.togglePreferencesCard = function(e){
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};
	editingWidget.togglePreferencesCard = $scope.togglePreferencesCard;

	var preferencesConfigsClick = function(){
		zhLayoutPreferences.startConfig(editingWidget);
	};

	var filtersConfigsClick = function(){
		zhFilterPreferences.startFilterConfig(editingWidget);
	};

	var viewsConfigsClick = function(){
		zhCustomizationPreferences.startViewSettings(editingWidget);
	};

	var preferencesRestoreClick = function(){
		zhLayoutPreferences.restoreDefault(editingWidget);
	};

	var filtersRestoreClick = function(){
		zhFilterPreferences.restoreDefault(editingWidget);
	};

	var viewsRestoreClick = function(){
		zhCustomizationPreferences.restoreDefault(editingWidget);
	};

	function applyFilter(customization){
		zhCustomizationsService.applyFilter(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyLayout(customization){
		zhCustomizationsService.applyLayout(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyVisualization(customization){
		zhCustomizationsService.applyVisualization(customization, editingWidget);
	}

	$scope.isCustomizationApplied = function(customization){
		if(editingWidget.customizationsApplied){
			var appliedCustomization = editingWidget.customizationsApplied[customization.customizationType];
			if(appliedCustomization == customization._id ){
				return true;
			}
		}
	};

	$scope.customizationsTypes = [
		{
			"label": ScreenService.i18n("Views"),
			"type": "view",
			"configsClick": viewsConfigsClick,
			"restoreClick": viewsRestoreClick,
			"applyOnClick": applyVisualization,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Layouts"),
			"type": "layout",
			"configsClick": preferencesConfigsClick,
			"restoreClick": preferencesRestoreClick,
			"applyOnClick": applyLayout,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Filters"),
			"type": "filter",
			"configsClick": filtersConfigsClick,
			"restoreClick": filtersRestoreClick,
			"applyOnClick": applyFilter,
			"dataSource": {
				"data": []
			}
		}
	];

	$scope.setActiveCustomization = function(type) {
		$scope.activeCustomization = $scope.customizationsTypes.filter(function(customizationType) {
			return customizationType.type == type;
		}).shift();
	};

	function fillFloatingCardRepositories(){
		return ZHPromise.all($scope.customizationsTypes.filter(function(customizationType) {
			return Util.isUndefined(editingWidget.floatingControl) || Util.isUndefined(editingWidget.floatingControl.customizationAction) || editingWidget.floatingControl.customizationAction[customizationType.type] !== false;
		}).map(function(customizationType){
			return zhCustomizationsService.getWidgetCustomizations(editingWidget, customizationType.type).then(function(custo){
				insertDefaultOption(custo, customizationType.type);
				customizationType.dataSource.data = custo;
				ScreenService.updateTemplate();
			});
		}));
	}
	fillFloatingCardRepositories();

	function insertDefaultOption(customizations, customizationType){
		var defaultOption = {
			"_id": "default",
			"label": ScreenService.i18n("Original"),
			"customizationType": customizationType,
			"isDefaultOption": true
		};
		if(defaultOption.customizationType == "filter" && zhFilterPreferences){
			defaultOption.filters = zhFilterPreferences.getDefaultFilterProperties(editingWidget) || [];
		}
		customizations.push(defaultOption);
	}

	$scope.previousCustomizationType = function(){
		if(customizationIndex === 0){
			customizationIndex = $scope.customizationsTypes.length - 1;
		} else {
			customizationIndex--;
		}
		updateCustomizationType(customizationIndex);
	};

	$scope.forwardCustomizationType = function(){
		if(customizationIndex === $scope.customizationsTypes.length - 1){
			customizationIndex = 0;
		} else {
			customizationIndex++;
		}
		updateCustomizationType(customizationIndex);
	};
}

// FILE: src/CustomizationsRequestsBuilder.js
(function() {

	function CustomizationsRequestsBuilder(templateManager) {

		this.buildRequest = function(typeRequest, method, dbProperties, dataset) {
			var request = {
				"typeRequest": typeRequest,
				"baseUrl": getbaseUrl(dbProperties),
				"service": getServiceUrl(dbProperties, method),
				"params": {
					"requestType": "DataSet",
					"dataset": dataset
				}
			};
			return request;
		};

		var getServiceUrl = function(dbProperties, method) {
			return !dbProperties.urlFullPath ? '/' + method + '/' + getRoute(dbProperties) : '';
		};

		var getbaseUrl = function(dbProperties) {
			return dbProperties.url ? dbProperties.url : templateManager.project.zhStoragePath;
		};

		var getRoute = function(dbProperties) {
			return dbProperties.dbName + "/" + dbProperties.collectionName;
		};
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationsRequestsBuilder", CustomizationsRequestsBuilder);
	});

})();

// FILE: src/CustomizationsService.js
(function() {

	function CustomizationService($rootScope, ScreenService, templateManager, RepositoryFactory, Query, ZhUtil, restEngine, ZHPromise, CustomizationsRequestsBuilder, ApplicationContext, containerMerger, eventAggregator, eventEngine, preferencesUserService, ZhLogger, ZhKeyboardService) {

		var self = this;
		var customizationsRepositories;
		var customizationsTypes = ["layout", "filter", "view"];
		var SUFFIX_ORIGINAL_VALUE = "original";
		var userPreference;

		function extendBehavior() {
			Util.extend(self, preferencesUserService);
		}
		extendBehavior();

		var validateRegister = function(register) {
			var valid = register && register.containerId && register.widgetId;
			if (!valid) {
				ZhLogger.warning('The container and the widget must have IDs to save customizations');
			}
			return valid;
		};

		var getCustomizationsRepositories = function() {
			return templateManager.projectPromise.then(function() {
				customizationsRepositories = templateManager.project.customizationsRepositories || [];
			});
		};

		var createCustomizationsRepositories = function() {
			customizationsRepositories.forEach(function(custoRepo) {
				var repositoryName = custoRepo.dbName + custoRepo.collectionName;
				ZhUtil.registerDataSource(repositoryName, "_id");
				repositoryName = "/" + repositoryName;
				custoRepo.repository = RepositoryFactory.factory(repositoryName, "LOCAL");
			});
		};

		self.fillCustomizationsRepositories = function() {
			var promises = customizationsRepositories.map(function(custoRepo) {
				var request = CustomizationsRequestsBuilder.buildRequest("GET", "findAll", custoRepo, []);
				return restEngine.request(request).then(function(responseData) {
					return custoRepo.repository.save(responseData.dataset.foundData);
				});
			});
			return ZHPromise.all(promises);
		};

		self.getRepository = function(custoId) {
			var customizationRepository = getCustomizationRepositoryById(custoId);
			if (!Util.isEmptyOrBlank(customizationRepository) && customizationRepository.repository) {
				return customizationRepository.repository;
			}
		};

		self.removeCustomizations = function(customizationsToDelete) {
			var customizationsMappedById = groupIdsByCustomizationRepository(customizationsToDelete);
			var promises = customizationsMappedById.map(function(customization) {
				var customizationRepository = getCustomizationRepositoryById(customization.custoRepo);
				var query = Query.build()
					.where("_id").in(customization.ids);
				var request = CustomizationsRequestsBuilder.buildRequest("DELETE", "remove", customizationRepository, createCustomizationsIdArrayIndexedById(customization.ids));
				return restEngine.request(request).then(function() {
					return customizationRepository.repository.remove(query);
				});
			});
			return ZHPromise.all(promises)
				.then(function(promisesResult) {
					eventAggregator.publish("OnUpdateCustomizationsRepositories");
					return promisesResult;
				});
		};

		self.saveCustomization = function(rows, customizationType) {
			rows = Util.parseToArray(rows);
			var promises = rows.map(function(row) {
				if (validateRegister(row) && customizationType && ~customizationsTypes.indexOf(customizationType)) {
					row.customizationType = row.customizationType || customizationType;
					var customizationRepository = getCustomizationRepositoryById(row.customizationRepository);
					var request = CustomizationsRequestsBuilder.buildRequest("POST", "save", customizationRepository, [row]);

					return restEngine.request(request).then(function(responseData) {
						var savedPreferences = responseData.dataset.response.saved;

						if (savedPreferences.length === 1) {
							row._id = savedPreferences[0]._id;
							delete row.unsync_id;
						}

						return customizationRepository.repository.save([row]);
					});
				} else {
					return ZHPromise.when({});
				}
			});

			return ZHPromise.all(promises)
				.then(function(promisesResult) {
					eventAggregator.publish("OnUpdateCustomizationsRepositories");
					return promisesResult;
				});
		};

		self.getWidgetCustomizations = function(widget, customizationType) {
			var promises = customizationsRepositories.map(function(repository) {
				var query = Query.build()
					.where("containerId").equals(widget.container.id)
					.where("widgetId").equals(widget.id)
					.where("customizationType").equals(customizationType);
				var userId = preferencesUserService.getPreferenceUserRegistered();
				if (userId && repository.customizationsByUser) {
					query.where("userId").equals(userId);
				}
				return repository.repository.find(query);
			});
			return ZHPromise.all(promises).then(function(reponses) {
				return reponses.reduce(function(accumulatedCustomizations, response) {
					return accumulatedCustomizations.concat(response);
				}, []);
			});
		};

		self.transferCustomizationsRepository = function(widget, rows, destinyRepository) {
			ScreenService.showLoader();
			return self.removeCustomizations(rows).then(function() {
				var syncIds = {};
				promises = rows.map(function(row) {
					return transferViewCustomizations(row, destinyRepository, widget).then(function() {
						row.customizationRepository = destinyRepository;
						row.unsync_id = new Date().getTime();
						removeNotInterestingProperties(row);
						delete row._id;
						return self.saveCustomization(row, row.customizationType).then(function(syncId) {
							syncIds[row.customizationType] = syncId;
						});
					});
				});
				return ZHPromise.all(promises).then(function() {
					ScreenService.hideLoader();
					return syncIds;
				});
			});
		};

		var transferViewCustomizations = function(view, destinyRepository, widget) {
			if (view.customizationType == "view") {
				var viewCustomizationsMapped = [{
					"customizationType": "filter",
					"id": view.filterId
				}, {
					"customizationType": "layout",
					"id": view.preferenceId
				}];
				var viewCustomizations = [];
				var customizationRepository = view.customizationRepository;
				var promises = viewCustomizationsMapped.map(function(customization) {
					return self.getWidgetCustomizationsByRepository(widget, customization.customizationType, customizationRepository, customization.id)
						.then(function(customizationObject) {
							viewCustomizations.push(customizationObject.shift());
						});
				});
				return ZHPromise.all(promises).then(function() {
					return self.transferCustomizationsRepository(widget, viewCustomizations, destinyRepository).then(function(syncIdsMapped) {
						view.filterId = syncIdsMapped.filter;
						view.preferenceId = syncIdsMapped.layout;
					});
				});
			} else {
				return ZHPromise.when();
			}
		};

		self.getCustomizationView = function(widget, customizationRepository, customizationType, id) {
			var idPropertyName = customizationType == "filter" ? "filterId" : "preferenceId";
			return self.getWidgetCustomizationsByRepository(widget, "view", customizationRepository)
				.then(function(viewObjects) {
					var viewObject;
					if (Util.isArray(viewObjects) && viewObjects.length > 0) {
						viewObject = viewObjects.filter(function(view) {
							return view[idPropertyName] == id;
						});
					}
					return viewObject;
				});
		};

		var removeNotInterestingProperties = function(object) {
			Object.keys(object).forEach(function(key) {
				if (~key.indexOf('__')) {
					delete object[key];
				}
			});
		};

		self.getWidgetCustomizationsByRepository = function(widget, customizationType, repositoryType, customizationId) {
			var customizationRepository = getCustomizationRepositoryById(repositoryType);
			var query = Query.build()
				.where("containerId").equals(widget.container.id)
				.where("widgetId").equals(widget.id)
				.where("customizationType").equals(customizationType);
			var userId = preferencesUserService.getPreferenceUserRegistered();
			if (userId && customizationRepository.customizationsByUser) {
				query.where("userId").equals(userId);
			}
			if (customizationId) {
				query.where("_id").equals(customizationId);
			}
			return customizationRepository.repository.find(query);
		};

		self.getCustomizationsRepositoriesTypes = function() {
			return customizationsRepositories;
		};

		self.getUpdatedCustomizationsRepositoriesTypes = function() {
			return self.fillCustomizationsRepositories().then(function() {
				return customizationsRepositories;
			});
		};

		self.getLabelRepositoryType = function(repositoryTypeValue) {
			return customizationsRepositories.filter(function(repository) {
				return repository.value == repositoryTypeValue;
			}).shift().label;
		};

		self.applyLayout = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.layoutOnClick) {
				self.layoutOnClick(eventData);
			}
		};

		self.applyFilter = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.filterOnClick) {
				self.filterOnClick(eventData);
			}
		};

		self.applyVisualization = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.visualizationOnClick) {
				self.visualizationOnClick(eventData);
			}
		};

		function createEventData(customization, editingWidget) {
			return {
				"customization": customization,
				"editingWidget": editingWidget
			};
		}

		var groupIdsByCustomizationRepository = function(customizations) {
			var customizationsMappedById = [];
			customizationsRepositories.forEach(function(custoRepo) {
				var sameCustoRepoIds = customizations.filter(function(customization) {
					return customization.customizationRepository == custoRepo.value;
				});
				var map = {
					"custoRepo": custoRepo.value,
					"ids": sameCustoRepoIds.map(function(customization) {
						return customization._id;
					})
				};
				if (Util.isArray(map.ids) && map.ids.length > 0) {
					customizationsMappedById.push(map);
				}
			});
			return customizationsMappedById;
		};

		function createCustomizationsIdArrayIndexedById(customizationsId) {
			var customizationsToDelete = [];
			customizationsId.forEach(function(id) {
				customizationsToDelete.push({
					"_id": id
				});
			});
			return customizationsToDelete;
		}

		var getCustomizationRepositoryById = function(custoId) {
			return customizationsRepositories.filter(function(custoRepo) {
				return custoRepo.value == custoId;
			}).shift();
		};

		//-------------Begin Execution
		getCustomizationsRepositories().then(function() {
			createCustomizationsRepositories();
			self.fillCustomizationsRepositories().then(function() {
				eventAggregator.publish("OnUpdateCustomizationsRepositories");
			});
		});
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("zhCustomizationsService", CustomizationService);
	});

})();

// FILE: ./bower_components/zh-cache-bypass/src/CacheBypass.js
(function() {
    angular.module("ZeedhiApp").config(function($httpProvider) {
        if (Boolean(window.applicationVersion)) {
            $httpProvider.interceptors.push(function() {
                return {
                    'request': function cacheBypass(config) {
                        if (config && config.url && shouldApplyBypass(config.url)) {
                            if (config.url.indexOf('?') !== -1) {
                                config.url += '&';
                            } else {
                                config.url += '?';
                            }
                            config.url += 'v='+window.applicationVersion;
                        }
                        return config;
                    }
                };
            });
        } else {
            console.warn('Missing "applicationVersion" global property for CacheBypass.');
        }
    });

    function shouldApplyBypass(url) {
        const urlWithoutParams = url.split('?')[0];
        return (
            Util.stringEndsWith(urlWithoutParams, '.json') ||
            Util.stringEndsWith(urlWithoutParams, '.html') ||
            Util.stringEndsWith(urlWithoutParams, '.svg')
        );
    }
})();


// FILE: ./bower_components/red-ui/dist/rui.js
// FILE: src/assets/js/controllers/ruiController.js
function ruiController($rootScope, ScreenService, eventAggregator, templateManager) {

	eventAggregator.subscribe("beforeProjectStartup", function() {
		var interval = setInterval(function() {
			if (window.__back__) {
				window.__back__ = function() {
					if (ScreenService.isAlertShown()) {
						return;
					} else if ($rootScope.showAbout) {
						ScreenService.hideAbout();
					} else if (ScreenService.isPopupOpen()) {
						ScreenService.closePopup();
					} else if (ScreenService.isSideMenuOpen()) {
						ScreenService.toggleSideMenu();
					} else if ($rootScope.headerDropdownOpened) {
						toggleHeaderDropdown();
						$rootScope.$apply();
					} else if ($rootScope.searching) {
						toggleHeaderSearch();
						$rootScope.$apply();
					} else {
						var widget = getActiveWidget();
						if (widget.mode === 'edit' && widget.onCancel) {
							widget.onCancel();
						} else if (widget.widgetOnBack) {
							widget.widgetOnBack();
						} else if (widget.container && widget.container.onBack) {
							widget.container.onBack();
						} else {
							ScreenService.goBack();
						}
					}
				};
				clearInterval(interval);
			}
		}, 300);
		htmlFixesToRUI();
	});

	function getActiveWidget() {
		return templateManager.container.getActiveWidget();
	}
	function htmlFixesToRUI (){
		$('html').removeClass('splash');
		$('html').addClass('hide-scroll');
	}
}

Configuration(function(ContextRegister) {
	ContextRegister.register("ruiController", ruiController);
});

(function() {
	setInterval (function(){
		$('.container[zh-scroller]').scroll(function(){
			$('html').removeClass('hide-scroll');
			setTimeout(function(){
				$('html').addClass('hide-scroll');
			},1500);
		});
	},800);
})();



// FILE: ./bower_components/red-ui-header/dist/rui-header.js
// FILE: src/assets/js/controllers/ruiHeaderController.js
/**
 * The controller of header template
 * @param {TemplateManager} templateManager - An instance of TemplateManager
 */
Configuration(function(eventEngine) {
	eventEngine.registerEvent("WidgetOnBack", "widgetOnBack");
});

function ruiHeaderController($scope, templateManager, ScreenService) {

	var self = this;

	var showAsActionTypes = {
		always: "always",
		list: "list",
		view: "view",
		edit: "edit",
		checked_rows: "checked_rows",
		never: "never",
		never_list: "never_list",
		never_view: "never_view",
		never_edit: "never_edit",
		never_checked_rows: "never_checked_rows",
		back: "back",
		back_list: "back_list",
		back_view: "back_view",
		back_edit: "back_edit",
		back_checked_rows: "back_checked_rows"
	};
	var searchStatus = {
		opened: "SO",
		closed: "SC",
		notSearching: "NS"
	};


	/**
	 * Retrieves actions conllection length
	 * @param  {String} type - the action type
	 * @return {Integer} Returns the length of actions collection by type
	 */
	this.getActionsLengthByType = function (type) {
		return getActionsByType(type).length;
	};

	/**
	 * Retrieves actions by type
	 * @param  {String} type - the action type
	 * @return {Array} Array of actions by type
	 */
	this.getActionsByType = getActionsByType;
	function getActionsByType(type) {
		return getActions().filter(function (action) {
			return findShowAsAction(type, action) && action.isVisible;
		});
	}

	function findShowAsAction (types, action) {
		var exist = false;
		types = types.split('|');
		var asAction = action.showAsAction.split('|');
		for  (var i = asAction.length - 1; i >= 0 && exist === false; i--) {
			exist = showAsActionTypes[types[0]] == asAction[i] ? true : showAsActionTypes[types[1]] == asAction[i] ? true : false;
		}
		return exist;
	}

	this.getActionsVisibleLength = function(nameArrayActions) {
		return $scope[nameArrayActions].filter(function(action) {
			return action.isVisible;
		}).length;
	};

	/**
	 * Toggle dropdown visibility with fast animation
	 */
	this.toggleHeaderDropdown = function () {
		$scope.$root.headerDropdownOpened = !$scope.$root.headerDropdownOpened;
	};
	window.toggleHeaderDropdown = this.toggleHeaderDropdown;

	/**
	 * Retrieves the container's active widget
	 * @return {Widget} Active widget
	 */
	this.getActiveWidget = getActiveWidget;
	function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

	/**
	 * Retrieves active widget's current row of current container
	 * @return {Object} Current row
	 */
	this.getCurrentRow = function () {
		return getActiveWidget().currentRow;
	};

	/**
	 * Execute selected action and toggle dropdown
	 * @param  {Action} action - selected action
	 * @param  {Function} actionClick - Zeedhi action click event
	 */
	this.selectDropdownOption = function (action, actionClick) {
		actionClick(action, getActiveWidget());
	};

	/**
	 * Attach actions and methods in container
	 */
	this.updateContainer = updateContainer;
	function updateContainer() {
		var container = getContainer();
		updateCurrentWidget(container);
		container.eachWidget(function(widget){
			widget.getHeaderActions = getHeaderActions;
		});
	}


	function updateCurrentWidget (container) {
		var widget = getActiveWidget();
		widget.headerActions = widget.headerActions || [];
		widget.headerActions = factoryActions(container, widget.headerActions);
		widget.searchCriteria =  widget.searchCriteria || {};
		updateSearching(widget);
		updateHeaderShadow(widget);
		updateHeaderForTWWI(widget, searchStatus.notSearching);
	}
	function updateHeaderForTWWI(widget,search){
		var fatherTemp = widget.getParent().template;
		var srcTemp = 'rui-tabs#widget/tabbedWidgetWithImage.html';
		var ruiHeader = $('.rui-header');
		var ruiContainer = $('.container[zh-fixed]');
		if(search ===  searchStatus.notSearching){
			if(fatherTemp === srcTemp){
				ruiHeader.addClass('header-over-tabs');
				ruiContainer.css('top',0);
			}else{
				ruiHeader.removeClass('header-over-tabs');
				ruiHeader.removeClass('new-bg-header');
				ruiContainer.css('top',ruiHeader.css('height'));
			}
		}else if (search ===  searchStatus.opened && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').hide();
			$('.rui-header').removeClass('header-over-tabs');
			$('.rui-header').removeClass('new-bg-header');
			ruiContainer.css('top',ruiHeader.css('height'));
		}
		else if (search ===  searchStatus.closed && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').show();
			$('.rui-header').addClass('header-over-tabs');
			$('.rui-header').addClass('new-bg-header');
			ruiContainer.css('top',0);
		}
	}
	function updateHeaderShadow(widget) {
		if (widget.parent && isTabbedTemplate(widget.parent.template)) {
			$('.rui-header').removeClass('searchShadow');
		} else {
			$('.rui-header').addClass('searchShadow');
		}
	}

	function isTabbedTemplate(template) {
		return template.search('tabbedWidget.html') !== -1 || template.search('tabbedWidgetWithImage.html') !== -1;
	}

	function updateSearching(widget) {
		if (widget.showSearch && widget.headerSearchBy) {
			$scope.searchVal = widget.searchCriteria._ALL;
		}

		if (widget.keepHeaderSearchingMode && searchFilled()) {
			$scope.$root.searching = true;
		} else if (searchFilled() && widget.mode == "list") {
			$scope.$root.searching = true;
		} else {
			$scope.$root.searching = false;
		}
	}
	this.getNeverMode = getNeverMode;
	function getNeverMode (){
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "never_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "never_edit|never";
			} else if(widget.mode === "view") {
				return "never_view|never";
			} else if(widget.mode === "list") {
				return "never_list|never";
			}
		}
	}
	this.getBackMode = getBackMode;
	function getBackMode (){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "back_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "back_edit|back";
			} else if(widget.mode === "view") {
				return "back_view|back";
			} else if(widget.mode === "list") {
				return "back_list|back";
			}
		}
	}
	this.getCenterMode = getCenterMode;
	function getCenterMode(){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "edit|always";
			} else if(widget.mode === "view") {
				return "view|always";
			} else if(widget.mode === "list") {
				return "list|always";
			}
		}
	}

	/**
	 * Build actions applying mobile and default properties
	 * @param  {Container} container - current container
	 * @param  {Array} actions - array of actions to build
	 * @return {Array} An array of builded actions
	 */
	function factoryActions(container, actions) {
		return actions.map(function (action) {
			action = Util.extend(new Action(), action);
			_extendMobileProperties(action);
			action.container = container;
			action.widget = getActiveWidget();
			return action;
		});
	}

	/**
	 * Set mobile properties if it's a mobile device
	 * @param  {Object} obj - action object
	 */
	function _extendMobileProperties(obj) {
		if (Util.isMobile() && obj.mobile) {
			var mobile = obj.mobile;
			Object.keys(mobile).forEach(function extendMobileProperty(propertyToExtend) {
				if (obj.replaceByMobile || !obj[propertyToExtend] || typeof obj[propertyToExtend] !== 'object') {
					obj[propertyToExtend] = mobile[propertyToExtend];
				} else {
					obj[propertyToExtend] = Util.extend(obj[propertyToExtend], mobile[propertyToExtend]);
				}
			});
		}
	}

	/**
	 * Retrieves header actions of current container
	 * @return {Array} An array of actions
	 */
	function getActions() {
		return getActiveWidget().headerActions || [];
	}

	/**
	 * Retrieves current container
	 * @return {Container} Current container
	 */
	function getContainer() {
		return templateManager.container;
	}

	function widgetOnChangeMode(args) {
		if (getContainer()) {
			var widget = getActiveWidget();
			if (widget.mode != "list") {
				$scope.$root.searching = false;
			}
			updateContainer(getContainer());
		}
	}

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().mode : null;
	}, widgetOnChangeMode);

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().hasCheckedRows() : null;
	}, function(args) {
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget() : null;
	}, function(args){
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer();
	}, function(args){
		if (getContainer()){
			updateContainer();
		}
	});
	 
	/**
	 * Change header layout to search mode
	 * @param  {Boolean} type - true: Search mode, false: Normal mode
	 */
	this.toggleHeaderSearch = function(type) {
		var widget = getActiveWidget();
		if(type) {
			$scope.$root.searching = true;
			$('.rui-tabsMobileCont').css('top','-50px');
			$('.rui-TWWI-mainContainer').css('top','-50px');
			setTimeout(function() {
				$('#search-input').attr("placeholder", ScreenService.i18n("Search..."));
				$('#search-input').focus();
				updateHeaderForTWWI(widget, searchStatus.opened);
			}, 150);
		} else {
			if (searchFilled()) {
				clearSearch();
			}
			$('.rui-tabsMobileCont').css('top','0');
			$('.rui-TWWI-mainContainer').css('top','0');
			$('#search-input').attr("placeholder", "");
			$scope.$root.searching = false;
			updateHeaderForTWWI(widget, searchStatus.closed);
		}
	};
	window.toggleHeaderSearch = this.toggleHeaderSearch;

	/**
	 * Update the searchCriteria of active widget
	 */
	this.searchInputChange = searchInputChange;
	function searchInputChange() {
		var inputVal = $scope.searchVal;
		var widget = getActiveWidget();
		var showSearch = widget.showSearch;
		if (showSearch && !widget.headerSearchBy) {
			console.error("ERROR: headerSearchBy property not defined on widget '" + widget.name + "'.");
		} else {
			widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
			if (!Util.isEmptyOrBlank(inputVal)) {
				widget.searchCriteria._ALL = inputVal;
			} else {
				delete widget.searchCriteria._ALL;
			}

			if ($scope.searchTimeout) {
				clearTimeout($scope.searchTimeout);
			}
			$scope.searchTimeout = setTimeout(setSearchFilter.bind(this), 500);
		}
	}

	/**
	 * Set dataSourceFilter of widget according to searchCriteria
	 */
	function setSearchFilter() {
		var widget = getActiveWidget();

		if (!Object.keys(widget.searchCriteria).length) {
			widget.dataSourceFilter = widget.defaultDataSourceFilter || widget.dataSourceFilter || [];
		} else {
			if (!widget.defaultDataSourceFilter) {
				widget.defaultDataSourceFilter = widget.dataSourceFilter || [];
			}
			var name = widget.headerSearchBy.join("|");
			var operator = "LIKE_ALL";
			var value = '%'+widget.searchCriteria._ALL+'%';
			addFilterCondition(widget, name, operator, value);
		}
		widget.reload();
	}

	/**
	 * Add condition to widget's dataSourceFilter
	 * @param {Widget} widget
	 * @param {String} name
	 * @param {String} operator
	 * @param {String} value
	 */
	function addFilterCondition(widget, name, operator, value) {
		var dataSourceFilter = widget.dataSourceFilter || [];
		dataSourceFilter = dataSourceFilter.filter(function(row) {
			return row.name != name;
		});
		// if (!Util.isUndefinedOrNull(inDataSourceFilter[0])) {
		// 	dataSourceFilter.splice(inDataSourceFilter[0], 1);
		// }
		dataSourceFilter.push({
			"name": name,
			"operator": operator,
			"value": value
		});
		widget.dataSourceFilter = dataSourceFilter;
	}

	/**
	 * Manage visibility of clear search icon
	 * @return {Boolean} Status of widget's searchCriteria property
	 */
	this.searchFilled = searchFilled;
	function searchFilled() {
		var widget = getActiveWidget();
		if (widget.searchCriteria && widget.headerSearchBy) {
			return !Util.isEmptyOrBlank(widget.searchCriteria._ALL);
		} else {
			return false;
		}
	}

	/**
	 * Clear value of search input
	 */
	this.clearSearch = clearSearch;
	function clearSearch() {
		$scope.searchVal = "";
		searchInputChange();
	}

	function getHeaderActions(name) {
		if (this.headerActions) {
			return this.headerActions.filter(function (item) {
				return item.name === name;
			}).shift();
		}
	}

	this.getTitle = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();
		var headerTitle = widget.title || container.label;

		if (widget.showCheckedRowsCounterOnHeader && widget.getCheckedRows().length) {
			headerTitle = widget.getCheckedRows().length;
		}

		return headerTitle;
	};

	this.menuActionVisibility = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();

		if (!container.showMenu || widget.getCheckedRows().length) {
			return false;
		} else {
			if (widget.showMenu === undefined) {
				return true;
			} else {
				return widget.showMenu;
			}
		}
	};
}


// FILE: ./bower_components/red-ui-fields/dist/rui-fields.js
// FILE: src/assets/js/controllers/ruiDateController.js
function ruiDateController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    var nomeDosMeses = [
        "janeiro",
        "fevereiro",
        "março",
        "abril",
        "maio",
        "junho",
        "julho",
        "agosto",
        "setembro",
        "outubro",
        "novembro",
        "dezembro"
    ];

    var diasDaSemanaAbreviado = ["dom", "seg", "ter", "qua", "qui", "sex", "sab"];

    var container = $element.get()[0];
    var dateInput = container.querySelector('#date-input');
    var calendarMode = 0x1;
    var yearMode = 0x2;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            dateInput.classList.add('rui-has-value');
            var dateArray = fieldValue.split('/'); //[0] == day, [1] == month, [2] == year
            dateContainer.date.day = parseInt(dateArray[0], 10);
            dateContainer.selectedDay = parseInt(dateArray[0], 10);
            dateContainer.date.month = parseInt(dateArray[1], 10) - 1;
            dateContainer.selectedMonth = parseInt(dateArray[1], 10) - 1;
            dateContainer.date.year = parseInt(dateArray[2], 10);
            dateContainer.selectedYear = parseInt(dateArray[2], 10);
            dateContainer.dateWasSelected = true;
            dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
            dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
            var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
            dateContainer.monthDisplay.textContent = monthYearString;
            dateInput.children[1].textContent = fieldValue;
            generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        }
        var label = ScreenService.i18n($scope.field.label);
        if ($scope.field.validations && $scope.field.validations.required) {
            label += " *";
        }
        dateInput.children[0].textContent = label;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            dateInput.classList.add('disabled');
        }
    };


    var dateContainer = container.querySelector('#calendar-container');
    var overlay = container.querySelector('#overlay');
    var modal = container.querySelector('#modalWindow');
    dateContainer.okButton = container.querySelector('#ok');
    dateContainer.cancelButton = container.querySelector('#cancel');
    dateContainer.table = container.querySelector('#calendar-table');
    dateContainer.days = container.querySelector('#days-container');
    dateContainer.monthDisplay = container.querySelector('#month-display');
    dateContainer.previousMonthButton = container.querySelector('#previous-month-button');
    dateContainer.nextMonthButton = container.querySelector('#next-month-button');
    dateContainer.topYearDisplay = container.querySelector('#top-year-display');
    dateContainer.topFullDateDisplay = container.querySelector('#top-full-date-display');
    dateContainer.yearPicker = container.querySelector('#year-picker-container');
    dateContainer.yearPickerOutterWrapper = container.querySelector('#year-picker-outter-wrapper');
    dateContainer.currentSelectedYearSpanInYearPicker = undefined;

    dateContainer.dateWasSelected = false;

    dateContainer.mode = calendarMode;

    dateContainer.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (dateContainer.dateWasSelected) {
            dateInput.classList.add('rui-has-value');
        }
        dateInput.classList.remove('active');
        dateContainer.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        dateInput.classList.add('active');
        dateInput.classList.remove('rui-has-value');
        toggleCalendarMode();
        overlay.classList.add('active');
        modal.classList.add('active');
        generateCalendar(dateContainer.date.month, dateContainer.date.year);
    };

    $scope.field.confirm = function(e) {
        dateContainer.dateWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            ((dateContainer.selectedDay) < 10 ? "0" + (dateContainer.selectedDay) : "" + (dateContainer.selectedDay)) +
            "/" + ((dateContainer.selectedMonth + 1) < 10 ? "0" + (dateContainer.selectedMonth + 1) : "" + (dateContainer.selectedMonth + 1)) +
            "/" + dateContainer.selectedYear;
        dateInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        dateInput.classList.add('rui-has-value');
        dateInput.classList.remove('active');
        dateContainer.validate();
    };


    overlay.onclick = $scope.field.cancel;
    dateInput.onclick = $scope.field.open;
    dateContainer.okButton.onclick = $scope.field.confirm;
    dateContainer.cancelButton.onclick = $scope.field.cancel;


    var dayClickHandler = function(eventInfo) {
        var previousActiveDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        previousActiveDay.classList.remove('active');
        dateContainer.selectedDay = Number.parseInt(eventInfo.target.textContent, 10);
        var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        activeDay.classList.add('active');
        dateContainer.date.day = dateContainer.selectedDay;
        dateContainer.date.month = dateContainer.selectedMonth;
        dateContainer.date.year = dateContainer.selectedYear;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    };

    function generateCalendar( /*number */ month, /*number */ year) /*number: phantomOffset*/ {
        //debugger;
        //dateContainer.table;
        //dateContainer.days.opacity = 0;
        while (dateContainer.days.firstChild) {
            dateContainer.days.removeChild(dateContainer.days.firstChild);
        }

        var phantomDay = document.createElement('span');
        phantomDay.classList.add('phantom-day');
        phantomDay.textContent = '00';

        var firstDayOfWeek = new Date(year, month, 1).getDay();
        dateContainer.phantomDaysOffset = firstDayOfWeek - 1;

        for (var i = 0; i < firstDayOfWeek; i++) {
            dateContainer.days.appendChild(phantomDay.cloneNode());
        }

        var lastDayOfMonth = new Date(year, month + 1, 0).getDate();

        for (var j = 1; j <= lastDayOfMonth; j++) {
            var day = document.createElement('span');
            day.textContent = "" + j; //((i < 10) ? "0" + i : "" + i);
            day.onclick = dayClickHandler;
            dateContainer.days.appendChild(day);
        }

        if (dateContainer.date.month === month &&
            dateContainer.date.year === year) {
            var phantomDaysOffset = new Date(year, month, 1).getDay() - 1;
            var activeDay = dateContainer.days.children[phantomDaysOffset + dateContainer.date.day];
            activeDay.classList.add('active');
        }

    }

    /*monthsToAdd = 1 vai para o proximo mes, e -1 para o anterior*/
    /* retorna o objeto da nova data*/
    var addMonths = function( /*number*/ monthsToAdd) {
        var newDate = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        newDate.setMonth(dateContainer.selectedMonth + monthsToAdd);
        var newString = buildMonthWithYearString(newDate.getMonth(), newDate.getFullYear());
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.selectedYear = newDate.getFullYear();
        dateContainer.selectedMonth = newDate.getMonth();
        dateContainer.selectedDay = newDate.getDate();
        generateCalendar(newDate.getMonth(), newDate.getFullYear());
        return newDate;
    };

    var buildMonthWithYearString = function( /*number */ month, /*number */ year) {
        return nomeDosMeses[month] + " de " + year;
    };

    var buildTopDisplayString = function() /* string */ {
        //return nomeDosMeses[month] + " de " + year;
        var date = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        var nomeDiaDaSemana = diasDaSemanaAbreviado[date.getDay()];
        var nomeDoMesAbreviado = nomeDosMeses[dateContainer.selectedMonth].substr(0, 3);
        return nomeDiaDaSemana + ", " + dateContainer.selectedDay + " de " + nomeDoMesAbreviado;
    };

    var setDateUsingJSDateObject = function( /* Date */ data) {
        dateContainer.date.year = data.getFullYear();
        dateContainer.date.month = data.getMonth();
        dateContainer.date.day = data.getDate();
    };

    dateContainer.previousMonthButton.onclick = function() {
        var newDate = addMonths(-1);
    };

    dateContainer.nextMonthButton.onclick = function() {
        var newDate = addMonths(1);
    };

    var toggleYearMode = function() {
        dateContainer.classList.remove('active');
        dateContainer.yearPickerOutterWrapper.classList.add('active');
        dateContainer.topYearDisplay.classList.add('active');
        dateContainer.topFullDateDisplay.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker.scrollIntoView();
    };

    var toggleCalendarMode = function() {
        dateContainer.classList.add('active');
        dateContainer.yearPickerOutterWrapper.classList.remove('active');
        dateContainer.topYearDisplay.classList.remove('active');
        dateContainer.topFullDateDisplay.classList.add('active');
    };

    var pickYear = function( /* clickEvent */ e) {
        var newYear = e.target.textContent;
        dateContainer.selectedYear = newYear;
        var newString = buildMonthWithYearString(dateContainer.selectedMonth, newYear);
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker = e.target;
        e.target.classList.add('active');
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.topYearDisplay.textContent = newYear;
        dateContainer.date.year = newYear;
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        toggleCalendarMode();
    };

    /*initial setup*/

    var today = new Date();
    var hoje = today.getDate();
    dateContainer.selectedDay = hoje;
    dateContainer.selectedMonth = today.getMonth();
    dateContainer.selectedYear = today.getFullYear();
    dateContainer.date = {
        year: today.getFullYear(),
        day: hoje,
        month: today.getMonth()
    };
    dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
    dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    dateContainer.monthDisplay.textContent = buildMonthWithYearString(today.getMonth(), today.getFullYear());
    generateCalendar(today.getMonth(), today.getFullYear());
    var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
    activeDay.classList.add('active');

    dateContainer.topYearDisplay.onclick = toggleYearMode;
    dateContainer.topFullDateDisplay.onclick = toggleCalendarMode;

    for (var i = 1970; i < 2051; i++) {
        var n = document.createElement("span");
        n.textContent = i;
        dateContainer.yearPicker.appendChild(n);
        n.onclick = pickYear;
    }

    dateContainer.yearPicker.currentYearNode = dateContainer.yearPicker.children[today.getFullYear() - 1970];
    dateContainer.yearPicker.currentYearNode.classList.add('active');
    dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;

    /******* ORIENTATION ********/

    if (window.orientation !== undefined) {
        window.addEventListener("orientationchange", function() {
            detectOrientationAndRotate();
        });
    }

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
    };

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        //
        dateInput.classList.remove('rui-has-value');
        var today = new Date();
        dateContainer.date.day = today.getDate();
        dateContainer.selectedDay = dateContainer.date.day;
        dateContainer.date.month = today.getMonth();
        dateContainer.selectedMonth = dateContainer.date.month;
        dateContainer.date.year = today.getFullYear();
        dateContainer.selectedYear = dateContainer.date.year;
        dateContainer.dateWasSelected = false;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
        var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
        dateContainer.monthDisplay.textContent = monthYearString;
        dateInput.children[1].textContent = '';
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        $scope.fieldRow = '';
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.yearPicker.currentYearNode.classList.add('active');
        dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;
        dateContainer.validate();
    };
}


// FILE: src/assets/js/controllers/ruiDateNativeController.js
function ruiDateNativeController($scope, $element, ZhUtil, ScreenService) {
	var defaultFormat = "YYYY-MM-DD";
	var field = $scope.field;
	var input = $element.get(0).querySelector(".rui-txt-input-text");
	var range = {
		maxDate: field.maxDate,
		minDate: field.minDate
	};

	field.applyDefaultValue = function() {
		if (field.widget.currentRow) {
			field.widget.currentRow[field.name] = typeof field.defaultValue === "undefined" ? "" : ZhUtil.evaluate(field.defaultValue);
			field.currentDate = getValueWithDefaultFormat(field.widget.currentRow[field.name]);
		}
	};

	if(field.widget.currentRow && field.defaultValue) {
		field.applyDefaultValue();
	}

	$scope.getLabel = function(label, validations) {
		label = ScreenService.i18n(label);
		if (validations && Object.keys(validations).length && validations.required) {
			label += ' *';
		}
		return label;
	};
	$scope.formatValue = function() {
		if (field.currentDate) {
			field.widget.currentRow[field.name] = moment(field.currentDate, defaultFormat).format(field.dateFormat);
		} else {
			field.widget.currentRow[field.name] = field.currentDate;
		}
	};

	updateRange(field.minDate, field.maxDate);

	if (Object.getOwnPropertyDescriptor(field, 'maxDate') && !Object.getOwnPropertyDescriptor(field, 'maxDate').get) {
		Object.defineProperty(field, 'maxDate', {
	        set: function(maxDate) {
	            range.maxDate = maxDate;
	            updateRange(null, maxDate);
	        },
	        get: function(newValue) {
	            return range.maxDate;
	        }
	    });
	}

	if (Object.getOwnPropertyDescriptor(field, 'minDate') && !Object.getOwnPropertyDescriptor(field, 'minDate').get) {
		Object.defineProperty(field, 'minDate', {
	        set: function(minDate) {
	            range.minDate = minDate;
	            updateRange(minDate, null);
	        },
	        get: function(newValue) {
	            return range.minDate;
	        }
	    });
	}

	function updateRange(minDate, maxDate) {
		if (maxDate) {
			var max = moment(ZhUtil.evaluate(maxDate), field.dateFormat).format(defaultFormat);
			input.setAttribute("max", max);
		}
		if (minDate) {
			var min = moment(ZhUtil.evaluate(minDate), field.dateFormat).format(defaultFormat);
			input.setAttribute("min", min);
		}
	}

	function getValueWithDefaultFormat(value) {
		return moment(value, field.dateFormat).format(defaultFormat);
	}
}

// FILE: src/assets/js/controllers/ruiFABController.js
function ruiFABController($rootScope, ApplicationContext,eventAggregator, templateManager) {


    function getNewFAB() {
        var FABRoot = document.createElement('span');
        var FABIcon = document.createElement('span');
        var FABAction = {};

        FABIcon.className = "material-icons fab-icon";
        FABIcon.textContent = "pregnant_woman";

        FABRoot.classList.add('fab-root');
        FABRoot.appendChild(FABIcon);

        FABRoot.setIcon = function( /*string*/ iconName) {
            if (typeof iconName !== 'string') {
                console.log('%cfunction setIcon expects parameter 1 ' +
                    'to be of type string. Received ' + typeof iconName + ' instead',
                    'font-size:20px;background-color:red;color:yellow;');
                return "YOUR CALL WAS REJECTED LOSER";
            }
            FABIcon.textContent = iconName;
        };

        FABRoot.setAction = function( /*object*/ action) {
            if (typeof action !== 'object') {
                console.log('%cfunction setIcon expects parameter 1 ' +
                    'to be of type Object. Received ' + typeof action + ' instead',
                    'font-size:20px;background-color:red;color:yellow;');
                return "YOUR CALL WAS REJECTED LOSER";
            }
            FABAction = action;
        };


        FABRoot.id = "rui-FAB";
        FABRoot.addEventListener('click', function() {
            var eventContext = getApplicationContextWithEventArgs(FABAction);
            $rootScope.$eval(FABRoot.code, eventContext);
            $rootScope.$apply();
        });

        return FABRoot;
    }

    var FABElement = getNewFAB();
    var defaultIconName = "sentimentverysatisfied";

    var getApplicationContextWithEventArgs = function(eventOwner) {
        ApplicationContext.args = {
            owner: eventOwner
        };
        return ApplicationContext;
    };


    var modes = [
        "FAB_edit",
        "FAB_view",
        "FAB_list",
        "FAB_checked_rows",
        "FAB"
    ];

    // find the first action with the FAB mode
    var findActionToDisplay = function(widget) {
        var currentWidgetMode = widget.mode;
        var actions = widget.actions;
        if (widget.hasCheckedRows()) {
            for (var j = actions.length; j--;) {
                if (actions[j].showAsAction === 'FAB_checked_rows') {
                    return actions[j];
                }
            }
        } else {
            for (var i = actions.length; i--;) {
                if (actions[i].showAsAction === 'FAB'||
                    actions[i].showAsAction.indexOf('FAB_'+currentWidgetMode) !== -1) {
                    return actions[i];
                }
            }
        }
    };

    //add a watch to check if the active widget has changed.
    //TODO: find a better solution that DOESN'T use ANGULARJS

    var changeFABVisibility = function() {
        if(!templateManager.container){
            return;
        }
        var widget = templateManager.container.getActiveWidget();
        var action = findActionToDisplay(widget);
        // /*if the*/ TODO: add comment here
        FABElement.className = 'fab-root';
        if (action) {
            FABElement.code = action.code;
            FABElement.setIcon(action.ruiIcon || defaultIconName);
            FABElement.setAction(action);
            FABElement.zhWidgetMode = action.showAsAction;
            FABElement.className += ' active';
            if(action.class){
                FABElement.className += " " + action.class;
            }
        }
    };

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget().hasCheckedRows();
        // else , will return undefined
    }, function(hasCheckedRows) {
        changeFABVisibility();
    });

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget();
        // else , will return undefined
    }, function(hasChanged) {
        changeFABVisibility();
    });

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget().mode;
        // else , will return undefined
    }, function(hasChanged) {
        changeFABVisibility();
    });

    var attachFABToZhContainer = function() {
        var containerToInjectFAB = document.querySelector('div.container');
        if(containerToInjectFAB){
            containerToInjectFAB.appendChild(FABElement);
            return true;
        } else {
            return false;
        }
    };

    // attach the button to the container, using the attempt and error strategy
    var setIntervalReference = setInterval(function(){
        if(attachFABToZhContainer()){
            clearInterval(setIntervalReference);
        }
    },50);


}

Configuration(function(ContextRegister) {
    ContextRegister.register("ruiFABController", ruiFABController);
});


// FILE: src/assets/js/controllers/ruiSelectController.js
function ruiSelect($scope,$element,$timeout) {
	var realInput = $scope.field;
	var FAKE_INPUT_NAME = "#rui_fake_" + realInput.name;
	
	function switchDataDown(fakefieldname,open){
		var selectContainer  = $(fakefieldname).parent().parent();
		if(open){
			var newplaceholder = selectContainer.find('label').text();
			selectContainer.addClass('rui-fixed-at-top');
		}else{
			selectContainer.removeClass('rui-fixed-at-top');
		}
	}
	
	$scope.initAutoCompSelect = function (){
		$timeout(function (){
			var fieldName = realInput.name;
			$('#'+fieldName).on('click',function(){
				switchDataDown(FAKE_INPUT_NAME,true);
				$(FAKE_INPUT_NAME).focus();
			});
		},300);
	};
	
	$scope.closeDataDown = function(){
		switchDataDown(FAKE_INPUT_NAME,false);
		$(FAKE_INPUT_NAME).blur();
	};
	
	$scope.filterComparising = function(row) {
		var filterOptions = realInput.filterOptions;
		var isVisible = false;
		if ($scope.filterQuery) {
			for (var i = 0; i < filterOptions.length; i++) {
				var rowValue = row[filterOptions[i]].toLowerCase();
				rowValue = rowValue.latinize();
				var filterValue = $scope.filterQuery.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					isVisible = true;
					break;
				}
			}
		} else {
			isVisible = true;
		}
		return isVisible;
	};
	
	$scope.chooseAutoComplete = function(row) {
		var outData = realInput.outData;
		if (outData && Object.keys(outData).length) {
			if (outData && angular.isObject(outData)) {
				for(var attr in outData) {
					realInput.widget.currentRow[outData[attr]] = row[attr];
				}
			}
		} else {
			var descriptionField = realInput.descriptionField || realInput.name;
			realInput.widget.currentRow[realInput.name] = row[descriptionField];
			if (realInput.valueField) {
				realInput.widget.currentRow[realInput.valueField] = row[realInput.valueField];
			}
		}
		$scope.closeDataDown();
	};
	
	$scope.cleanField = function() {
		$scope.filterQuery = "";
		$element.find(FAKE_INPUT_NAME).val($scope.filterQuery);
		$(FAKE_INPUT_NAME).focus();
	};
	
	$scope.cleanSelection = function() {
		realInput.widget.currentRow[realInput.name] = '';
	};
}

// FILE: src/assets/js/controllers/ruiTextController.js
function ruiTextController($scope, ScreenService) {
	var defaultFormat = {
		time: "HH:mm"
	};
	var types = {
		time: "time"
	};

	$scope.initField = function() {
		console.log($('#' + $scope.field.name));
	};

	$scope.getFieldVal = function(fieldName) {
		return $("#" + fieldName).val();
	};
	$scope.getLabel = function(label, validations) {
		label = ScreenService.i18n(label);
		if (validations && Object.keys(validations).length && validations.required) {
			label += ' *';
		}
		return label;
	};
	$scope.formatValue = function(filedRow, field) {
		var type = field.type;
		if (type === types.time && field.timeFormat) {
			field.widget.currentRow[field.name] = moment(filedRow, defaultFormat.date).format(field.timeFormat);
		}
	};
}

// FILE: src/assets/js/controllers/ruiTimeController.js
function ruiTimeController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            var tempoArray = fieldValue.split(':'); //[0] ==  minuto [1] == hora
            clock.time.hora = parseInt(tempoArray[0]);
            clock.time.minuto = parseInt(tempoArray[1]);
            clock.hourView.children[12].style.opacity = '0';
            clock.hourView.children[clock.time.hora].style.opacity = '1';
            clock.minuteView.children[0].style.opacity = '0';
            clock.minuteView.children[clock.time.minuto].style.opacity = '1';
            timeInput.classList.add('rui-has-value');
            clock.hourWasSelected = true;
            toggleHourMode();
        }
        timeInput.children[1].textContent = fieldValue;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            timeInput.classList.add('disabled');
        }
    };

    var container = $element.get()[0];
    var clock = container.querySelector('#clock-container');
    var overlay = container.querySelector('#overlay');
    var pointerContainer = container.querySelector('#pointer-container'); /*clock.children*/
    var touchArea = container.querySelector('#touch-area');
    var timeInput = container.querySelector('#time-input');

    /* define a label */
    var label = ScreenService.i18n($scope.field.label);
    if ($scope.field.validations && $scope.field.validations.required) {
        label += " *";
    }
    timeInput.children[0].textContent = label;


    clock.minuteDisplay = container.querySelector('#red-minute');
    clock.hourInnerDisplay = container.querySelector('#red-hours-inner');
    clock.hourOutterDisplay = container.querySelector('#red-hours-outter');
    clock.pointer = pointerContainer;
    clock.HOUR_MODE = 1;
    clock.MINUTE_MODE = 2;
    clock.mode = clock.HOUR_MODE;
    clock.time = {
        hora: 12,
        minuto: 0
    };
    $scope.field.time = clock.time;
    clock.detalhesDoCirculoMinuto = {};
    clock.detalhesDoCirculoHora = {};
    clock.activeMinute = 0; /* Posicao do vetor que indica o minuto selecionado no relogio  */
    clock.activeHour = 0; /* Posicao do vetor que indica a hora selecionado no relogio  */
    clock.activeHourArrayPosition = 2; /*contém a posicao (no array de filhos de relogio, que corresponde ...)*/
    clock.hourWasSelected = false;
    clock.hourView = container.querySelector('#hour-display-container');
    clock.minuteView = container.querySelector('#minute-display-container');
    clock.okButton = container.querySelector('#ok');
    clock.cancelButton = container.querySelector('#cancel');

    clock.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    var clickHander = function(e) {
        var rotationDetails = '';
        if (clock.mode == clock.MINUTE_MODE) {
            var previousMinute = clock.time.minuto;
            var minuto = getMinute(e.clientX, e.clientY, clock.detalhesDoCirculoMinuto);
            if (minuto === 60) {
                minuto = 0;
            }
            clock.time.minuto = minuto;
            rotationDetails = "rotate3d(0,0,1," + (minuto * 6) + "deg)";
            requestAnimationFrame(function() {

                pointerContainer.style.transform = rotationDetails;
                clock.minuteView.children[minuto].style.opacity = '1';
                if (previousMinute !== minuto) {
                    clock.minuteView.children[previousMinute].style.opacity = '0';
                    clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
                    if (Number.isInteger(minuto / 5)) {
                        clock.minuteDisplay.children[minuto / 5].children[0].style.color = 'white';
                        clock.activeMinute = minuto / 5;
                    }
                }
            });
        }
        else /* clock.mode = clock.HOUR_MODE*/ {
            /* se o clique for dentro do circulo interno ...*/
            var previousActiveHourArrayPosition = clock.activeHourArrayPosition;
            var hora = getHour(e.clientX, e.clientY, clock.detalhesDoCirculoHora);
            var rawHora = ((hora === 12) ? 0 : hora);
            rotationDetails = "rotate3d(0,0,1," + (hora * 30) + "deg)";
            var previousHour = clock.time.hora;
            /* se o clique for dentro do circulo interno ...*/
            if (isWithinCircumnference(clock.detalhesDoCirculoHora, {
                    x: e.clientX,
                    y: e.clientY
                })) {
                switchToInnerPointer();
                hora = ((hora) ? hora : 12);
                clock.time.hora = hora;
                clock.activeHourArrayPosition = 2;
            }
            else {
                switchToOutterPointer();
                hora = ((hora === 12 || hora === 0) ? 0 : hora + 12);
                clock.time.hora = hora;
                clock.activeHourArrayPosition = 1;
            }
            requestAnimationFrame(function() {
                pointerContainer.style.transform = rotationDetails;
                clock.hourView.children[hora].style.opacity = '1';
                if (previousHour !== hora) {
                    clock.hourView.children[previousHour].style.opacity = '0';
                    clock.children[previousActiveHourArrayPosition].
                    children[clock.activeHour].children[0].style.color = 'black';
                    //if(Number.isInteger(rawHora/5)){
                    clock.children[clock.activeHourArrayPosition].
                    children[rawHora].children[0].style.color = 'white';
                    clock.activeHour = rawHora;
                    //}
                }
            });
        }
    };




    /* se for uma tela de toque, usa o evento padrão para telas de toque*/
    if (('ontouchmove' in window) ||
        (window.navigator.MaxTouchPoints > 0) ||
        (window.navigator.msMaxTouchPoints > 0)) {

        var touchHandler = function(e) {
            //console.log('%cTOCADO','background-color: green; color: yellow');
            //e.preventDefault();
            var touch = {};
            var rotationDetails = '';
            if (clock.mode == clock.MINUTE_MODE) {
                touch = e.targetTouches[0];
                var previousMinute = clock.time.minuto;
                var minuto = getMinute(touch.pageX, touch.pageY, clock.detalhesDoCirculoMinuto);
                if (minuto === 60) {
                    minuto = 0;
                }
                clock.time.minuto = minuto;
                rotationDetails = "rotate3d(0,0,1," + (minuto * 6) + "deg)";
                requestAnimationFrame(function() {
                    pointerContainer.style.transform = rotationDetails;
                    clock.minuteView.children[minuto].style.opacity = '1';
                    if (previousMinute !== minuto) {
                        clock.minuteView.children[previousMinute].style.opacity = '0';
                        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
                        if (Number.isInteger(minuto / 5)) {
                            clock.minuteDisplay.children[minuto / 5].children[0].style.color = 'white';
                            clock.activeMinute = minuto / 5;
                        }
                    }
                });
            }
            else /* clock.mode = clock.HOUR_MODE*/ {
                var previousActiveHourArrayPosition = clock.activeHourArrayPosition;
                touch = e.targetTouches[0];
                var hora = getHour(touch.pageX, touch.pageY, clock.detalhesDoCirculoHora);
                var rawHora = ((hora === 12) ? 0 : hora);
                rotationDetails = "rotate3d(0,0,1," + (hora * 30) + "deg)";
                var previousHour = clock.time.hora;
                /* se o clique for dentro do circulo interno ...*/
                if (isWithinCircumnference(clock.detalhesDoCirculoHora, {
                        x: touch.pageX,
                        y: touch.pageY
                    })) {
                    switchToInnerPointer();
                    hora = ((hora) ? hora : 12);
                    clock.time.hora = hora;
                    clock.activeHourArrayPosition = 2;
                }
                else {
                    switchToOutterPointer();
                    hora = ((hora === 12 || hora === 0) ? 0 : hora + 12);
                    clock.time.hora = hora;
                    clock.activeHourArrayPosition = 1;
                }
                requestAnimationFrame(function() {
                    pointerContainer.style.transform = rotationDetails;
                    clock.hourView.children[hora].style.opacity = '1';
                    if (previousHour !== hora) {
                        clock.hourView.children[previousHour].style.opacity = '0';
                        clock.children[previousActiveHourArrayPosition].
                        children[clock.activeHour].children[0].style.color = 'black';
                        //if(Number.isInteger(rawHora/5)){
                        clock.children[clock.activeHourArrayPosition].
                        children[rawHora].children[0].style.color = 'white';
                        clock.activeHour = rawHora;
                        //}
                    }
                });
            }
        };
        touchArea.ontouchmove = touchHandler;

        touchArea.ontouchend = function(e) {
            touchArea.ontouchmove = null;
            setTimeout(function() {
                touchArea.ontouchmove = touchHandler;
            }, 200);
            toggleMinuteMode();
            return true;
        };

        touchArea.ontouchstart = function(e) {
            //e.preventDefault();
            touchHandler(e);
        };
    }
    else {
        touchArea.onmousedown = function(e) {
            e.preventDefault();
            clickHander(e);
            document.addEventListener("mousemove", clickHander);
        };
        var removeListenerAndSwitchToMinutes = function(e) {
            document.removeEventListener("mousemove", clickHander);
            toggleMinuteMode();
        };
        document.addEventListener('mouseup', removeListenerAndSwitchToMinutes);
    }

    var getMinute = function( /*number*/ x, /*number*/ y,
        /*object {x: pontoX da origem, y: pontoY da origem} */
        detalhesCirculo) {
        var angleRadian = Math.atan2(y - detalhesCirculo.y, x - detalhesCirculo.x);
        var angle = angleRadian * (180 / Math.PI) + 90;
        if (angle < 0) {
            angle += 360;
        }
        return Math.round(angle / 6);
    };


    var getHour = function( /*number*/ x, /*number*/ y,
        /*object {x: pontoX da origem, y: pontoY da origem} */
        detalhesCirculo) {
        var angleRadian = Math.atan2(y - detalhesCirculo.centerPoint.y, x - detalhesCirculo.centerPoint.x);
        var angle = angleRadian * (180 / Math.PI) + 90;
        if (angle < 0) {
            angle += 360;
        }
        return Math.round(angle / 30);
    };


    // document.querySelector('html').appendChild(overlay);
    var modal = container.querySelector('#modalWindow');

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (clock.hourWasSelected) {
            timeInput.classList.add('rui-has-value');
        }
        timeInput.classList.remove('active');
        toggleHourMode();
        clock.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        timeInput.classList.add('active');
        timeInput.classList.remove('rui-has-value');
        overlay.classList.add('active');
        modal.classList.add('active');
        recalculateGeometry();
    };

    $scope.field.confirm = function(e) {
        clock.hourWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            (clock.time.hora < 10 ? "0" + clock.time.hora : clock.time.hora) + ":" +
            (clock.time.minuto < 10 ? "0" + clock.time.minuto : clock.time.minuto);
        timeInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        timeInput.classList.remove('active');
        timeInput.classList.add('rui-has-value');
        //timeInput.classList.remove('active');
        toggleHourMode();
        clock.validate();
    };

    overlay.onclick = $scope.field.cancel;
    timeInput.onclick = $scope.field.open;
    clock.okButton.onclick = $scope.field.confirm;
    clock.cancelButton.onclick = $scope.field.cancel;

    window.addEventListener("orientationchange", function() {
        detectOrientationAndRotate();
    }, false);

    /**
     *  C L O C K  M E T H O D S
     */


    var recalculateGeometry = function() {

        var rect = clock.getBoundingClientRect();
        var raioRelogio = (clock.clientHeight / 2);
        clock.detalhesDoCirculoMinuto = {
            x: rect.left + raioRelogio,
            y: rect.top + raioRelogio
        };
        clock.detalhesDoCirculoHora = {
            centerPoint: {
                x: rect.left + 40 + 90,
                y: rect.top + 40 + 90
            },
            radius: 90
        };
    };

    var toggleMinuteMode = function() {
        if (clock.mode === clock.MINUTE_MODE)
            return;

        clock.minuteDisplay.classList.remove('rui-opacity-hidden');
        clock.hourInnerDisplay.classList.add('rui-opacity-hidden');
        clock.hourOutterDisplay.classList.add('rui-opacity-hidden');
        clock.pointer.classList.remove('rui-inner-pointer');
        clock.mode = clock.MINUTE_MODE;
        clock.minuteView.classList.add('active');
        clock.hourView.classList.remove('active');


        var rotationDetails = "rotate3d(0,0,1," + (clock.time.minuto * 6) + "deg)";
        requestAnimationFrame(function() {
            pointerContainer.style.transform = rotationDetails;
        });
    };

    var toggleHourMode = function() {
        if (clock.mode === clock.HOUR_MODE)
            return;


        // esconde os minutos
        clock.minuteDisplay.classList.add('rui-opacity-hidden');
        // mostra as horas
        clock.hourInnerDisplay.classList.remove('rui-opacity-hidden');
        clock.hourOutterDisplay.classList.remove('rui-opacity-hidden');
        clock.pointer.classList.add('rui-inner-pointer');
        clock.minuteView.classList.remove('active');
        clock.hourView.classList.add('active');
        clock.mode = clock.HOUR_MODE;

        var rotationDetails = "rotate3d(0,0,1," + (clock.time.hora * 30) + "deg)";
        requestAnimationFrame(function() {
            pointerContainer.style.transform = rotationDetails;
            if (clock.time.hora <= 12 && clock.time.hora > 0) {
                clock.pointer.classList.add('rui-inner-pointer');

            }
            else {
                clock.pointer.classList.remove('rui-inner-pointer');
            }
        });
    };

    var switchToInnerPointer = function() {
        clock.pointer.classList.add('rui-inner-pointer');
    };

    var switchToOutterPointer = function() {
        clock.pointer.classList.remove('rui-inner-pointer');
    };

    var distanceBetween = function(pointA, pointB) {
        return (Math.sqrt(Math.pow(pointA.x - pointB.x, 2) + Math.pow(pointA.y - pointB.y, 2)));
    };


    var isWithinCircumnference = function(circleProperties, pointToCheck) {
        return (distanceBetween(circleProperties.centerPoint, pointToCheck) <= circleProperties.radius);
    };

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
        setInterval(recalculateGeometry, 800);
    };

    clock.hourView.onclick = toggleHourMode;
    clock.minuteView.onclick = toggleMinuteMode;


    for (var i = 1; i < 60; i++) {
        var n = clock.minuteView.children[0].cloneNode();
        if (i < 10) {
            i = "0" + i;
        }
        n.textContent = i;
        n.style.opacity = '0';
        clock.minuteView.appendChild(n);
    }

    /* GERA OS MOSTRADORES DE HORA */
    for (var j = 1; j < 24; j++) {
        var node = clock.hourView.children[0].cloneNode();
        if (j < 10) {
            j = "0" + j;
        }
        node.textContent = j;
        node.style.opacity = '0';
        clock.hourView.appendChild(node);
    }
    /* DEFINE A HORA PADRAO COMO 12h*/
    clock.hourView.children[0].style.opacity = '0';
    clock.hourView.children[12].style.opacity = '1';
    /* DEFINE A HORA COMO 'ATIVA' (PARA MOSTRAR A FONTE EM BRANCO)*/
    clock.hourView.classList.add('active');
    /**/

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        timeInput.classList.remove('rui-has-value');

        clock.children[clock.activeHourArrayPosition].children[clock.activeHour].children[0].style.color = 'black';

        clock.activeHourArrayPosition = 2;
        clock.activeHour = 11;
        clock.children[clock.activeHourArrayPosition].children[0].children[0].style.color = 'white';

        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
        clock.activeMinute = 0;
        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'white';


        clock.hourView.children[clock.time.hora].style.opacity = '0';
        clock.time.hora = 12;
        clock.hourView.children[12].style.opacity = '1';

        clock.minuteView.children[clock.time.minuto].style.opacity = '0';
        clock.time.minuto = 0;
        clock.minuteView.children[0].style.opacity = '1';

        clock.hourWasSelected = false;
        timeInput.children[1].textContent = '';
        $scope.fieldRow = '';
        /*força re-renderização*/
        clock.mode = clock.MINUTE_MODE;
        toggleHourMode();
        /*******/

        clock.validate();

    };
}



// FILE: ./bower_components/red-ui-sidenav/dist/rui-sidenav.js
// FILE: src/assets/js/controllers/SideNavController.js
function SideNavController($element, $scope, ScreenService, $rootScope, eventAggregator) {


    var transformPropertyName;// = ;
    if(typeof document.body.style.webkitTransform === 'string'){
        transformPropertyName = 'webkitTransform';
    } else if(typeof document.body.style.transform === 'string'){
        transformPropertyName = 'transform';
    } else {
        console.error('TRANSFORM IS NOT SUPPORTED BY THIS BROWSER. \n The sidenav is not supported as well');
    }

    eventAggregator.subscribe('onContainerOpen', function() {
        setupCurrentSelectedOption();
    });

    var setupCurrentSelectedOption = function() {
        /*if this event was still not handled */
        if (!menu.newSelectedOptionWasReStyledProperly) {
            /*if any option was selected before, disable it*/
            if (menu.currentSelectedOption) {
                menu.currentSelectedOption.classList.remove('selected');
                menu.currentSelectedOption.children[0].classList.remove('selected');
                menu.currentSelectedOption.children[1].classList.remove('selected');
            }
            /* find the new option to reStyle searching by the option.name*/
            for (var i = itemListContainer.children.length; i--;) {
                if (itemListContainer.children[i].menuName === $rootScope.templateManager.container.name) {

                    menu.currentSelectedOption = itemListContainer.children[i];
                    menu.currentSelectedOption.classList.add('selected');
                    menu.currentSelectedOption.children[0].classList.add('selected');
                    menu.currentSelectedOption.children[1].classList.add('selected');
                    return;
                }
            }
        }
        /* always specify that the next handler call should reStyle the menu */
        menu.newSelectedOptionWasReStyledProperly = false;
    };

    var htmlContainer = $element.get()[0];
    var self = this;
    menu = htmlContainer.querySelector('.rui-sidenav-menu-container');
    overlay = htmlContainer.querySelector('.rui-sidenav-overlay');
    itemListContainer = htmlContainer.querySelector('.rui-sidenav-itens-list');

    menuContainer = document.querySelector('.something-else');

    /* gain controll over the default toggleMenuButton*/
    var hijackZhSideMenu = function() {
        var toggleMenuButton = document.querySelector("*[zh-click='toggleSideMenu()']");
        if (toggleMenuButton) {
            toggleMenuButton.removeAttribute('zh-click');
            toggleMenuButton.onclick = openRuiSideMenu;
            return true;
        }
        else
            return false;
    };


    $scope.clearListItens = () => {
        while(itemListContainer.lastChild){
            itemListContainer.removeChild(itemListContainer.lastChild);
        }
    };

    $scope.init = function() {
        var rootMenu = $scope.templateManager.project.groupMenu;
        $scope.clearListItens();
        for (var i = 0; i < rootMenu.length; i++)
            renderMenuItens($scope.templateManager.project.groupMenu[i]);
    };
    
    eventAggregator.subscribe('onChangeLanguage', $scope.init);

    var addClickListener = function( /*menuItem : object*/ item, /*HTMLElement*/ htmlElement) {
        if (Util.isFunction(item.click)) {
            htmlElement.addEventListener('click', function(e) {
                menu.preventInteraction();
                $scope.$eval(item.click);
            });
        }
        else if (item.windowName && !item.disabled) {
            htmlElement.addEventListener('click', function(e) {
                menu.preventInteraction();
                ScreenService.openWindow(item.windowName);
                selectMenuOption(e.currentTarget);
                menu.closeMenu(false);
            });
        }
    };

    var selectMenuOption = function( /*HTMLElement*/ newMenuOption) {
        menu.currentSelectedOption.classList.remove('selected');
        menu.currentSelectedOption.children[0].classList.remove('selected');
        menu.currentSelectedOption.children[1].classList.remove('selected');

        newMenuOption.classList.add('selected');
        newMenuOption.children[0].classList.add('selected');
        newMenuOption.children[1].classList.add('selected');

        menu.currentSelectedOption = newMenuOption;
        menu.newSelectedOptionWasReStyledProperly = true;
    };

    menu.previousX = null;
    menu.previousTime = null;

    var openingTouchHandler = function(e) {
        if (window.isInScrollMode) {
            return;
        }
        var absoluteDeltaY = Math.abs(e.targetTouches[0].clientY - menu.previousY);
        if (absoluteDeltaY > 100 &&
            menu.currentTranslation < -135 
        ) {
            window.isInScrollMode = true;
            menu.classList.remove('rui-sidenav-transition-mode');
            menu.doTranslate(-280);
            return;
        }
        if (this.firstX < this.openingLimit && $scope.templateManager.container.showMenu) {
            var newTranslationValue = e.touches[0].clientX -
                applicationContentSurface.firstX -
                280;
            menu.isOpening = true;
            if (newTranslationValue < 0) {
                menu.classList.remove('rui-sidenav-transition-mode');
                menu.doTranslate(newTranslationValue);
            }
            else {
                $rootScope.menuOpen = true;
            }
        }
    };

    var openingTouchStartHandler = function(e) {
        applicationContentSurface.firstX = e.touches[0].clientX;
        menu.previousY = e.targetTouches[0].clientY;
        applicationContentSurface.firstTime = e.timeStamp;
        window.isInScrollMode = false; // bugFix #M49
    };

    var openingTouchEndHandler = function(e) {
        if (menu.isOpening) {
            applicationContentSurface.firstX = 0;
            applicationContentSurface.lastX = 0;
            applicationContentSurface.acumulatedX = 0;
            if (menu.currentTranslation > -135 ||
                (menu.currentTranslation - applicationContentSurface.firstX > -250 &&
                    e.timeStamp - applicationContentSurface.firstTime <= 300)
            ) {
                $rootScope.menuOpen = true;
            }
            else {
                $rootScope.menuOpen = false;
            }
            menu.isOpening = false;
            window.isInScrollMode = false;
        }
    };

    var initAppContentSurface = function() {
        applicationContentSurface = document.querySelector('div.container');
        if (applicationContentSurface === null) {
            return;
        }
        else {

            applicationContentSurface.addEventListener('touchend', openingTouchEndHandler);
            applicationContentSurface.addEventListener('touchmove', openingTouchHandler, false);
            applicationContentSurface.addEventListener('touchstart', openingTouchStartHandler);

            applicationContentSurface.firstX = 51;
            applicationContentSurface.lastX = 0;
            applicationContentSurface.acumulatedX = 0;
            menu.previousY = 0;
            applicationContentSurface.openingLimit = 30;
            applicationContentSurface.lastMenuOpeningTime = 0;
            clearInterval($scope.setIntervalID);
        }
    };

    $scope.initAppContentSurface = function() {
        $scope.setIntervalIterationCount = 0;
        $scope.setIntervalIterationLimit = 50;
        $scope.setIntervalID = setInterval(function() {
            if ($scope.setIntervalIterationCount > $scope.setIntervalIterationLimit)
                clearInterval($scope.setIntervalID);
            $scope.setIntervalIterationCount++;
            initAppContentSurface.call(self);
        }, 100);
    };

    menu.translatePlus = function(value) {
        menu.currentTranslation = Math.round(value + menu.currentTranslation);
        menu.style[transformPropertyName] = 'translate3d(' + menu.currentTranslation + 'px,0,0)';

    };

    menu.doTranslate = function(value) {
        menu.currentTranslation = Math.round(value);
        requestAnimationFrame(function() {
            menu.style[transformPropertyName] = 'translate3d(' + value + 'px,0,0)';
        });
    };

    var touchHandler = function(event) {
        if (window.isInScrollMode) {
            return;
        }
        var absoluteDeltaY = Math.abs(event.targetTouches[0].clientY - menu.previousY);
        if (absoluteDeltaY > 10 &&
            event.targetTouches[0].clientX <= 280 &&
            menu.currentTranslation > -3) {
            window.isInScrollMode = true;
            return;
        }
        var deltaX = event.targetTouches[0].clientX - menu.previousX;

        if (deltaX !== 0) {
            var translation = deltaX * menu.constanteDeAceleracao;
            var newTrans = translation + menu.currentTranslation;

            if (newTrans > 0) {
                menu.classList.add('rui-sidenav-transition-mode');
                menu.doTranslate(0);
                menu.previousTime = event.timeStamp;
                return;
            }

            if (newTrans < -265) {
                menu.classList.add('rui-sidenav-transition-mode');
                menu.doTranslate(-280);
                return;
            }
            menu.translatePlus(translation);
            menu.previousX = event.targetTouches[0].clientX;
        }
        menu.previousTime = event.timeStamp;

    };

    menuContainer.addEventListener('touchstart', function(event) {
        menu.previousTime = event.timeStamp;
        menu.previousX = event.targetTouches[0].clientX;
        menu.previousY = event.targetTouches[0].clientY;
    }, {
        passive: true
    });

    /* conclui o fechamento */
    menuContainer.addEventListener('touchend', function(event) {
        if (menu.currentTranslation <= -132) {
            $rootScope.menuOpen = false;
        }
        else {
           $rootScope.menuOpen = true;
        }
        window.isInScrollMode = false;
    }, {
        passive: true
    });

    menuContainer.addEventListener('touchmove', touchHandler, {
        passive: true
    });
    menu.classList.remove('nonbounce');
    menu.currentTranslation = 0;

    menu.currentTranslation = -280;
    menu.constanteDeAceleracao = 70;

    menu.openMenu = function() {
        menu.classList.add('rui-sidenav-transition-mode');
        requestAnimationFrame(function() {
            menu.style[transformPropertyName] = 'translate3d(' + 0 + 'px,0,0)';
        });
        overlay.classList.add('active');
        $rootScope.menuOpen = true;
    };

    menu.preventInteraction = function() {
        applicationContentSurface.style.pointerEvents = 'none';
        overlay.style.pointerEvents = 'none';
        setTimeout(function() {
            overlay.style.pointerEvents = 'auto';
            applicationContentSurface.style.pointerEvents = 'auto';
        }, 500);
    };

    menu.closeMenu = function( /*boolean*/ dontPreventInteraction) {
        /* prevent interactions with the content surface and then schedules it's reactivation*/
        if (!dontPreventInteraction) {
            menu.preventInteraction();
        }

        /*do the animations*/
        menu.classList.add('rui-sidenav-transition-mode');
        requestAnimationFrame(function() {
            menu.style[transformPropertyName] = 'translate3d(' + -280 + 'px,0,0)';
        });
        overlay.classList.remove('active');
        $rootScope.menuOpen = false;
    };

    /*
    * @deprecated This function is not necessary anymore, 
    * since the setter method of $rootScope.menuOpen will handle the menu animations by itself.  
    */
    menu.toggle = function() {
        if ($rootScope.menuOpen)
            menu.closeMenu();
        else
            menu.openMenu();
    };

    window.isInScrollMode = false;

    window.toggleRuiSideMenu = menu.toggle;
    window.openRuiSideMenu = menu.openMenu;
    $rootScope.menuOpen = false;
    overlay.addEventListener('click', menu.closeMenu);


    var renderMenuItens = function( /*array: object*/ menu) {

        if (menu.title) {
            var title = document.createElement('span');
            title.className = 'rui-sidenav-title';
            title.textContent = ScreenService.i18n(menu.title);
            itemListContainer.appendChild(title);
        }
        var menuItens = menu.menus;
        for (var i = 0; i < menuItens.length; i++) {
           

            /* create contianer */
            var newListItemContainer = document.createElement('span');
            newListItemContainer.className = 'rui-sidenav-item-container';
            addClickListener(menuItens[i], newListItemContainer);

            /* create label */
            var newListItemText = document.createElement('span');
            newListItemText.className = 'rui-sidenav-item-text';
            newListItemText.textContent = ScreenService.i18n(menuItens[i].label);

            /* create icon */
            var newListItemIcon = document.createElement('i');
            newListItemIcon.className = 'material-icons rui-sidenav-item-icon';
            newListItemIcon.textContent = menuItens[i].icon;

            newListItemContainer.appendChild(newListItemIcon);
            newListItemContainer.appendChild(newListItemText);
            newListItemContainer.menuName = menuItens[i].name;
            if (!menuItens[i].aclIsVisible || !menuItens[i].isVisible)
                newListItemContainer.style.display = 'none';
            itemListContainer.appendChild(newListItemContainer);
            
        }
        var divider = document.createElement('span');
        divider.className = 'rui-sidenav-divider';
        itemListContainer.appendChild(divider);
    };

    /*__menuOpen is a internal property used to control the state of the real property menuOpen */
    $rootScope.__menuOpen = false;
    
    Object.defineProperty($rootScope, 'menuOpen', {

        set: function(newValue) {
            this.__menuOpen = newValue;
            if (newValue) {
                menu.classList.add('rui-sidenav-transition-mode');
                menu.currentTranslation = 0;
                requestAnimationFrame(function() {
                    menu.style[transformPropertyName] = 'translate3d(' + 0 + 'px,0,0)';
                });
                overlay.classList.add('active');
            }
            else {
                menu.preventInteraction();
                menu.classList.add('rui-sidenav-transition-mode');
                menu.currentTranslation = -280;
                requestAnimationFrame(function() {
                    menu.style[transformPropertyName] = 'translate3d(' + -280 + 'px,0,0)';
                });
                overlay.classList.remove('active');
            }
        },
        get: function(newValue) {
            return this.__menuOpen;
        }
    });

}


function initiliazeNewMenuFunction(templateManager, eventAggregator) {
    eventAggregator.subscribe("beforeProjectStartup", function() {
        templateManager.project.setMenuItemVisibility =
            function( /*string*/ menuItemName, /* boolean */ visibility) {
                this.getMenu(menuItemName).isVisible = visibility; /* this === project*/

                /* if menu was already rendered, set the css display property */
                if (typeof itemListContainer !== "undefined") {
                    for (var i = itemListContainer.children.length; i--;) {
                        if (itemListContainer.children[i].menuName === menuItemName) {
                            menuItemToChangeVisibility = itemListContainer.children[i];
                            var newVisibility = (visibility ? 'table' : 'none');
                            menuItemToChangeVisibility.style.display = newVisibility;
                            return;
                        }
                    }
                }
            };
    });
}


Configuration(function(ContextRegister) {
    ContextRegister.register("initiliazeNewMenuFunction", initiliazeNewMenuFunction);
});



// FILE: ./tecdiet_templates/js/FormLoginController.js
function FormLoginController($scope, $timeout) {
    var classes = {
        login: ".lf-login"
    };

    this.startButton = true;

    this.init = function() {
        setBackground();
    };

    function setBackground() {
        if ($scope.widget.backgroundImage) {
            $(classes.login).css("background-image", "url(" + $scope.widget.backgroundImage + ")");
        }
    }
}


// FILE: ./tecdiet_templates/js/HistoryAvaliationsController.js
function HistoryAvaliationsController(ApplicationContext, templateManager, ScreenService, eventAggregator, localStorageEngine) {
    
    this.openPopup = function(historyAvaliations){
        var appointment = getLocalVar('__appointmentdata');
        var filterPatient = [{
            name: "id",
            operator: "=",
            value: historyAvaliations.container.getWidget('WidgetDetailsPatient').curPatient.id
        },{
            name: "appointmentDate",
            operator: "=",
            value: appointment.normalizedDate
        }];
        localStorageEngine.getDataSource('/AnthAvaliationResults', filterPatient).then(function(anthResults) {
            historyAvaliations.results = anthResults;
        }).then(function(){
            var methodAndResult = [];

            var getResultsByMethod = function(m){
                return methodAndResult.findIndex(function(metodo){
                    return metodo.idMetodo === m.idMetodo;
                });
            };
            
            historyAvaliations.results.forEach(function(result){
                var index = getResultsByMethod(result);
                if(index !== -1){
                    methodAndResult[index].resultados.push(result);
                }else{
                    var metodoToResult = {
                        'idMetodo': result.idMetodo,
                        'nomeMetodo': result.nomeMetodo,
                        'dataAval': result.anthDate,
                        'resultados': [result]
                    };
                    methodAndResult.push(metodoToResult);
                }
            });
            historyAvaliations.methodAndResult = methodAndResult;
            historyAvaliations.appointmentDate = appointment.normalizedDate;
            historyAvaliations.methods = historyAvaliations.container.getWidget('WidgetDetailsPatient').antrLoadedFields;
            historyAvaliations.isVisible = true;
        });
        
    };
    
    this.closePopup = function(historyAvaliations){
        historyAvaliations.isVisible = false;
    };
    this.clique = function (id) {
        var itemList = $(".resultados_"+id);
        var arrow = $("#rotate_"+id);
        if(itemList.hasClass('panel-history')){
            itemList.removeClass('panel-history');
            itemList.addClass('antro-result-row');
        }else{
            itemList.addClass('panel-history');
            itemList.removeClass('antro-result-row');
        }
        arrow.hasClass('rotate') ? arrow.removeClass('rotate') : arrow.addClass('rotate');
    };
}

// FILE: ./tecdiet_templates/js/MNListController.js
function MNListController ($scope, $timeout, ScreenService, eventAggregator, templateManager, localStorageEngine, restEngine, ZHPromise, ApplicationContext) {
    var container = $("#mn-list-container");
    var widget = $scope.widget;
    var keys = ['/Profiles', '/Units', '/Patients', '/Halls', '/Prescriptions', '/NormalFeedings', '/BioEvaluations', '/BedsPrescriptions', '/NutritionalStatus', '/Diets', '/DietsServices', '/Storeys', '/Beds', '/Appointments', '/HealthInsurance', '/MedicalTeams', '/Examinations', '/AnthEvaluationMethods', '/AnthEvaluationGeneral', '/DataTypeEvaluations', '/DataEvaluations', '/ControlFeedings', '/Services', '/LactaryPlates', '/Plates', '/LactaryFeedings', '/EnteralFeedings', '/HallsDistincts', '/PlatesByUnit', '/PlatesPla', '/FoodIntoleranceGroup', '/AllGroupSubgroup', '/PatientIntoRestriction', '/EnteralPlates', '/EnteralProducts', '/OralServices', '/LactaryServices', '/EnteralServices'];
    var intoleranciaCrud = ApplicationContext.CRUD.init("/FoodIntoleranceGroup");
    var controlFeedingCrud = ApplicationContext.CRUD.init("/ControlFeedings");
    var lactaryFeedingCrud = ApplicationContext.CRUD.init("/LactaryFeedings");
    var enteralFeedingCrud = ApplicationContext.CRUD.init("/EnteralFeedings");

    this.sendPatientUpdate = function(patient, situation) {
        var widget = $scope.widget;        
        var messages = ['Deseja confirmar <b>alta</b> deste paciente selecionado?', 'Deseja confirmar <b>óbito</b> do paciente selecionado?'];
        ScreenService.confirmMessage(messages[situation], 'question', function() {
            eventAggregator.publish("sendPatientUpdate", {'patient': [patient], 'situation': situation,'widget': widget});
        });
    };
    
    eventAggregator.subscribe('openPatientDetails', function(patient){
        openPatientDetails(patient);
    });
    this.openPatientDetails = function(patient){
        openPatientDetails(patient);
    };
    function openPatientDetails (patient) {
        $scope.widget.container.getWidget('WidgetMain').curPatient = patient;
        var widgetPatient = widget.container.getWidget('WidgetDetailsPatient');
        widgetPatient.activate();
        widgetPatient.isVisible = true;
        widgetPatient.curPatient = patient;
        widgetPatient.currentScreen = 'personalData';
        widgetPatient.showBioqExamList = false;
        loadStorage(widgetPatient).then(function (dataSources) {
            widgetPatient.localDataSourceLoaded = dataSources;
            sizeWindow(); 
            addData(patient);
            setLocalVar('__patient', patient.id);
            setLocalVar('__CDFILIAL', patient.unit);
            
            var intoleranciaFilter = [{
                'name': 'cdpaciente',
                'operator': '=',
                'value': widgetPatient.curPatient.id
            }];
            intoleranciaCrud.find(intoleranciaFilter, false, "/FoodIntoleranceGroup").then(function(dataFoodGroup){
                widgetPatient.dataFoodGroup = angular.copy(dataFoodGroup);
            });

        });
    };
    
    function sizeWindow() {
        var pacientDetailsControlers = parseInt($('.pacientData ').css('height')) + parseInt($('.viewControlsContainer ').css('height')) - 30;
        $('.pacientDetailsContainer').css('height', 'calc(100% - ' + pacientDetailsControlers + 'px)');
    }
    
    function addData(patient) {
        var datasources = widget.container.getWidget('WidgetDetailsPatient').localDataSourceLoaded;
        var searchForId = patient.id;
        var dataToBeAnexed = [];
        keys.forEach(function (keyDirty) {
            var keyName = keyDirty.replace('/', '');

            if (keyName !== 'Patients' &&
                keyName !== 'NutritionalStatus' &&
                keyName !== 'AnthEvaluationGeneral' &&
                keyName !== 'NormalFeedingsNow' &&
                datasources[keyName] !== null) {

                datasources[keyName].forEach(function (feedInfo) {
                    if (feedInfo.id == searchForId) {
                        dataToBeAnexed.push(feedInfo);
                    }
                });

                patient[keyName] = dataToBeAnexed;
                dataToBeAnexed = [];
            }
        });
        addLocationDataToPrescription(patient);
    }
    
    function addLocationDataToPrescription(patient) {
        if (patient.Prescriptions) {
            patient.Prescriptions.forEach(function (pres, key) {
                var curBedID = pres.bed;
                widget.container.getWidget('WidgetDetailsPatient').localDataSourceLoaded.Beds.forEach(function (beditem) {
                    if (beditem.bed === curBedID) {
                        patient.Prescriptions[key].hall = beditem.hall;
                        patient.Prescriptions[key].storey = beditem.storey;
                    }
                });
            });
        }
    }
    
    function loadStorage(widget) {
        return ZHPromise.all(keys.map(function (key) {
            if(key === '/ControlFeedings'){
                return controlFeedingCrud.find(null,false,'/ControlFeedings').then(function(dataSource){
                    return dataSource;
                });
            }else if(key === '/LactaryFeedings'){
                return lactaryFeedingCrud.find(null,false,'/LactaryFeedings').then(function(dataSource){
                    return dataSource;
                });
            } else if(key === '/EnteralFeedings'){
                return enteralFeedingCrud.find(null, false, '/EnteralFeedings').then(function(dataSource){
                    return dataSource;
                });
            }else{
                return localforage.getItem(key).then(function (value) {
                    return JSON.parse(value);
                }.bind(this));
            }
        }.bind(this))).then(function (loadedDataSources) {
            var dataSources = {};
            keys.forEach(function (key, index) {
                var keyName = key.replace('/', '');
                if ((keyName == 'Appointments' || keyName == 'Prescriptions') && loadedDataSources[index]) {
                    loadedDataSources[index].filter(function (attr) {
                        attr.normalizedDate = attr.date;
                        attr.date = moment(attr.date, 'DD/MM/YYYY HH:mm:ss').format('YYYY-MM-DDTHH:mm:ss');
                    });
                }
                dataSources[keyName] = loadedDataSources[index] ? loadedDataSources[index] : []; 
            });
            return dataSources;
        });
    }
    this.addPatient = function(widget){
        var newPatientPopup = widget.container.getWidget('newPatientPopup');
        newPatientPopup.newRow();
        var ISSEQUENTIAL = getLocalVar("__ISSEQUENTIAL");
        if(!ISSEQUENTIAL){
            ScreenService.showMessage(ScreenService.i18n('Não existe parametrização definida nesta unidade para gerar código do paciente!'));
        }else{
            if(ISSEQUENTIAL === 'S'){
                newPatientPopup.getField("id").readOnly = true;
                restEngine.post('/Patients/patientSequential', {
                    "requestType": "Row",
                    "row": {
                        'unit': localStorageEngine.getLocalVar('__CDFILIAL'),
                        'isAjusta': getLocalVar("__ISAJUSTA")
                    }
                }).then(function(callback) {
                    newPatientPopup.currentRow.id = callback.dataset.patientSequential.CDPACIENTE;
                });
            
            }else{
                newPatientPopup.getField("id").readOnly = false;
            }
            ScreenService.openPopup(newPatientPopup);
            newPatientPopup.newRow();
            newPatientPopup.edit();
        }
    };
    
    this.addHospitalization = function(widget, patient){
        
        var newHospitalizationPopup = widget.container.getWidget('newHospitalizationPopup');
        newHospitalizationPopup.newRow();
        newHospitalizationPopup.edit();
        newHospitalizationPopup.getField('bed').dataSourceFilter =  [{
                "name": [{
                    "name": "bed",
                    "direction": "ASC"
                }],
                "operator": "ORDER_BY",
                "value": ""
            },
            {
                "name": "unit",
                "operator": "=",
                "value": getLocalVar('__CDFILIAL')
            }
        ];
        if(patient){
            setTimeout(function(){
                newHospitalizationPopup.getField('id').disabled = true;
                newHospitalizationPopup.currentRow.id = patient.id;
                newHospitalizationPopup.currentRow.idnamepaci = patient.idnamepaci;
                newHospitalizationPopup.getField('id').reload();
            }, 1000);
            newHospitalizationPopup.getField('id').readOnly = true;
        }else{
            newHospitalizationPopup.getField('id').disabled = false;
            newHospitalizationPopup.getField('id').readOnly = false;
        }
        newHospitalizationPopup.currentRow.hospitalizationDate = moment().format('YYYY-MM-DDTHH:mm');
        eventAggregator.publish("reloadAllFields", newHospitalizationPopup);
        ScreenService.openPopup(newHospitalizationPopup);
    };
    
    $scope.setPatientAge = function(fieldEdited) {
    	eventAggregator.publish("setPatientAge", fieldEdited);
    };

    this.infiniteScroll = function() {
        var scrollHeight = container[0].scrollHeight - 15;
        var scrollPosition = container.height() + container.scrollTop() - 15; //padding
        var previousScroll = previousScroll || scrollPosition;
        if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll <= scrollPosition) {
            this.loadData(true);
        }
        previousScroll = scrollPosition;
    };

    /**
     * Retrieve content from list dataSource
     * @param  {Boolean} apply - Scope apply selector
     */
    this.loadData = function(apply) {
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        this.nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            $scope.loadingData = false;
        }.bind(this));
    };
    
    this.nextPage = function(widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    };
    
    this.searchPatient = function(widget){
        var search = angular.copy(widget.typeSearch);
        if(search){
            search = removerAcentos(search);
        }
        if(!widget.dataSourceFilter){
            widget.dataSourceFilter = [{
                name: "normalizedName",
                operator: "LIKE",
                value: "%"+search+"%"
            }];
        }else{
            var noDataFound = true;
            widget.dataSourceFilter.forEach(function(filter){
               if(filter.name === 'normalizedName'){
                   filter.value = "%"+search+"%";
                   noDataFound = false;
               } 
            });
            if(noDataFound){
                widget.dataSourceFilter.push({
                    name: "normalizedName",
                    operator: "LIKE",
                    value: "%"+search+"%"
                });
            }
        }
        widget.reload();
    };
    
    function removerAcentos(newStringComAcento) {
        var string = newStringComAcento;
        var mapaAcentosHex = {
            a: /[\xE0-\xE6]/g,
            A: /[\xC0-\xC6]/g,
            e: /[\xE8-\xEB]/g,
            E: /[\xC8-\xCB]/g,
            i: /[\xEC-\xEF]/g,
            I: /[\xCC-\xCF]/g,
            o: /[\xF2-\xF6]/g,
            O: /[\xD2-\xD6]/g,
            u: /[\xF9-\xFC]/g,
            U: /[\xD9-\xDC]/g,
            c: /\xE7/g,
            C: /\xC7/g,
            n: /\xF1/g,
            N: /\xD1/g
        };

        for (var letra in mapaAcentosHex) {
            var expressaoRegular = mapaAcentosHex[letra];
            string = string.replace(expressaoRegular, letra);
        }

        return string;
    }

    container.on("scroll", this.infiniteScroll.bind(this));
    
    $scope.formatAge = function(patient){
        if(patient){
            var anos = patient.yearAndMonth[0];
            var meses = patient.yearAndMonth[1];
            
            if(anos === "0"){
                return meses + ' ' + ScreenService.i18n('Mês(es)');
            }else if(meses === "0"){
                return anos + ' ' + ScreenService.i18n('Ano(s)');
            }else{
                return anos + ' ' + ScreenService.i18n('Ano(s)')  + ' ' +  ScreenService.i18n('e') + ' '+ meses  + ' ' + ScreenService.i18n('Mês(es)');
            }
        }
    };
}


// FILE: ./tecdiet_templates/js/PatientDetailPopupController.js
function PatientDetailPopupController(ApplicationContext, templateManager, ScreenService, eventAggregator, ZHPromise) {
    var self = this;
    
    var patientsRepository = ApplicationContext.PatientsRepository;
    var bedsService = ApplicationContext.BedsService;
    
    this.openPopup = function(detailPopupWidget, patient){
        detailPopupWidget.isVisible = true;
        detailPopupWidget.isEditingPatient = false;
        detailPopupWidget.patient = patient;
        
        if (patient.bed) {
            bedsService.getBed(patient.bed).then(function(bedObject) {
                detailPopupWidget.bed = bedObject;
                templateManager.updateTemplate();
            });
        } else {
            detailPopupWidget.bed = {
                nullBed: true
            };
        }
    };
    this.closePopup = function(detailPopupWidget){
        if (!detailPopupWidget.isEditingPatient) {
            detailPopupWidget.isVisible = false;
            delete detailPopupWidget.patient;
        }
    };
    this.editPatientDetails = function(detailPopupWidget) {
        detailPopupWidget.backupRow = {};
        detailPopupWidget.backupRow.patient = Object.assign({}, detailPopupWidget.patient);
        detailPopupWidget.backupRow.bed = Object.assign({}, detailPopupWidget.bed);
        detailPopupWidget.choosedOptions = [];
        bedsService.getVacantBeds().then(function(beds) {
            let vacantBeds = beds || [];
            let currentBedArr = [detailPopupWidget.bed];
            detailPopupWidget.vacantBeds = currentBedArr.concat(vacantBeds);

            detailPopupWidget.isEditingPatient = true;
        });
        var bed = detailPopupWidget.getField('bedName');
        bed.dataSourceFilter = [];
        detailPopupWidget.currentRow['storeyName'] = '';
        detailPopupWidget.currentRow['hallName'] = '';
        detailPopupWidget.currentRow['bedName'] = '';
        eventAggregator.publish("reloadAllFields", detailPopupWidget);
    };
    function convertStringToJson(string){
        try{
            return JSON.parse(string);
        }catch (e){
            return false;
        }
    }
    this.saveEditPatientDetails = function(detailPopupWidget) {
        
       let patientToSave = detailPopupWidget.patient;
        patientToSave.changed = true;
        var bed = convertStringToJson(detailPopupWidget.currentRow['bedName']);
        if(bed){
            patientToSave.bed = bed.bed;
            patientToSave.bedName = bed.bedName;
            patientToSave.hall = bed.hall;
            patientToSave.storey = bed.storey;
            patientToSave.oldBed = detailPopupWidget.backupRow.bed;
            patientsRepository.getPatientById(patientToSave.id).then(function(patient) {
                patient = Object.assign(patient, patientToSave);
                var promise = ZHPromise.defer();
                var params = {
                    'dataSourceName': '/Patients',
                    'rowToSave': [patient],
                    'route': '/Patients/save',
                    'type' : 'DataSet',
                    'promise': promise
                };
                eventAggregator.publish('saveAlterPatient', params);
                promise.then(function(){
                    patientsRepository.save(patient).then(function(){
                        bedsService.getBed(bed.bed).then(function(vacantBedToOccupy){
                            bedsService.getBed(detailPopupWidget.backupRow.bed.bed).then(function(occupiedBedToVacate){
                                bedsService.localSwapBeds(vacantBedToOccupy, occupiedBedToVacate);
                                detailPopupWidget.bed = bed;
                                detailPopupWidget.isEditingPatient = false;
                                changeDataSource(detailPopupWidget.getField('hallName'), '/HallsDistincts', true);
                                delete detailPopupWidget.backupRow;
                            });
                        });
                    });
                });
            });
        }else{
            ScreenService.showMessage(ScreenService.i18n('Selecione um leito!'));
        }
    };
    function changeDataSource(field, dataSourceName, clearFilters){
        field.dataSource.name = dataSourceName;
        if(clearFilters){
            field.dataSourceFilter = [];
        }
        field.reload();
    }
    this.cancelEditPatientDetails = function(detailPopupWidget) {
        var fieldHall = detailPopupWidget.getField('hallName');
        var fieldBed = detailPopupWidget.getField('bedName');
        detailPopupWidget.isEditingPatient = false;
        
        detailPopupWidget.patient = Object.assign(detailPopupWidget.patient, detailPopupWidget.backupRow.patient);
        detailPopupWidget.bed = Object.assign(detailPopupWidget.bed, detailPopupWidget.backupRow.bed);
        changeDataSource(fieldHall, '/HallsDistincts', true);
        fieldBed.dataSourceFilter = [];
        fieldBed.reload();
        delete detailPopupWidget.backupRow;
    };
    this.changeStorey = function(widget){
        var value = JSON.parse(widget.currentRow['storeyName']);
        widget.getField('bedName').dataSourceFilter = [{
            "name": "storey",
            "operator": "=",
            "value": value.storey
        }];
        widget.getField('hallName').dataSource.name = '/Halls';
        widget.getField('hallName').dataSourceFilter = [{
            "name": "storey",
            "operator": "=",
            "value": value.storey
        }];
        widget.currentRow['bedName'] = '';
        widget.getField('bedName').reload();
        widget.currentRow['hallName'] = '';
        widget.getField('hallName').reload();
    };
    this.changeBed = function(widget){
        var valor = JSON.parse(widget.currentRow['bedName']);
        widget.currentRow['storeyName'] = valor.storey;
        widget.currentRow['hallName']   = valor.hall;
    };
    this.changeHall = function(widget){
        var bed = widget.getField('bedName');
        if(!hasHallInFilter(bed)){
            bed.dataSourceFilter.push({
                "name": "hall",
                "operator": "=",
                "value": widget.currentRow['hallName']
            });
        }else{
            bed.dataSourceFilter.forEach(function(filtro){
               if(filtro.name === "hall"){
                   filtro.operator = "=";
                   filtro.value = widget.currentRow['hallName'];
               }
            });
        }
        var storey = convertStringToJson(widget.currentRow['storeyName']);
        if(typeof storey === "number"){
            bed.dataSourceFilter.push({
                "name": "storey",
                "operator": "=",
                "value": storey
            });
        }
        widget.currentRow['bedName'] = '';
        bed.reload();
    };
    
    function hasHallInFilter(field){
        var has = false;
        if(field.dataSourceFilter){
            field.dataSourceFilter.forEach(function(filtro){
               if(filtro.name === "hall"){
                   has = true;
               } 
            });
        }
        return has;
    }
    
    this.changePatientSituation = function(detailPopupWidget, patient, situation) {
        let messages = [
            'Deseja confirmar <b>alta</b> para '+patient.formattedName+'?',
            'Deseja confirmar <b>óbito</b> para '+patient.formattedName+'?'
        ];
        ScreenService.confirmMessage(messages[situation], 'question', function() {
            eventAggregator.publish("sendPatientUpdate", {'patient': [patient], 'situation': situation,'widget': detailPopupWidget});
            self.closePopup(detailPopupWidget);
        });
    };
};

// FILE: ./tecdiet_templates/js/PatientDetailsController.js
function PatientDetailsController($scope, $timeout, eventAggregator, ZHPromise, localStorageEngine, ScreenService, ApplicationContext, templateManager, $element) {
    /*global moment, angular, $, Util, excelFormulaUtilities, getLocalVar*/
    var widget = $scope.widget;
    var controlFeedingCrud = ApplicationContext.CRUD.init("/ControlFeedings");
    var lactaryFeedingCrud = ApplicationContext.CRUD.init("/LactaryFeedings");
    var enteralFeedingCrud = ApplicationContext.CRUD.init("/EnteralFeedings");
    var normalFeedingCrud = ApplicationContext.CRUD.init("/NormalFeedings");

    $scope.bindScrollListener = function(selector) {
        if(!selector){
            selector = '.list-plates';
        }
        var element = $element.get(0);
        var listPlates = element.querySelector(selector);
        listPlates.addEventListener('scroll', function(e) {
            var scrollHeight = e.target.scrollHeight - 15;
            var scrollPosition = e.target.clientHeight + e.target.scrollTop - 15; //padding
            var previousScroll = previousScroll || scrollPosition;
            if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll <= scrollPosition) {
                loadData(true);
            }
            previousScroll = scrollPosition;
        });
    };
    
    function loadData(apply) {
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            $scope.loadingData = false;
        }.bind(this));
    }
    
    function nextPage (widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    }
    
    $scope.updateProcedurePrescription = function(widget){
        eventAggregator.publish('updateProcedureRestriction', widget);
    };

    function setClassification(bioAval) {
        if (!bioAval) {
            return null;
        }
        var FORMULA = bioAval.examinationFormula;
        if (FORMULA) {
            var result = FORMULA.replace(/SEXO/g, 'window.SEXO').replace(/IDADE/g, 'window.IDADE').replace(/RESULTADO/g, 'window.RESULTADO');
            window.IDADE = bioAval.age;
            window.RESULTADO = bioAval.examinationValue;
            if (bioAval.gender === 0) {
                window.SEXO = 'MALE';
            } else {
                window.SEXO = 'FEMALE';
            }
            var formulaJS = excelFormulaUtilities.formula2JavaScript(result);
            formulaJS = formulaJS.split("AND").join("&&");
            
            if(formulaJS.charAt(formulaJS.length-1) !== ")"){
                formulaJS = formulaJS.substr(0, formulaJS.length-1);
            }
            try{
                return eval(formulaJS); // jshint ignore:line|
            }catch(error){
                return null;
            }
        } else {
            return null;
        }
    }

    function createNewFeeding(feedingData) {
        widget.newControlData = true;
        widget.filtredFeedingsForWidget = [];
        var idtpservhosp = { //Servico
            'oral': '0',
            'pare': '1',
            'lact': '2'
        };
        var idformdiet = { //Diet
            'oral': ['0', '3', '4'],
            'lact': '1',
            'pare': '2'
        };
        var typeClicked = feedingData;
        feedingData = {};
        feedingData.unit = localStorageEngine.getLocalVar('__CDFILIAL');
        feedingData.idtpservhosp = idtpservhosp[typeClicked];
        feedingData.idformdiet = idformdiet[typeClicked];
        feedingData.id = widget.curPatient.id;
        feedingData.typeClicked = typeClicked;
        feedingData.serviceDate = moment().format("YYYY-MM-DD");
        return feedingData;
    }
    $scope.onClickSelectNative = function(event, field){
        var select = document.activeElement;
        if(!field.addEventBlur){
            document.activeElement.onblur = function(){
                field.readOnly = true;
                templateManager.updateTemplate();
            };
            field.addEventBlur = true;
        }
        if(!templateManager.keyboardOpen){
                field.readOnly = false;
                templateManager.updateTemplate();
        } else {
            if(Util.isPhone() || Util.isTablet()){
                field.readOnly = true;
                templateManager.updateTemplate();
                setTimeout(function(){
                    field.readOnly = false;
                    templateManager.updateTemplate();
                    select.click();
                    event.stopPropagation();
                },800);
            } else {
                field.readOnly = false;
                templateManager.updateTemplate();
            }
        }
    };
    function filterServicesForFeedingControlPopUp(feedingData, filter, isNew) {
        return localStorageEngine.getDataSource('/Services', filter).then(function (filteredServices) {
            widget.filteredServicesFeeding = filteredServices;

            filter = [
                {
                    "name": "unit",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar('__CDFILIAL')
                }
            ];
            var service = feedingData.cdservfili ? feedingData.cdservfili : feedingData.serviceId; 
            if (!isNew) {
                filter.push({
                    "name": "serviceId",
                    "operator": "=",
                    "value": service
                });
            }

            if (feedingData.idtpservhosp === "0") {//if Oral
                filter.push({
                    "name": "service",
                    "operator": "IN",
                    "value": ['0', '3', '4']
                });
            } else {
                var filterService = feedingData.idformdiet ? feedingData.idformdiet : feedingData.dietService;
                filter.push({
                    "name": "service",
                    "operator": "=",
                    "value": filterService
                });
            }

            filterDietsForFeedingControlPopUp(feedingData, filter, isNew);
        })
            .catch(function (error) {
                console.log(error);
            });
    }

    function filterDietsForFeedingControlPopUp(feedingData, filter, isNew) {
        localStorageEngine.getDataSource('/DietsServices', filter).then(function (filteredDiets) {
            // widget.filteredDietsFeeding = isNew ? [] : filteredDiets;     //Changed for tests
            widget.filteredDietsFeeding = isNew ? filteredDiets : filteredDiets;
            // $timeout(function () {
            openPopupFeedingControl(feedingData, isNew);
            // }, 100);
        });
    }

    function openPopupFeedingControl(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        widgetPopup.currentRow = feedingData.id ? angular.copy(feedingData) : feedingData;
        widgetPopup.currentRow.serviceDate = moment(widgetPopup.currentRow.serviceDate, "DD/MM/YYYY").format("YYYY-MM-DD");
        widgetPopup.currentRow.__is_new = isNew;
        widgetPopup.container.showMenu = false;

        if (isNew) {
            widget.edit();
        } else {
            widgetPopup.currentRow.indexOfOriginalControlFeedings = widget.localDataSourceLoaded.ControlFeedings.indexOf(feedingData);
            widgetPopup.currentRow.indexOfOriginalCurPatFeedings = widget.curPatient.ControlFeedings.indexOf(feedingData);
            
            if (widgetPopup.currentRow.idtpservhosp === '0') { //Oral feeding
                openOralEditor(feedingData, isNew);
            } else if (widgetPopup.currentRow.idtpservhosp === '1') { //Enteral feeding
                openEnteEditor(feedingData, isNew);
            } else { //Lactary Feeding
                openLacEditor(feedingData, isNew);
            }
        }
        widget.container.showMenu = false;
       //widget.currentRow.isEditting = true;
    }

    function openOralEditor(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var dtCard = moment(feedingData.serviceDate, "DD/MM/YYYY");
            
            dtCard = dtCard.isValid() ? dtCard.format("DD/MM/YYYY") : moment(feedingData.serviceDate, "YYYY-MM-DD")
            .format("DD/MM/YYYY");
        var filter = [{
            "name": "unit",
            "operator": "=",
            "value": localStorageEngine.getLocalVar('__CDFILIAL')
        }, 
        {
            "name": "id",
            "operator": "=",
            "value": localStorageEngine.getLocalVar('__patient')
        },
        {
            "name": "serviceDate",
            "operator": "=",
            "value": dtCard
        },
        {
            "name": "sequential",
            "operator": "=",
            "value": feedingData.sequential
        },
        {
            "name": [{
                "name": "unique",
                "direction": "ASC"
            }],
            "operator": "ORDER_BY",
            "value": ""
        }];
        var patientIntoRestriction = widget.container.getWidget('WidgetDetailsPatient').curPatient.PatientIntoRestriction;
        if(Array.isArray(patientIntoRestriction) && patientIntoRestriction.length){
            widgetPopup.currentRow.PatientIntoRestriction = patientIntoRestriction.filter(function(intoRestriction){
                return intoRestriction.NRSEQUSERV === feedingData.sequential && intoRestriction.DTSERVINTE === feedingData.serviceDate;
            });
        } else {
            widgetPopup.currentRow.PatientIntoRestriction = [];
        }
        normalFeedingCrud.find(filter,false,'/NormalFeedings').then(function (feedings) {
            feedings.map(function(feeding){
                feeding.plateObservationMode = !!feeding.observation;
            });
            var arrayFinal = [];
            feedings.reduce(function(prev, current) { // funções para organizar e separar o primeiro registro de cada tipo de prato
                var index = prev.findIndex(function(j) {// a fim de escrever a label apenas acima do primeiro
                    return j.plateType === current.plateType;
                });
                if (index < 0) {
                    index = prev.length;
                    prev.push({
                        plateType: current.plateType,
                        lista: []
                    });
                }
                prev[index].lista.push(current);
                return prev;
            }, []).map(function(item) {
                return item.lista;
            }).forEach(function(it) {
                if (it.length === 0) {
                    it.isFirst = true;
                    arrayFinal.push(it);
                } else {
                    for (var i = 0; i < it.length; i++) {
                        if (i === 0) {
                            it[i].isFirst = true;
                        }
                        arrayFinal.push(it[i]);
                    }
                }
            });
            widgetPopup.filtredFeedingsForWidget = angular.copy(arrayFinal);
            if (!widgetPopup.currentRow.selectedDiet && widgetPopup.currentRow.dietId) {
                widgetPopup.currentRow.selectedDiet = [];
                widgetPopup.currentRow.selectedDiet.dietId = widgetPopup.currentRow.dietId;
            }
            if(widgetPopup.currentRow.idformdiet === '4' || widgetPopup.currentRow.dietService === '4'){
                widgetPopup.bkpFeedings = angular.copy(arrayFinal);
            }
        });
        widgetPopup.currentRow.serviceId = widgetPopup.currentRow.cdservfili;
        widgetPopup.getField('serviceId').dataSource.data = 
            [{
                "serviceId":widgetPopup.currentRow.serviceId,
                "serviceName":widgetPopup.currentRow.serviceName
            }];
        widgetPopup.getField('dietId').dataSource.data = 
            [{
                "dietId":widgetPopup.currentRow.dietId,
                "dietName":widgetPopup.currentRow.dietName
            }];    
        widgetPopup.getField('dietId').value(widgetPopup.currentRow.dietId);
        ScreenService.openPopup(widgetPopup);
        widgetPopup.edit();
    }
    function openEnteEditor(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var fieldServico = widgetPopup.getField('serviceId');
        var fieldDieta = widgetPopup.getField('dietId');
        fieldServico.readOnly = true;
        fieldDieta.readOnly = true;
        widgetPopup.currentRow.serviceId = widgetPopup.currentRow.cdservfili;
        widgetPopup.getField('serviceId').dataSource.data = 
            [{
                "serviceId":widgetPopup.currentRow.serviceId,
                "serviceName":widgetPopup.currentRow.serviceName
            }];
        widgetPopup.getField('dietId').dataSource.data = 
            [{
                "dietId":widgetPopup.currentRow.dietId,
                "dietName":widgetPopup.currentRow.dietName
            }];
        widgetPopup.getField('dietId').value(widgetPopup.currentRow.dietId);
        ScreenService.openPopup(widgetPopup);
        widgetPopup.view();
    }
    function openLacEditor(feedingData, isNew) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        widgetPopup.currentRow.dietName = !widgetPopup.currentRow.dietName ? widgetPopup.currentRow.selectedDiet.dietName : widgetPopup.currentRow.dietName;
        widgetPopup.currentRow.serviceId = widgetPopup.currentRow.cdservfili;
        widgetPopup.getField('serviceId').dataSource.data = 
            [{
                "serviceId":widgetPopup.currentRow.serviceId,
                "unit":widgetPopup.currentRow.unit,
                "serviceName":widgetPopup.currentRow.serviceName
            }];
        widgetPopup.getField('dietId').dataSource.data = 
            [{
                "dietId":widgetPopup.currentRow.dietId,
                "dietName":widgetPopup.currentRow.dietName
            }];
        widgetPopup.getField('dietId').value(widgetPopup.currentRow.dietId);
        ScreenService.openPopup(widgetPopup);
        widgetPopup.view();
    }

    function setLactaryFeedingColumnsAndPropertiesOnFieldEdited(controlFeeding){
        
        var lactaryFeedingColumns = ["Data", "Hora", "Formulação", "Qtd.Formulação"];
        var lactaryFeedingProperties = ["lactaryDate", "hourLactaryPlate", "formulationName", "plateNumbers"];
        
        controlFeeding.columns = lactaryFeedingColumns;
        controlFeeding.propertyFeedings = lactaryFeedingProperties;
        
    }
    
    function getCurEnteralFeeding(feedingData) {
        var filter = [
            {
                "name": "serviceDate",
                "operator": "=",
                "value": feedingData.serviceDate
            }, 
            {
                "name": "id",
                "operator": "=",
                "value": widget.curPatient.id
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": feedingData.sequential
            }
        ];
        enteralFeedingCrud.find(filter,false,'/EnteralFeedings').then(function (entFeeding) {
            var info = ['enteralDate', 'time', 'accessRoadName', 'preparationName', 'formulationName', 'volPrescription', 'volInfudido', 'serviceNumbers'];
            var header = ['Data', 'Hora', 'Via de Acesso', 'Preparação', 'Formulação', 'Vol.Prescrito', 'Vol.Infundido', 'Velocidade (ML/H)'];

            //## FOR INPUTS ON ADITION  # d = aberto ou fechado # n = nao tem # f = funcao
            var cds = ['n', 'n', 'accessRoadId', 'n', 'formulationId', 'n', 'n', 'n'];
            var source = ['n', 'n', 'AccessRoads', 'd', 'Formulations', 'n', 'n', 'n'];
            widget.fieldSource = {};

            var fields = [];
            for (var i = 0; i < header.length; i++) {
                header[i] = ScreenService.i18n(header[i]);
                fields.push({
                    'name': header[i],
                    'value': info[i],
                    'source': source[i],
                    'cd': cds[i]
                });
            }

            fields.forEach(function (field) {
                if (field.source !== 'n' &&
                    field.source !== 'f' &&
                    field.source !== 'd') {
                    var source = '/' + field.source;

                    localStorageEngine.getDataSource(source, null).then(function (data) {
                        widget.fieldSource[field.source] = data;
                    });

                }
            });

            fields.push({
                'name': ScreenService.i18n('Observação'),
                'value': 'observation',
                'source': 'n',
                'cd': 'n'
            });
            widget.subItemData = [];
            feedingData.propertyFeedings = info;
            feedingData.fields = fields;
            feedingData.colums = header;
            feedingData.feedings = entFeeding;

        });
    }

    function createOrEditFeedingControl(feedingData) {
        feedingData.serviceDate = feedingData.serviceDate ? moment(feedingData.serviceDate, "DD/MM/YYYY").format("DD/MM/YYYY") : moment().format("DD/MM/YYYY");
        var originalIsNewStatus = feedingData.__is_new ? feedingData.__is_new : false;
        if (feedingData.isNewRegister) {
            feedingData = createNewFeeding(feedingData);
        } else {
            widget.newControlData = false;
        }
        feedingData.originalIsNewStatus = originalIsNewStatus;
        feedingData.isNewRegister = feedingData.hasOwnProperty('isNewRegister') ? feedingData.isNewRegister : true; 
        var service = feedingData.idtpservhosp;
        var filter = [
            {
                "name": "service",
                "operator": "=",
                "value": service
            }, {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        if (feedingData.idtpservhosp === '0' || feedingData.idtpservhosp === '3' || feedingData.idtpservhosp === '4') {//ORAL
            feedingData.typeClicked = 'oral';
        }
        else if (feedingData.idtpservhosp === '1') {//Enteral
            feedingData.typeClicked = 'pare';
            widget.subitemtoadd = 'enteralProd';
            if (!feedingData.isNewRegister) {
                getCurEnteralFeeding(feedingData);
            }
        }
        else if (feedingData.idtpservhosp === '2') {//LACTARIO
            feedingData.typeClicked = 'lact';

            if (!feedingData.isNewRegister) {
                
                setLactaryFeedingOnSelectedLactaryPrescription(feedingData);
                setLactaryFeedingColumnsAndPropertiesOnFieldEdited(feedingData);
                feedingData.isLactaryPrescriptionEditing = true;
                
            }

        }
        setTimeout(function () {
            filterServicesForFeedingControlPopUp(feedingData, filter, feedingData.isNewRegister);
        }, 130);

    }
    
    $scope.editEnteralFeeding = function (entFeeding) {
        var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
        var fieldFormulation = widgetPopupItemEnt.getField('formulationId');
        widget.isSubeditting = true;
        widgetPopupItemEnt.edit();
        $scope.setEnteralFeedingInfosOnEditingEnvironment(widget, entFeeding);
        widgetPopupItemEnt.getField('automatic').isVisible = false;
        widgetPopupItemEnt.getField('period').isVisible = false;
        widgetPopupItemEnt.getField('interval').isVisible = false;
        fieldFormulation.disabled = false;
        if(widgetPopupItemEnt.currentRow.preparationId === 'F'){
            fieldFormulation.valueField = 'productId';                                            
        } else {
            fieldFormulation.valueField = 'formulationId';
        }
        fieldFormulation.value(widgetPopupItemEnt.currentRow[fieldFormulation.valueField]);
        configurationWidget(widgetPopupItemEnt, 'modeEdit');
        ScreenService.openPopup(widgetPopupItemEnt);

    };
    
    $scope.editLactaryFeeding = function (lactaryFeeding) {
        var widgetPopupItemLac = widget.container.getWidget('widgetPopupItemLac');
        
        widgetPopupItemLac.edit();
        widgetPopupItemLac.currentRow = angular.copy(lactaryFeeding);
        widgetPopupItemLac.currentRow.lactaryDate = moment(widgetPopupItemLac.currentRow.lactaryDate, "DD/MM/YYYY").format("YYYY-MM-DD"); 
        if(typeof widgetPopupItemLac.currentRow.plateNumbers === 'string'){
            widgetPopupItemLac.currentRow.plateNumbers = parseFloat(widgetPopupItemLac.currentRow.plateNumbers.split(',').join('.'));
        } else {
            widgetPopupItemLac.currentRow.plateNumbers = parseFloat(widgetPopupItemLac.currentRow.plateNumbers);
        }
        widgetPopupItemLac.currentRow.hourLactaryPlate = widgetPopupItemLac.currentRow.hourLactaryPlate.split(":").join("");
        widgetPopupItemLac.getField('hourLactaryPlate').reload();
        widgetPopupItemLac.getField('automatic').isVisible = false;
        widgetPopupItemLac.getField('period').isVisible = false;
        widgetPopupItemLac.getField('interval').isVisible = false;
        configurationWidget(widgetPopupItemLac, 'modeEdit');
        ScreenService.openPopup(widgetPopupItemLac);
        widget.lactaryFeedingBackUp = angular.copy(lactaryFeeding);
        
        setTimeout(function(){
            $scope.onChangeLactaryFeeding(widget);
        }, 200);
        
        $scope.setLactaryFeedingInfosOnEditingEnvironment(widget, lactaryFeeding);

    };
    
    $scope.setLactaryFeedingInfosOnEditingEnvironment = function(widget, lactaryFeeding){
        
        widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding = lactaryFeeding;
        var curLactaryFeeding = widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding;
        
        //curLactaryFeeding.lactaryDate = moment(lactaryFeeding.lactaryDate, "DD/MM/YYYY HH:mm").format("YYYY-MM-DDTHH:mm");
        curLactaryFeeding.formulationId = curLactaryFeeding.formulationId ? curLactaryFeeding.formulationId : lactaryFeeding.formulationId;
        if(typeof curLactaryFeeding.plateNumbers === 'string'){
            curLactaryFeeding.plateNumbers = Number(lactaryFeeding.plateNumbers.split(',').join('.'));
        } else {
        curLactaryFeeding.plateNumbers = Number(lactaryFeeding.plateNumbers);
        }
        curLactaryFeeding.observation = lactaryFeeding.observation;
    };
    
    $scope.setEnteralFeedingInfosOnEditingEnvironment = function (widget, enteralFeeding) {
        var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
        widgetPopupItemEnt.currentRow = angular.copy(enteralFeeding);
        var currentRowSubItemEnt = widgetPopupItemEnt.currentRow;
        currentRowSubItemEnt.time = moment(enteralFeeding.time,"HH:mm").format("HHmm");
        currentRowSubItemEnt.enteralDate = moment(enteralFeeding.enteralDate, "DD/MM/YYYY").format("YYYY-MM-DD");
        widgetPopupItemEnt.getField('accessRoadId').value(currentRowSubItemEnt.accessRoadId);
        widgetPopupItemEnt.getField('preparationId').value(currentRowSubItemEnt.preparationId);
        var formulation = widgetPopupItemEnt.getField('formulationId');
        var fieldAccess = widgetPopupItemEnt.getField('accessRoadId');
        fieldAccess.widget.currentRow.accessRoadId = fieldAccess.value();
        var accessRoad = fieldAccess.dataSource.data.filter(function(access) {
            return access.accessRoadId === fieldAccess.value();
        })[0];
        fieldAccess.widget.currentRow.accessRoadName = accessRoad.accessRoadName;
        fieldAccess.reload();
        fieldAccess.value(accessRoad.accessRoadId);
        formulation.dataSourceFilter = [
            {
                "name": "preparationId",
                "operator": "=",
                "value": currentRowSubItemEnt.preparationId
            },
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        formulation.value(currentRowSubItemEnt.formulationId);
        formulation.readOnly = false;
    };

    function setLactaryFeedingOnSelectedLactaryPrescription(prescription) {

        var filter = [
            {
                "name": "id",
                "operator": "=",
                "value": prescription.id
            },
            {
                "name": "unit",
                "operator": "=",
                "value": prescription.unit
            },
            {
                "name": "serviceDate",
                "operator": "=",
                "value": moment(prescription.serviceDate, "DD/MM/YYYY").format("DD/MM/YYYY")
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": prescription.sequential
            }
        ];

        return lactaryFeedingCrud.find(filter,false,'/LactaryFeedings').then(function (lactaryFeedings) {
                prescription.lactaryFeedings = lactaryFeedings;
            })
            .catch(function (error) {
                console.log(error);
            });

    }

    $scope.pressToChooseAll = function (widget, dataSource) {

        widget.allFieldsAreSelected = widget.allFieldsAreSelected ? false : true;

        if (widget.allFieldsAreSelected) {
            dataSource.forEach(function (field) {
                if (!field.selectedRow) {
                    field.selectedRow = true;
                    widget.selectedRows.push(field);
                }
            });
        } else {
            widget.selectedRows = [];
            dataSource.forEach(function (field) {
                field.selectedRow = false;
            });
        }
    };// 
    //
    $scope.pressToChoose = function (row, widget, dataSource) {
        var indexOfAtr = widget.selectedRows.indexOf(row);
        if(widget.showPopUpRestr){
            indexOfAtr = widget.selectedRows.findIndex(function(rowToSelect){
                return rowToSelect.cdgrupprod === row.cdgrupprod && rowToSelect.cdsubgrprod === row.cdsubgrprod;
            });
        }
        row.selectedRow = row.selectedRow !== true ? true : false;
        if (indexOfAtr > -1) {
            widget.selectedRows.splice(indexOfAtr, 1);
        } else {
            widget.selectedRows.push(row);
        }
        if(Array.isArray(dataSource) && dataSource.length === widget.selectedRows.length){
            widget.allFieldsAreSelected = true;
        }else{
            widget.allFieldsAreSelected = false;
        }
    };
    
    $scope.afterMoveRowGrid = function(widget, container, element){
        widget.currentRow = angular.copy(element);
        switch (widget.name){
            case 'WidgetUnit':
                eventAggregator.publish('afterMoveRowUnit', container);
                break;
            case 'widgetPopupHospitalization':
                $scope.pressToChooseSelect(element,widget);
                break;
            default:
                return null;
        }
    };
    
    $scope.pressToChooseSelect = function (row, widget) {
        if(!widget.selectUnique){
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
            var indexOfAtr = widget.selectedRows.findIndex(function(selectedRow){
                return row.formulationId === selectedRow.formulationId && row.plateType === selectedRow.plateType;
            });
            row.selectedRow = row.selectedRow !== true ? true : false;

            if(Array.isArray(widgetPopupPrescricao.filtredFeedingsForWidget) && widgetPopupPrescricao.filtredFeedingsForWidget.length){
                var index = widgetPopupPrescricao.filtredFeedingsForWidget.findIndex(function(feeding){
                    return feeding.formulationId === row.formulationId && feeding.plateType === row.plateType;
                });
                if(index !== -1){ 
                    widgetPopupPrescricao.filtredFeedingsForWidget.splice(index, 1);   
                }
            }
            if (indexOfAtr > -1) {
                updateBkpFeedings(widgetPopupPrescricao, widget.selectedRows[indexOfAtr], true);
                widget.selectedRows.splice(indexOfAtr, 1);
            } else {
                updateBkpFeedings(widgetPopupPrescricao, row, false);
                widget.selectedRows.push(row);
            }
            if(widget.selectedRows.length === widget.elementsToSearch.length){
                widget.allFieldsAreSelected = true;
            }else{
                widget.allFieldsAreSelected = false;
            }
        } else {
            var fieldFormulation;
            switch(widget.name){
                case 'widgetPopupFormulationLact':
                    var widgetPopupItemLac = widget.container.getWidget('widgetPopupItemLac');
                    fieldFormulation = widgetPopupItemLac.getField('formulationId');
                    fieldFormulation.value(row.cdformulacao);
                    widget.container.getWidget('widgetPopupItemLac').getField('un').value(row.un);
                    widgetPopupItemLac.currentRow.cdformulacao = row.cdformulacao; 
                    widgetPopupItemLac.currentRow.formulationName = row.nmformulacao;
                    break;
                case 'widgetPopupFormulationEnt':
                    var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
                    fieldFormulation = widgetPopupItemEnt.getField('formulationId');
                    fieldFormulation.value(row.cdformulacao);
                    widgetPopupItemEnt.currentRow.cdformulacao = row.formulationId; 
                    widgetPopupItemEnt.currentRow.formulationId = row.formulationId; 
                    widgetPopupItemEnt.currentRow.formulationName = row.formulationName;
                    break;
                case 'widgetPopupHospitalization':
                    var newHospitalizationPopup = widget.container.getWidget('newHospitalizationPopup');
                    var fieldPatient = newHospitalizationPopup.getField('id');
                    fieldPatient.value(row.id);
                    newHospitalizationPopup.currentRow.id = row.id; 
                    newHospitalizationPopup.currentRow.idnamepaci = row.idnamepaci; 
                    break;
                    
            }    
            widget.isVisible = false;
            widget.descriptionPopupSelect.search = "";
            ScreenService.closePopup(widget);
        } 
        /*
        
            var itemList = $(event.currentTarget);
            itemList.hasClass('hide-all') ? itemList.removeClass('hide-all') : itemList.addClass('hide-all');
            itemList.siblings('.fields-dietPrescription').addClass('hide-all');
        
        */
    };
    $scope.onChangeSearch = function(fieldOrWidget, textToSearch){
        var promise = ZHPromise.defer();
        setTimeout(function(){
            var text = "";
            if(textToSearch){
                text = textToSearch;
            }else{
                text = (fieldOrWidget.search === "" || fieldOrWidget.search) ? fieldOrWidget.search : "";
                text = (!text && fieldOrWidget.descriptionPopupSelect && fieldOrWidget.descriptionPopupSelect.search) ? fieldOrWidget.descriptionPopupSelect.search : text;
            }
            text = removerAcentos(text);
            var searchFromDataSourceFilter = fieldOrWidget.searchFromDataSourceFilter;
            var arrayToFilter = fieldOrWidget[fieldOrWidget.arrayToFilter];
            var propertyToFind = fieldOrWidget.propertyToFind;
            var functionToSearch = fieldOrWidget.functionToSearch;
            if(typeof text === 'string' && typeof propertyToFind === 'string' && typeof fieldOrWidget === 'object' && text !== fieldOrWidget.lastFindText) {
                    if(searchFromDataSourceFilter){
                        if(Array.isArray(fieldOrWidget.dataSourceFilter) && fieldOrWidget.dataSourceFilter.length){
                            var index = fieldOrWidget.dataSourceFilter.findIndex(function(filter){
                                return filter.name === propertyToFind;
                            });
                            if(index > -1){
                                fieldOrWidget.dataSourceFilter[index].value = '%'+text+'%';
                            } else {
                                fieldOrWidget.dataSourceFilter.push({
                                    'name': propertyToFind,
                                    'operator': "LIKE",
                                    'value': '%'+text+'%'
                                });
                            }
                            fieldOrWidget.reload().then(function(){
                                if(fieldOrWidget.widget){
                                    fieldOrWidget.widget.reload().then(function(){
                                        promise.resolve();
                                    });
                                }else{
                                    promise.resolve();
                                }
                            }, function(){
                                promise.reject();
                            });
                        } else {
                           fieldOrWidget.dataSourceFilter = [
                                {
                                    'name': propertyToFind,
                                    'operator': "LIKE",
                                    'value': '%'+text+'%'
                                }
                           ]; 
                           fieldOrWidget.reload().then(function(){
                                if(fieldOrWidget.widget){
                                    fieldOrWidget.widget.reload().then(function(){
                                        promise.resolve();
                                    });
                                }else{
                                    promise.resolve();
                                }
                            }, function(){
                                promise.reject();
                            });
                        }
                    } else {
                        if(Array.isArray(arrayToFilter)){
                            arrayToFilter = arrayToFilter.filter(function(element){
                                return removerAcentos(element[propertyToFind]).indexOf(removerAcentos(text)) > -1; 
                            });
                            promise.resolve();
                        } else {
                            promise.reject();
                        }
                    }
                    fieldOrWidget.lastFindText = text;
            } else {
                if(text !== fieldOrWidget.lastFindText){
                        switch(functionToSearch){
                            case 'searchPlate':
                                $scope.searchPlate(widget);
                                break;
                            default:
                                promise.reject();
                        }
                        promise.resolve();
                        fieldOrWidget.lastFindText = text;
                    
                }
            }
        },1000);
        return promise;
    };
    
    function updateBkpFeedings (widgetPopupPrescricao, row, statusOfRemoved){
        if(Array.isArray(widgetPopupPrescricao.bkpFeedings) && widgetPopupPrescricao.bkpFeedings.length){
            var deleteToBkp = widgetPopupPrescricao.bkpFeedings.filter(function(bkp){
                return bkp.formulationId === row.formulationId && bkp.plateType === row.plateType;
            })[0];
            if(deleteToBkp){
                deleteToBkp.remove = statusOfRemoved;
            }
        }
    }
    
    $scope.deleteControlFeeding = function(controlFeeding){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir este controle alimentar?"), 'question',
        function(accept){
            var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
            var index = widgetDetailsPatient.localDataSourceLoaded.ControlFeedings.indexOf(controlFeeding);
            var indexCurPatient = widgetDetailsPatient.curPatient.ControlFeedings.indexOf(controlFeeding);
            var platesToRemove;
            var dataSourceName;
            var params = {
                'widgetDetailsPatient': widgetDetailsPatient,
                'prescription': widgetDetailsPatient.localDataSourceLoaded.ControlFeedings[index]
            };
            eventAggregator.publish('callDeleteOnline', params);
            var datePrescription = controlFeeding.serviceDate;
            var dateFormat = datePrescription.indexOf('/') > -1 ? datePrescription : moment(datePrescription).format('DD/MM/YYYY');
            switch(controlFeeding.idtpservhosp){
                case '0': 
                case '3':
                case '4':
                    dataSourceName = "/NormalFeedings";
                    platesToRemove = widgetDetailsPatient.curPatient.NormalFeedings.filter(function(feeding){
                        return feeding.sequential === controlFeeding.sequential && feeding.serviceDate === dateFormat;
                    });
                    break;
                case '2':
                    dataSourceName = "/LactaryFeedings";
                    platesToRemove = widgetDetailsPatient.curPatient.LactaryFeedings.filter(function(feeding){
                        return feeding.sequential === controlFeeding.sequential && feeding.serviceDate === dateFormat;
                    });
                    break;
                case '1':
                    dataSourceName = "/EnteralFeedings";
                    platesToRemove = widgetDetailsPatient.curPatient.EnteralFeedings.filter(function(feeding){
                        return feeding.sequential === controlFeeding.sequential && feeding.serviceDate === dateFormat;
                    });
                    break;
            }
            if(dataSourceName === '/NormalFeedings'){
                eventAggregator.publish('callSaveOnline',widgetDetailsPatient.localDataSourceLoaded.ControlFeedings[index]);
            }
            var plateParams = {
                'plates': platesToRemove,
                'dataSourceName': dataSourceName,
                'widgetDetailsPatient': widgetDetailsPatient
            };
            eventAggregator.publish('callDeletePlatesOnline', plateParams);
           // controlFeedingCrud.removeDataset(platesToRemove, dataSourceName);
            widgetDetailsPatient.localDataSourceLoaded.ControlFeedings.splice(index,1);
            widgetDetailsPatient.curPatient.ControlFeedings.splice(indexCurPatient,1);
            
        });
    };
    
    $scope.deleteLactaryFeeding = function(lactaryFeeding){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir este prato da prescrição?"), 'question', 
        function(accept){
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');  
            var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');  
            var index = widgetPopupPrescricao.currentRow.lactaryFeedings.indexOf(lactaryFeeding);
            var funcIndex;
            var getIndexOnline = function(feeding){
                return feeding.unique === lactaryFeeding.unique && feeding.sequential === lactaryFeeding.sequential && feeding.id ===lactaryFeeding.id;
            };
            var getIndexOffline = function (feeding){
                return feeding.key === lactaryFeeding.key && feeding.id === lactaryFeeding.id;  
            };
            funcIndex = getLocalVar('__ONLINE') ? getIndexOnline : getIndexOffline;

            var indexLactaryFeedings = widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings.findIndex(funcIndex);
            var indexPatientLactaryFeedings = widgetDetailsPatient.curPatient.LactaryFeedings.findIndex(funcIndex);
            // lactaryFeedingCrud.removeRow(widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings[indexLactaryFeedings], "/LactaryFeedings");
            var plateParams = {
                'plates': widgetDetailsPatient.curPatient.LactaryFeedings[indexPatientLactaryFeedings],
                'dataSourceName': "/LactaryFeedings",
                'widgetDetailsPatient': widgetDetailsPatient
            };
            eventAggregator.publish('callDeletePlatesOnline', plateParams);
            widgetPopupPrescricao.currentRow.lactaryFeedings.splice(index,1);
            widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings.splice(indexLactaryFeedings,1);
            widgetDetailsPatient.curPatient.LactaryFeedings.splice(indexPatientLactaryFeedings,1);
        });
    };
    $scope.deleteEnteralFeeding = function(enteralFeeding){
        ScreenService.confirmMessage(ScreenService.i18n("Deseja realmente excluir este prato da prescrição?"), 'question', 
        function(accept){
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');  
            var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');  
            var index = widgetPopupPrescricao.currentRow.feedings.indexOf(enteralFeeding);
            var indexEnteralFeedings = widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings.findIndex(function(feeding){
                return feeding.key === enteralFeeding.key;
            });
            var indexPatientEnteralFeedings = widgetDetailsPatient.curPatient.EnteralFeedings.findIndex(function(feeding){
                return feeding.key === enteralFeeding.key;
            });
            // enteralFeedingCrud.removeRow(widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings[indexEnteralFeedings], "/EnteralFeedings");
            var plateParams = {
                'plates': widgetDetailsPatient.curPatient.EnteralFeedings[indexPatientEnteralFeedings],
                'dataSourceName': "/EnteralFeedings",
                'widgetDetailsPatient': widgetDetailsPatient
            };
            eventAggregator.publish('callDeletePlatesOnline', plateParams);
            widgetPopupPrescricao.currentRow.feedings.splice(index,1);
            widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings.splice(indexEnteralFeedings,1);
            widgetDetailsPatient.curPatient.EnteralFeedings.splice(indexPatientEnteralFeedings,1);
        });
    };
    $scope.copiedPrescriptionDiet = function(){
        var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        if(widgetPopupPrescricao.getField('serviceId').value()){
            ScreenService.confirmMessage(ScreenService.i18n("Ao copiar informações da prescrição dietética, a configuração atual pode sofrer alterações. Confirmar operação?"), 'question',
            function(accept){
                var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
                var prescriptions = widgetDetailsPatient.curPatient.Prescriptions;
                if(Array.isArray(prescriptions) && prescriptions.length){
                    var service = widgetPopupPrescricao.currentRow.idformdiet;
                    var exists = false; 
                    var diets = widgetDetailsPatient.localDataSourceLoaded.Diets;
                    var dietsServices = widgetDetailsPatient.localDataSourceLoaded.DietsServices;
                    for(var i=0; i<prescriptions.length; i++){
                        var selectedDiet = diets.filter(function(diet){
                            return diet.dietId === prescriptions[i].dietId;
                        })[0];
                        if(Array.isArray(service) && service.length){
                            var index = service.indexOf(selectedDiet.service);
                            if(index > -1){
                                var foundedDiet = dietsServices.filter(function(dietService){
                                    return dietService.serviceId === widgetPopupPrescricao.getField('serviceId').value() && dietService.service === selectedDiet.service && dietService.dietId === selectedDiet.dietId;
                                });
                                if(Array.isArray(foundedDiet) && foundedDiet.length){
                                    
                                    exists = true;
                                    widgetPopupPrescricao.getField('dietId').value(selectedDiet.dietId);
                                    eventAggregator.publish('updateDietFromPrescriptionDiet', widgetPopupPrescricao);
                                    break;
                                }
                            }
                        }else{
                            if(selectedDiet.service === service){
                                var foundedDietService = dietsServices.filter(function(dietService){
                                    return dietService.serviceId === widgetPopupPrescricao.getField('serviceId').value() && dietService.service === selectedDiet.service;
                                });
                                if(Array.isArray(foundedDietService) && foundedDietService.length){
                                    exists = true;
                                    widgetPopupPrescricao.getField('dietId').value(selectedDiet.dietId);
                                    eventAggregator.publish('updateDietFromPrescriptionDiet', widgetPopupPrescricao);
                                    break;
                                }
                            }
                        }
                    }
                    if(exists){
                        ScreenService.showMessage(ScreenService.i18n("Prescrição dietética copiada com sucesso!"), 'success'); 
                    }else{
                        ScreenService.showMessage(ScreenService.i18n("Não foi encontrada prescrição dietética compatível com o tipo de prescrição que está sendo inserida!"));
                    }
                    
                }else{
                    ScreenService.showMessage(ScreenService.i18n("Não há prescrições dietéticas para o paciente atual!"));
                }
            }); 
        } else{
            ScreenService.showMessage(ScreenService.i18n("Selecione um serviço para realizar a cópia de Prescrição dietética!"));
        }
    };

    $scope.startEditing = function (otherData, model, field) {
        if(field.name == 'foodIntoleranceGroup'){
            $scope.startEdditingFoodGroup(field);
        } else{
            $scope.editorOptions = {
                toolbar: [
                    ['edit', ['undo', 'redo']],
                    ['style', ['bold', 'italic', 'underline', 'superscript', 'subscript', 'strikethrough', 'clear']],
                    ['textsize', ['fontsize']],
                    ['fontclr', ['color']]
                ]
            };
            widget.fieldEdited = {};
            widget.fieldEdited.currentEditedModel = model;
            widget.fieldEdited.isEditting = true;
            widget.fieldEdited.type = field.type;
            widget.fieldEdited.currentEditedLabel = field.label;
            widget.fieldEdited.currentEditedName = field.name;
            widget.fieldEdited.otherData = otherData;
            widget.fieldEdited.__is_new = false;
            widget.fieldEdited.serviceDate = moment().format('DD/MM/YYYY');
            widget.edit();
            widget.container.showMenu = false;
        }
    };
    
    $scope.startEdditingFoodGroup = function(field){
        widget.popupFoodGroup = angular.copy(widget.localDataSourceLoaded.AllGroupSubgroup);
        var popupFoodGroup = widget.popupFoodGroup;
        var dataFoodGroup = widget.dataFoodGroup;
        dataFoodGroup.forEach(function(group){
            var foundedGroup = popupFoodGroup.filter(function(g){
                return g.cdsubgrprod === group.cdsubgrprod && g.cdgrupprod === group.cdgrupprod;
            });
            var selectedGroup = foundedGroup[0];
            if(selectedGroup){
                selectedGroup.selectedRow = true;
            }
            
        });
        if(dataFoodGroup && dataFoodGroup.length){
        	widget.selectedRows = angular.copy(dataFoodGroup); 
        }else{
        	widget.selectedRows = [];
        }
        var rowsSelected = popupFoodGroup.filter(function(row) {
            return row.selectedRow;
        });
        widget.allFieldsAreSelected = rowsSelected.length === popupFoodGroup.length ? true : false;
        widget.showPopUpRestr = true;
        widget.edit();
    };

    $scope.getBedsSource = function (fieldEdited) {

        var isNew = fieldEdited.__is_new;
        setSource = isNew ? 'BedsPrescriptions' : 'Beds';

        var bedsSource = widget.localDataSourceLoaded[setSource];

        if (isNew) {

            if (!$scope.copyLoadedBedsSource || $scope.loadedBedsSource != bedsSource) {
                $scope.loadedBedsSource = bedsSource;
                $scope.copyLoadedBedsSource = angular.copy(bedsSource);
                $scope.copyLoadedBedsSource.push(getCurPatientHospitalizationOnBedsSource());
            }
            return $scope.copyLoadedBedsSource;

        }

        return bedsSource;
    };

    /**
     * Retorna os dados de internação do paciente ativo.
     */
    function getCurPatientHospitalizationOnBedsSource() {

        var curPatient = widget.curPatient;
        var hospitalizationsDataSourceName = "Beds";
        var hospitalizations = widget.localDataSourceLoaded[hospitalizationsDataSourceName];

        var patientsHopitalization = hospitalizations.filter(function (hospitalization) {
            return hospitalization.bed === curPatient.bed && hospitalization.hall === curPatient.hall && hospitalization.storey === curPatient.storey;
        });

        var patientHopitalization = patientsHopitalization[0];
        var serializedHospitalization = [];

        if (patientHopitalization) {
            serializedHospitalization = {
                "unit": patientHopitalization.unit,
                "storeyName": patientHopitalization.storeyName,
                "hallName": patientHopitalization.hallName,
                "bedName": patientHopitalization.bedName,
                "bed": patientHopitalization.bed
            };
        }
        return serializedHospitalization;

    }

    $scope.checkAval = function (appointMentDate) {
        var currAppDate = angular.copy(appointMentDate);
        var dateSaved = getLocalVar('__appointment');
        currAppDate = moment(currAppDate, 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY HH:mm');
        return (currAppDate === dateSaved || currAppDate === moment(dateSaved, 'YYYY-MM-DDTHH:mm').format('DD/MM/YYYY HH:mm') || currAppDate === moment(dateSaved, 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY HH:mm'));
    };
    
    $scope.showElementsHidden = function (event) {
        var itemList = $(event.currentTarget);
        itemList.hasClass('hide-all') ? itemList.removeClass('hide-all') : itemList.addClass('hide-all');
        itemList.siblings('.fields-dietPrescription').addClass('hide-all');
    };
    $scope.getNutritionalName = function (nutritionalStatusId) {
        var nutritionalStatus = widget.localDataSourceLoaded.NutritionalStatus;
        var retorno = false;
        nutritionalStatus.forEach(function (nut) {
            if (nutritionalStatusId === nut.nutritionalStatusId) {
                retorno = nut.nutritionalStatusName;
            }
        });
        return retorno;
    };

    $scope.setLocationData = function (bedId, prescription) {
        widget.localDataSourceLoaded.BedsPrescriptions.forEach(function (beditem) {
            if (beditem.bed === bedId) {
                prescription.unit = beditem.unit;
                prescription.hallName = beditem.hallName;
                prescription.storeyName = beditem.storeyName;
            }
        });
    };

    $scope.startEditingPrescription = function (prescription) {
        var indexEdited = widget.curPatient.Prescriptions.indexOf(prescription);
        var newDietPrescriptionPopup = widget.container.getWidget('dietPrescriptionPopup');
        newDietPrescriptionPopup.newRow();
        
        newDietPrescriptionPopup.currentRow = {};
        newDietPrescriptionPopup.currentRow = widget.curPatient.Prescriptions[indexEdited];
        newDietPrescriptionPopup.currentRow.isEditting = true;
        newDietPrescriptionPopup.currentRow.changed = true;
        newDietPrescriptionPopup.label = ScreenService.i18n('Editar Prescrição Dietética');
        newDietPrescriptionPopup.getField('date').readOnly = true;
        newDietPrescriptionPopup.getField('dietId').readOnly = true;
        widget.prescriptionBackup = angular.copy(widget.curPatient.Prescriptions[indexEdited]);
        reloadAllFields(newDietPrescriptionPopup);
        ScreenService.openPopup(newDietPrescriptionPopup);
    };

    $scope.createNewPrescription = function () {
        
        var newDietPrescriptionPopup = widget.container.getWidget('dietPrescriptionPopup');
        newDietPrescriptionPopup.newRow();
        widget.curEditing = 'Prescriptions';
        newDietPrescriptionPopup.label = ScreenService.i18n('Nova Prescrição Dietética');

        newDietPrescriptionPopup.currentRow.isEditting = false;
        newDietPrescriptionPopup.currentRow.id = widget.curPatient.id;
        newDietPrescriptionPopup.currentRow.unit = widget.curPatient.unit;
        newDietPrescriptionPopup.currentRow.date = moment().format("YYYY-MM-DDTHH:mm:ss");
        newDietPrescriptionPopup.currentRow.changed = true;
        newDietPrescriptionPopup.currentRow.__is_new = true;
        newDietPrescriptionPopup.currentRow.originalIsNewStatus = true;
        
        var curPatientBed = $scope.getCurPatientBedName();
        newDietPrescriptionPopup.currentRow.bed         = curPatientBed.bed;
        newDietPrescriptionPopup.currentRow.bedName     = curPatientBed.bedName;
        
        newDietPrescriptionPopup.currentRow.hall        = curPatientBed.hall;
        newDietPrescriptionPopup.currentRow.hallName    = curPatientBed.hallName;
        
        newDietPrescriptionPopup.currentRow.storey      = curPatientBed.storey;
        newDietPrescriptionPopup.currentRow.storeyName  = curPatientBed.storeyName;
        
        newDietPrescriptionPopup.getField('date').readOnly = false;
        newDietPrescriptionPopup.getField('dietId').readOnly = false;
        reloadAllFields(newDietPrescriptionPopup);
        ScreenService.openPopup(newDietPrescriptionPopup);
    };

    $scope.getCurPatientHallName = function () {

        var curPatientInfos = widget.curPatient;
        var localDataSourceLoaded = widget.localDataSourceLoaded;

        var curPatientHall = localDataSourceLoaded.Halls.filter(function (hall) {
            return hall.hall === curPatientInfos.hall;
        });

        return curPatientHall[0];

    };

    $scope.getCurPatientStoreyName = function () {

        var curPatientInfos = widget.curPatient;
        var localDataSourceLoaded = widget.localDataSourceLoaded;

        var curPatientStorey = localDataSourceLoaded.Storeys.filter(function (storey) {
            return storey.storey === curPatientInfos.storey;
        });

        return curPatientStorey[0];

    };

    $scope.getCurPatientBedName = function () {

        var curPatientInfos = widget.curPatient;
        var localDataSourceLoaded = widget.localDataSourceLoaded;

        var curPatientBed = localDataSourceLoaded.Beds.filter(function (bed) {
            return bed.bed === curPatientInfos.bed;
        });

        return curPatientBed[0];

    };

    $scope.startEditingAppointment = function (appointment) {
        var indexEdited = widget.curPatient.Appointments.indexOf(appointment);
        var openConsultaPopup = widget.container.getWidget('appointmentsPopup');

        openConsultaPopup.currentRow = widget.curPatient.Appointments[indexEdited];
        openConsultaPopup.currentRow.height = parseFloat(openConsultaPopup.currentRow.height);
        openConsultaPopup.currentRow.weight = parseFloat(openConsultaPopup.currentRow.weight);
        openConsultaPopup.currentRow.isEditting = true;
        openConsultaPopup.currentRow.__is_new = false;
        openConsultaPopup.label = ScreenService.i18n('Editar Consulta');
        widget.appointmentBackup = angular.copy(widget.curPatient.Appointments[indexEdited]);
        openConsultaPopup.edit();
        ScreenService.openPopup(openConsultaPopup);
        reloadAllFields(openConsultaPopup);

    };

    $scope.createNewApointment = function () {
        widget.curEditing = 'Appointments';
        widget.subitemtoadd = '';
        widget.onAddAppointment = true;
        var openConsultaPopup = widget.container.getWidget('appointmentsPopup');
        openConsultaPopup.newRow();
        openConsultaPopup.label = 'Nova Consulta';
        //changed push for unshift
        openConsultaPopup.currentRow.nutritionistName = getLocalVar('__NMOPERADOR');
        openConsultaPopup.currentRow.nutritionistId = getLocalVar('__CDOPERADOR');
        openConsultaPopup.currentRow.isEditting = false;
        openConsultaPopup.currentRow.id = widget.curPatient.id;
        openConsultaPopup.needToSave = true;
        openConsultaPopup.currentRow.changed = true;
        openConsultaPopup.currentRow.__is_new = true;
        openConsultaPopup.currentRow.date = moment().format('YYYY-MM-DDTHH:mm:ss');
        openConsultaPopup.actions[0].label = ScreenService.i18n('Cancelar');
        reloadAllFields(openConsultaPopup);
        ScreenService.openPopup(openConsultaPopup);
        openConsultaPopup.edit();
    };
    
    eventAggregator.subscribe('changeSelectedExam', function(widget) {
        changeSelectedExam(widget);
    });
    function changeSelectedExam(widget){
        var newExamId = widget.currentRow.examinationId;
        var newAvaliation = widget.currentRow;
        var exams = widget.getParent().localDataSourceLoaded.Examinations;
        var curExam;
        if(exams){
            exams.forEach(function (exam) {
                if (exam.examinationId === newExamId) {
                    curExam = exam;
                }
            });
    
            if (curExam) {
                newAvaliation.examinationId = curExam.examinationId;
                newAvaliation.examinationUnit = curExam.examinationUnit;
                newAvaliation.examinationName = curExam.examinationName;
                newAvaliation.examinationFormula = curExam.examinationFormula;
            }
            newAvaliation.age = widget.getParent().curPatient.age;
            newAvaliation.gender = widget.getParent().curPatient.gender;
            defineClassification(widget);
            newAvaliation.examinationId = newExamId;
        }
    }
    function defineClassification(widget){
        if(widget.currentRow.examinationValue){
            widget.currentRow.examinationStatus = setClassification(widget.currentRow);
            widget.getField('examinationStatus').setValue(ScreenService.i18n(widget.currentRow.examinationStatus));
        }else{
            widget.currentRow.examinationStatus = '';
        }
    }

    $scope.startEditingBioEval = function (bioAvaliation) {
        var bioEvalPopup = widget.container.getWidget('bioEvalPopup');
        var bioAval = angular.copy(bioAvaliation);
        bioEvalPopup.showAddEditBioExam = true;
        bioEvalPopup.isSubItemBeingAdded = false;
        bioEvalPopup.currentRow = bioAval;
        bioEvalPopup.currentRow.key = bioAval.key;
        bioEvalPopup.currentRow.oldExamination = bioEvalPopup.currentRow.examinationId;
        bioEvalPopup.currentRow.examinationValue = parseFloat(bioEvalPopup.currentRow.examinationValue);
        bioEvalPopup.currentRow.examinationDate = moment(bioEvalPopup.currentRow.examinationDate, 'DD/MM/YYYY HH:mm:ss').format('YYYY-MM-DDTHH:mm:ss');
        bioEvalPopup.currentRow.priority = parseFloat(bioEvalPopup.currentRow.priority);
        bioEvalPopup.currentRow.pregnancyWeek = parseFloat(bioEvalPopup.currentRow.pregnancyWeek);
        bioEvalPopup.getField('pregnancyWeek').readOnly = bioEvalPopup.currentRow.pregnancyId === 'S' ? false : true;
        bioEvalPopup.getField('examinationDate').readOnly = true;
        bioEvalPopup.currentRow.examinationStatus = setClassification(bioEvalPopup.currentRow);
        eventAggregator.publish("disablePregnancyWeeks", bioEvalPopup);
        reloadAllFields(bioEvalPopup);
        ScreenService.openPopup(bioEvalPopup);
        bioEvalPopup.edit();
    };
    
    $scope.openPopupFieldAutoComplete = function(field){
        if(!field.disabled){
            var widget = field.widget;
            var widgetPopup;
            switch(widget.name){
                case "widgetPopupItemEnt":
                    widgetPopup = field.widget.container.getWidget('widgetPopupFormulationEnt');
                    if(Array.isArray(widgetPopup.dataSourceFilter)){
                        widgetPopup.dataSourceFilter.forEach(function(filter){
                            if(widget.currentRow[filter.name]){
                                filter.value = widget.currentRow[filter.name];
                            }
                        });
                    }
                    break;
                case "widgetPopupItemLac":
                    widgetPopup = field.widget.container.getWidget('widgetPopupFormulationLact');
                    break;
                case "newHospitalizationPopup":
                    widgetPopup = field.widget.container.getWidget('widgetPopupHospitalization');
                    break;
            }
            if(widgetPopup){
                if(widgetPopup.propertyToFind && Array.isArray(widgetPopup.dataSourceFilter)){
                    widgetPopup.dataSourceFilter.forEach(function(filter){
                        if(filter.name === widgetPopup.propertyToFind){
                            if(filter.operator==="LIKE" || filter.operator==="LIKE_I"){
                                filter.value = "%%";
                            } else {
                                filter.value = "";
                            }
                        }
                    });
                }
                var dataSourceName = widgetPopup.descriptionPopupSelect.dataSourceName;
                if(field.dataSourceFilter && field.dataSourceFilter.length){
                    reloadFilterField(widget, [field.name], widget.currentRow).then(function(){
                        // widgetPopup.elementsToSearch = angular.copy(field.dataSource.data);
                        widgetPopup[dataSourceName] = field.dataSource.data;
                        widgetPopup.descriptionPopupSelect.titleFieldSearch = ScreenService.i18n(widgetPopup.descriptionPopupSelect.titleFieldSearch);
                        widgetPopup.reload().then(function(){
                            ScreenService.openPopup(widgetPopup);
                            widgetPopup.isVisible = true;
                        });
                    });
                }else{
                    // widgetPopup.elementsToSearch = angular.copy(field.dataSource.data);
                    // widgetPopup[dataSourceName] = angular.copy(field.dataSource.data);
                    widgetPopup.descriptionPopupSelect.titleFieldSearch = ScreenService.i18n(widgetPopup.descriptionPopupSelect.titleFieldSearch);
                    widgetPopup.reload().then(function(){
                        ScreenService.openPopup(widgetPopup);
                        widgetPopup.isVisible = true;
                    });
                    // widgetPopup.dataSource.data = [];
                }
            }
        }
    };
    function clearValueField(field){
        if(Array.isArray(field.dependentFields) && field.dependentFields.length){
            field.dependentFields.forEach(function(nameDependentField){
                var currentField = field.widget.getField(nameDependentField);
                currentField.value("");
                field.widget.currentRow[currentField.valueField] = "";
                field.widget.currentRow[currentField.descriptionField] = "";
                if(Array.isArray(currentField.dependentFields) && currentField.dependentFields.length){
                    clearValueField(currentField);
                }
            });
        }
    }
    
    $scope.clearField = function(field){
        if(!field.isReadOnly() || !field.disabled){
            clearValueField(field);
            field.value("");
            field.widget.currentRow[field.valueField] = "";
            field.widget.currentRow[field.descriptionField] = "";
        }
    };
    
    $scope.addNewAvaliation = function(widget, avaliationName){
        if(widget.isValid()){
            var appointment = [];
            appointment.__is_new = true;
            appointment.date = widget.container.getWidget('appointmentsPopup').currentRow.date;
            widget.allFieldsAreSelected = false;
            widget.getParent().allFieldsAreSelected = false;
            var existBioAval = false;
            existsAval(widget.getParent(), appointment);
            if(avaliationName === 'bioq'){
                existBioAval = widget.getParent().existsAval;
            }
            var parameters = [widget.getParent(), avaliationName, existBioAval];
            eventAggregator.publish("AppointmentSave", parameters);
        }
    };
    function openAntrList(widget){
        var antrList = widget.container.getWidget('antrList');
        antrList.edit();
        widget.subitemtoadd = 'avaliationantr';
        var filter = [{
                "name": "unit",
                "operator": "=",
                "value": getLocalVar('__CDFILIAL')
            },
            {
                "name": "appointmentDate",
                "operator": "=",
                "value": getLocalVar('__normalizedAppointment')
            },
            {
                "name": "id",
                "operator": "=",
                "value": getLocalVar('__patient')
            }
        ];
    
        localStorageEngine.getDataSource('/DataInfoEvaluations', filter).then(function(infos) {
            widget.infos = infos;
            getAvaliationsSaved(widget);
        });
        ScreenService.openPopup(antrList);
    }
    
    function removeDuplicates(originalArray, objKey, date) {
        var trimmedArray = [];
        var values = [];
        var value;

        for (var i = 0; i < originalArray.length; i++) {
            value = originalArray[i][objKey];

            if (values.indexOf(value) === -1) {
                var copyArray = angular.copy(originalArray[i]);
                copyArray.method = copyArray.methodEvaluationId;
                copyArray.date = date;
                copyArray.unit = getLocalVar('__CDFILIAL');
                trimmedArray.push(copyArray);
                values.push(value);
            }
        }
        return trimmedArray;
    }

    eventAggregator.subscribe('avaliationsSaved', function(widget) {
        getAvaliationsSaved(widget);
    });
    eventAggregator.subscribe('reloadAllFields',function(widget){
        reloadAllFields(widget);
    });
    
    function getAvaliationsSaved(widget){
        var patientWidget = widget.container.getWidget('WidgetDetailsPatient');
        var age = patientWidget.curPatient.yearAndMonth[0];
        var date;
        if(!widget.onAddAppointment){
            date = moment(getLocalVar('__appointment'), 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        }else{
            date = moment(getLocalVar('__appointment'), 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        }
        var filterPatient = [{
                name: "dataIni",
                operator: "<=",
                value: age
            },
            {
                name: "dataFim",
                operator: ">=",
                value: age
            },
            {
                name: "methodEvaluationId",
                operator: "NOT_IN",
                value: ['30', '31', '32', '33', '34', '35', '36']
            }
        ];

        localStorageEngine.getDataSource('/AntroMethods', filterPatient).then(function(methods) {

            methods = removeDuplicates(methods, 'name', date);
            widget.antrLoadedFields = methods;
            if(patientWidget.infos){
                widget.selectedRows = selecionados(methods, patientWidget.infos);
                if(widget.selectedRows.length === widget.antrLoadedFields.length){
                    widget.allFieldsAreSelected = true;
                }
            }
        });
    }
    
    function selecionados(avaible, selected) {
        var selectedRows = [];
        avaible.forEach(function(disponivel){
            var founded = selected.filter(function(selecionado){
                return selecionado.methodEvaluationId === disponivel.methodEvaluationId;
            })[0];
            if(founded){
                disponivel.selectedRow = true;
                selectedRows.push(disponivel);
            }
        });
        return selectedRows;
    }

    $scope.addAvaliation = function (widget, appointment, avaliationName) {

        localStorageEngine.setLocalVar('__appointment', appointment.date);
        localStorageEngine.setLocalVar('__appointmentdata', appointment);
        localStorageEngine.setLocalVar('__normalizedAppointment', appointment.normalizedDate);
        widget.view();
        widget.allFieldsAreSelected = false;
        existsAval(widget, appointment);

        if (avaliationName === 'bioq') {
            widget.subitemtoadd = 'avaliationbioq';
            if(widget.existsAval){
                var bioExamList = widget.container.getWidget('bioqList');
                bioExamList.edit();
                ScreenService.openPopup(bioExamList);
            }else{
                eventAggregator.publish("openBioEvalPopup", widget);
            }
        } else if (avaliationName === 'antr') {
            widget.onAddAppointment = false;
            openAntrList(widget);
        }
        else {
            console.log(avaliationName);
        }
    };
    
    function existsAval(widget, appointment){
        var dateSaved = angular.copy(appointment.date);
        dateSaved = moment(appointment.date, 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        var exists = false;
        var bioEval = widget.curPatient.BioEvaluations;
        if(!bioEval){
            return false;
        }
        
        for(var i=0; i < bioEval.length; i++){
            if(bioEval[i].appointmentDate === dateSaved){
                exists = true;
                break;
            }
        }
        widget.existsAval = exists;
    }

    $scope.onChangeFoodControlService = function (serviceName, widget, prescription) {
        widget.redraw();
        widget.fieldEdited.selectedDiet = {};
        widget.fieldEdited.dietId = "";
        widget.fieldEdited.dietTypeName = "";
        widget.filtredFeedingForWidget = [];
        widget.filtredFeedingsForWidget = [];
        var prescriptionInfos;
        if (!Boolean(widget.fieldEdited) && Util.isEmptyOrBlank(widget.fieldEdited)) {
            prescriptionInfos = prescription;
        } else {
            prescriptionInfos = widget.currentRow ? widget.currentRow : prescription;
        }

        var filter = [
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            },
            {
                "name": "serviceId",
                "operator": "=",
                "value": prescriptionInfos.cdservfili
            }
        ];

        if (prescriptionInfos.idtpservhosp === "0") {
            filter.push({
                "name": "service",
                "operator": "IN",
                "value": ['0', '3', '4']
            });
        } else {
            filter.push({
                "name": "service",
                "operator": "=",
                "value": prescriptionInfos.idformdiet
            });
        }

        localStorageEngine.getDataSource('/DietsServices', filter).then(function (filteredDiets) {
            widget.filteredDietsFeeding = filteredDiets;
            widget.filteredServicesFeeding.forEach(function (service) {
                if (serviceName === service.serviceId) {
                    prescriptionInfos.serviceName = service.serviceName;
                }
            });
        });
    };

    $scope.openPopupSelectPlates = function (widget){
        var widgetPopupPlates = widget.container.getWidget('widgetPopupPlates');
        widgetPopupPlates.showPopupPlates = true;
        widgetPopupPlates.isVisible = true;
        widgetPopupPlates.descriptionPopupSelect.titleFieldSearch = ScreenService.i18n(widgetPopupPlates.descriptionPopupSelect.titleFieldSearch);
        widgetPopupPlates.openPopup = true;
        eventAggregator.publish('loadDataPlatesByUnit', widgetPopupPlates);
    };
    
    
    $scope.onChangeLactaryFeeding = function (widget) {
            
        var curLactaryFeeding = widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding;
        var cdformulacao = curLactaryFeeding.formulationId;
        
        var filter = [{
            "name": "cdformulacao",
            "operator": "=",
            "value": cdformulacao
        }];
        localStorageEngine.getDataSource('/LactaryPlates', filter).then(function (plate) {
            widget.container.getWidget('widgetPopupItemLac').currentRow.un = plate[0].un;
            widget.container.getWidget('widgetPopupItemLac').getField('formulationId').dataSource.data = [{
                "cdformulacao": plate.cdformulacao,
                "nmformulacao": plate.nmformulacao
            }];
            widget.container.getWidget('widgetPopupItemLac').getField('formulationId').reload();
        });
    };

    $scope.filterFeedingPatientDate = function (widget) {
        var curDate = widget.fieldEdited.serviceDate;
        var dateEdited = moment(curDate, 'DD/MM/YYYY').format('DD/MM/YYYY');
        var filtredFeedingsForWidget = [];

        curDate = dateEdited === "Invalid date" ? curDate : dateEdited;

        widget.localDataSourceLoaded.NormalFeedings.forEach(function (feed) {
            if (feed.serviceDate === curDate && feed.id === widget.curPatient.id) {
                filtredFeedingsForWidget.push(feed);
            }
        });
        return filtredFeedingsForWidget;
    };

    $scope.createNewFeedingControl = function (typeOfFeeding) {
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        widgetPopup.typePrescription = typeOfFeeding;
        widgetPopup.currentRow = {};
        configurationWidget(widgetPopup, "modeInsert");
        var currentRow = createNewFeeding(typeOfFeeding);
        currentRow.isNewRegister = true;
        widgetPopup.getField('dietId').readOnly = true;
        switch(typeOfFeeding){
            case 'oral':
                widgetPopup.label = 'Prescrição Oral';
                widgetPopup.getField('serviceId').dataSource.name = '/OralServices';
                widgetPopup.getField('solicitante').isVisible = true;
                break;
            case 'lact':
                widgetPopup.label = 'Prescrição Lactária';
                widgetPopup.getField('solicitante').isVisible = false;
                widgetPopup.getField('serviceId').dataSource.name = '/LactaryServices';
                break;
            case 'pare':
                widgetPopup.label = 'Prescrição Enteral / Parenteral';
                widgetPopup.getField('solicitante').isVisible = false;
                widgetPopup.getField('serviceId').dataSource.name = '/EnteralServices';
                break;
        }
        openPopupPrescription(widgetPopup, currentRow);
    };

    function openPopupPrescription (widgetPopup, currentRow){
        widgetPopup.newRow();
        var isNew = widgetPopup.currentRow.__is_new;
        widgetPopup.currentRow = currentRow;
        widgetPopup.currentRow.__is_new = isNew;
        reloadFilterField(widgetPopup, ['serviceId'], widgetPopup.currentRow);
        widgetPopup.getField('serviceId').value("");
        ScreenService.openPopup(widgetPopup);
    }
    
    function reloadFilterField (widget, fields, currentRow){
        var defer = ZHPromise.defer();
        fields.forEach(function(nameField, index){
            var field = widget.getField(nameField);
            if(nameField === "dietId"){
                var idformdiet = { //Diet
                    'oral': ['0', '3', '4'],
                    'lact': '1',
                    'pare': '2'
                };
                if(field.dataSourceFilter && field.dataSourceFilter.length){
                    field.dataSourceFilter.forEach(function(currentFilter) {
                        if (currentFilter.name === "service") {
                            if (currentRow.idtpservhosp === "0") {
                                currentFilter.operator = "IN";
                            }
                            currentFilter.value = idformdiet[currentRow.typeClicked];
            
                        } else {
                            currentFilter.value = currentRow[currentFilter.name];
                        }
                    });
                    field.value(currentRow.dietId);
                }
            
            }else{
                if(field.dataSourceFilter && field.dataSourceFilter.length && field.dataSourceFilterProperties){
                    field.dataSourceFilter.forEach(function(filter){
                        filter.value = widget.currentRow[field.dataSourceFilterProperties[filter.name]];
                    });
                }
            }
            field.reload().then(function(){
                if(index === fields.length-1){
                    defer.resolve();
                }
            });
        });
        return defer;
    }
    
    function configurationWidget (widget, mode){
        Object.keys(widget[mode]).forEach(function(currentProperty){
            var property = widget[mode][currentProperty];
            if(Array.isArray(property.fields)){
                property.fields.forEach(function(nameField){
                    widget.getField(nameField)[property.name] = property.value;
                });
            }else{
                Object.keys(property.fields).forEach(function(nameField){
                    widget.getField(nameField)[property.name] = property.fields[nameField];
                });
            }
            if(typeof property.widget === "object"){
                if(property.widget.condition){
                    if(property.widget.condition.currentRow){
                        widget[property.name] = property.widget.condition.values[widget.currentRow[property.widget.condition.propertyWidget]];
                    }else if (property.widget.condition.modeEdit){
                        widget[property.name] = property.widget.condition.values["modeEdit"];
                    }else{
                        widget[property.name] = property.widget.condition.values[widget[property.widget.condition.propertyWidget]];
                    }
                }
            }
        });
    }

    $scope.startEditingFeeding = function (feeding) {
        feeding.isNewRegister = false;
        var widgetPopup = widget.container.getWidget('widgetPopupPrescricao');
        var fieldSolicitante = widgetPopup.getField('solicitante');
        var typeOfFeeding = "";
        switch(feeding.idtpservhosp){
            case "0":
            case "3":
            case "4":
                typeOfFeeding = "oral";
                fieldSolicitante.isVisible = true;
                break;
            case "1":
                typeOfFeeding = "pare";
                fieldSolicitante.isVisible = false;
                break;
            case "2":
                typeOfFeeding = "lact";
                fieldSolicitante.isVisible = false;
                break;
                
        }
        widgetPopup.typePrescription = typeOfFeeding;
        configurationWidget(widgetPopup, "modeEdit");
        
        createOrEditFeedingControl(feeding);
    };
    $scope.observationForCurrentPlate = function (feeding) {
        widget.plateObservationMode = !widget.plateObservationMode;
        feeding.plateObservationMode = !feeding.plateObservationMode;
    };
    
    $scope.showRestrictionIntoleranceByPlate = function(widget, feeding){
        var restrictionAndIntolerance;
        var intolerance = [];
        var restriction = [];
        var plates;
        var msg = "";
        if(widget.name === 'widgetPopupPrescricao'){
            restrictionAndIntolerance = widget.currentRow.PatientIntoRestriction.filter(function(restIntolerance){
                return restIntolerance.CDPRATO === feeding.formulationId && restIntolerance.CDTIPOPRAT === feeding.plateType;
            });
        } else if  (widget.name === 'WidgetDetailsPatient'){
            restrictionAndIntolerance = widget.curPatient.PatientIntoRestriction.filter(function(intoRestriction){
                return intoRestriction.NRSEQUSERV === feeding.sequential;
            });
            plates = widget.curPatient.NormalFeedings.filter(function(normalFeeding){
                return normalFeeding.sequential === feeding.sequential && normalFeeding.idrestconf === '1';
            });
        }
        if(Array.isArray(restrictionAndIntolerance) && restrictionAndIntolerance.length){
            restrictionAndIntolerance.forEach(function(restIntolerance){
                if(Array.isArray(restIntolerance.INTORESTRICTION) && restIntolerance.INTORESTRICTION.length){
                    restIntolerance.INTORESTRICTION.forEach(function(intoRestriction){
                        intoRestriction.CDPRATO = restIntolerance.CDPRATO;
                        intoRestriction.CDTIPOPRAT = restIntolerance.CDTIPOPRAT;
                        if(intoRestriction.IDINTREST === 'I'){
                            intolerance.push(intoRestriction);
                        } else if(intoRestriction.IDINTREST === 'R'){
                            restriction.push(intoRestriction);
                        }
                    });
                }
            });
            if(Array.isArray(intolerance) && intolerance.length){
                msg = msg.concat("<p class=titleStructError>"+ScreenService.i18n("Intolerâncias")+"</p><br />");
                
                intolerance.forEach(function(into){
                    if(widget.name === 'WidgetDetailsPatient' && Array.isArray(plates) && plates.length){
                        var currentPlate = plates.filter(function(plate){
                            return plate.plateType === into.CDTIPOPRAT && plate.formulationId === into.CDPRATO;
                        })[0];
                        if(currentPlate){
                            msg = msg.concat(currentPlate.plateTypeName + ' - ' + currentPlate.plateName + '<br />');
                        }
                    }
                    msg = msg.concat(into.NMSUBGRPROD+"<br />");
                });
            }
            if (Array.isArray(restriction) && restriction.length) {
                msg = msg.concat("<p class=titleStructError>"+ScreenService.i18n("Restrições")+"</p><br />");
                restriction.forEach(function(rest){
                    if(widget.name === 'WidgetDetailsPatient' && Array.isArray(plates) && plates.length){
                        var currentPlate = plates.filter(function(plate){
                            return plate.plateType === rest.CDTIPOPRAT && plate.formulationId === rest.CDPRATO;
                        })[0];
                        if(currentPlate){
                            msg = msg.concat(currentPlate.plateTypeName + ' - ' + currentPlate.plateName + '<br />');
                        }
                    }
                    msg = msg.concat(rest.NMDIAGNOST+"<br />");
                });
            }
            ScreenService.showMessage(msg);
        }

    };
    $scope.unSelectedPlate = function(feeding){
        var widgetPopup = widget.container.getWidget('widgetPopupPlates');
        var dataSourceName = widgetPopup.descriptionPopupSelect.dataSourceName;
        if(Array.isArray(widgetPopup[dataSourceName]) && widgetPopup[dataSourceName].length){
            var unSelectPlate = widgetPopup[dataSourceName].filter(function(plate){
                return plate.formulationId === feeding.formulationId && feeding.plateType;
            })[0];
            unSelectPlate.selectedRow = false;
        }
        var widgetPopupPrescricao = widgetPopup.container.getWidget('widgetPopupPrescricao');
        updateBkpFeedings(widgetPopupPrescricao, feeding, true);
        var index = widget.filtredFeedingsForWidget.indexOf(feeding);
        widget.filtredFeedingsForWidget.splice(index, 1);
        
    };
     $scope.searchPlate = function(widget) {
        var dataSourceName = widget.descriptionPopupSelect.dataSourceName;
        if(widget.name === 'widgetPopupPlates'){
            widget.elementsToSearch.forEach(function(element){
                if(element.selectedRow){
                    widget[dataSourceName].filter(function(item){
                        if(item.plateType === element.plateType && item.formulationId === element.formulationId){
                            item.selectedRow = true;
                        }
                    });
                }
            });
        }
        widget.elementsToSearch = angular.copy(widget[dataSourceName]);
        if (widget.descriptionPopupSelect.search) {
            widget.elementsToSearch = widget[dataSourceName].filter(function(element) {
                var elementName;
                var propertyToSearch = widget.descriptionPopupSelect.propertyToSearch;
                elementName = removerAcentos(element[propertyToSearch].toLowerCase());
                var changeSearch = removerAcentos(widget.descriptionPopupSelect.search.toLowerCase());
                return elementName.indexOf(changeSearch) !== -1;
            });
        }
        widget.elementsToBind = widget.elementsToSearch.reduce(function(prev, current) {
            var index = prev.findIndex(function(j) {
                return j.plateType === current.plateType;
            });
            if (index < 0) {
                index = prev.length;
                prev.push({
                    plateType: current.plateType,
                    lista: []
                });
            }
            prev[index].lista.push(current);
            return prev;
        }, []).map(function(item) {
            item.lista.opened = true;
            var selected = true;
            item.lista.forEach(function(it) {
                if (!it.selectedRow) {
                    selected = false;
                }
            });
            item.lista.selectedParent = selected;
            return item.lista;
        });
        
        if(widget.elementsToSearch.filter(row=>row.selectedRow).length == widget.elementsToSearch.length ){
            widget.allFieldsAreSelected = true;
        }else{
            widget.allFieldsAreSelected = false;
        }
        widget.reload();
    };

    function removerAcentos(newStringComAcento) {
        var string = newStringComAcento;
        var mapaAcentosHex = {
            a: /[\xE0-\xE6]/g,
            A: /[\xC0-\xC6]/g,
            e: /[\xE8-\xEB]/g,
            E: /[\xC8-\xCB]/g,
            i: /[\xEC-\xEF]/g,
            I: /[\xCC-\xCF]/g,
            o: /[\xF2-\xF6]/g,
            O: /[\xD2-\xD6]/g,
            u: /[\xF9-\xFC]/g,
            U: /[\xD9-\xDC]/g,
            c: /\xE7/g,
            C: /\xC7/g,
            n: /\xF1/g,
            N: /\xD1/g
        };

        for (var letra in mapaAcentosHex) {
            var expressaoRegular = mapaAcentosHex[letra];
            string = string.replace(expressaoRegular, letra);
        }

        return string;
    }
    $scope.desactivateCurrentPlate = function (feeding, currentRow) {
        if(currentRow.idformdiet !== '4' && currentRow.dietService !== '4'){
            feeding.selectedByUser = !feeding.selectedByUser;
            if(!feeding.selectedByUser){
                feeding.plateObservationMode = false;
                feeding.observation = "";
                if ($("#clear")[0]){
                    $("#clear")[0].style.right = "-50px";
                }
                    
            }else{
                if($("#clear")[0]){
                    $("#clear")[0].style.right = "-100px";
                }    
            }
        }
    };
    $scope.onEnteralPrescriptionSelectChange = function (curFieldSource, curValue) {

        if (curFieldSource === "Formulations") {

            widget.subItemData.formulationId = curValue;

            var filter = [{
                "name": "formulationId",
                "operator": "=",
                "value": curValue
            }];
            localStorageEngine.getDataSource("/Formulations", filter).then(function (filteredForms) {
                widget.subItemData.preparationName = filteredForms[0].preparationId
            });
        } else if (curFieldSource === "d") {

            widget.subItemData.preparationId = curValue;

            $scope.setPreparationFilter(widget);

            widget.subItemData.formulationId = null;
            widget.subItemData.formulationName = null;

        } else if (curFieldSource === "AccessRoads") {
            widget.subItemData.accessRoadId = curValue;
            //filtro para pegar o AcessRoadName que corresponde ao Id retornado pelo select da tela
            var acessName = widget.fieldSource.AccessRoads.filter(function(a){
                return a.accessRoadId === curValue;
            });
            // auxiliar para mostrar na tela de Editar Controle a AcessRoadName sem interferir na original
            widget.subItemData.acessRoad = acessName[0].accessRoadName;
        }
    };
    $scope.formatDateToRemoveTime = function (serviceDate) {
        return serviceDate.toString().split(" ")[0];
    };

    $scope.setPreparationFilter = function (widget) {

        var subItemData = widget.subItemData;

        var filter = [
            {
                "name": "preparationId",
                "operator": "=",
                "value": subItemData.preparationId
            }
        ];

        localStorageEngine.getDataSource("/Formulations", filter).then(function (filteredForms) {
            widget.fieldSource.Formulations = filteredForms;
        });

    };
    
    $scope.searchElementsGrid = function(widget){
        switch (widget.name){
            case 'WidgetUnit':
                searchUnit(widget);
                break;           
            default:
                return null;
        }    
    };
    
    function searchUnit(widget) {
        var search = angular.copy(widget.search);
        search = removerAcentos(search);
        widget.dataSourceFilter = [
            {
                "name": "NORMALIZEDNAME",
                "operator": "LIKE",
                "value": '%' + search + '%'
            },
            {
                "name": "CDFILIAL",
                "operator": "<>",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];
        widget.reload();
    }
    
    function reloadAllFields(widget) {
        widget.fields.forEach(function(field) {
            if (field.dataSource) {
                field.reload([{
                    "name": [{
                        "name": field.descriptionField,
                        "direction": "ASC"
                    }],
                    "operator": "ORDER_BY",
                    "value": ""
                }]);
            }
        });
    }
    
    $scope.checkFieldLength = function(args, field) {
        var maxLength = field.maxSize;
        var precision = field.precision;
        var modeToTrunc = field.modeToTrunc;
        var caracter = field.caracter;
        var value = document.getElementById(field.name).value;
        
        if (value) {
            if(field.validToSave && !isValidNumberMask(value, maxLength, precision)){
                var formatValue = formatarNumero(value, maxLength, precision, modeToTrunc, caracter);
                formatValue = parseFloat(formatValue);
                field.clearValue();
                field.setValue(formatValue);
                field.reload();
            }
            field.validToSave = true;
        }else{
            if(!field.validToSave){
                field.clearValue();
                field.validToSave = true;
            }else{
                field.validToSave = false;
            }
        }
    };
    
    function isValidNumberMask(value, maxLength, precision){
        var formatNumber = getPartsNumber(value, precision, maxLength);
        var number = value && typeof value.toString === 'function' ? value.toString().split(',').join('').split('.').join('') : "";
        if(number.length <= maxLength && formatNumber.length === value.length){
            return true;
        }else{
            return false;
        }
    }
    
    var getPartsNumber = function(number, precision, maxLength,caracter) {
        var formatNumber = '';
        number = number.split(',').join('.');
        var precisionUser = number.split('.').length > 1 ?  number.split('.')[1].length : 0; 
        if (typeof number === 'string' && typeof precision === 'number') {
            if(typeof maxLength === 'number' && (number.length-1)-precisionUser <= maxLength-precision && precisionUser <= precision && ((precisionUser !== 0) || (precisionUser === 0 && number.length <= maxLength-precision))){
                formatNumber = number;
            }else{
                var part1 = number.substring(0, number.length - precision);
                var part2 = number.substring(number.length - precision, number.length);
                if (typeof caracter === 'string') {
                    formatNumber = part1 + caracter + part2;
                } else {
                    formatNumber = part1 + '.' + part2;
                }
            }
        }
        return formatNumber;
    };
        
    var formatarNumero = function(value, maxLength, precision, modeToTrunc, caracter) {
        var arrayFromString;
        var number = value && typeof value.toString === 'function' ? value.toString().split(',').join('').split('.').join('') : "";
        if (number && typeof caracter === 'string') {
            number = value.split(caracter).join('');
        }
        if (typeof maxLength === 'number' && typeof precision === 'number') {
            if (number && number.length > maxLength) {
                do {
                    arrayFromString = number.split('');
                    if (modeToTrunc === 'END') {
                        number = number.substring(0, maxLength);
                    } else {
                        arrayFromString.shift();
                        number = arrayFromString.join('');
                    }
                } while (number.length > maxLength);
                number = getPartsNumber(number, precision, caracter);
            } else if (number.length > precision) {
                number = getPartsNumber(number, precision, caracter);
            } else if (number.length === precision){
                number = getPartsNumber(number.padStart(maxLength, '0'), precision);
            }
        } else if (typeof precision === 'number') {
            number = getPartsNumber(number, precision, caracter);
        } else if (typeof maxLength === 'number') {
            do {
                arrayFromString = number.split('');
                if (modeToTrunc === 'END') {
                    number = number.substring(0, maxLength);
                } else {
                    arrayFromString.shift();
                    number = arrayFromString.join('');
                }
            } while (number.length > maxLength);
        }
        return number;
    };
    
    $scope.selecionarTudo = function(widget){
        
        widget.allFieldsAreSelected = !widget.allFieldsAreSelected;

        if (widget.allFieldsAreSelected) {
            widget.elementsToBind.forEach(function (parent) {
                if(!parent.selectedParent){
                    parent.selectedParent = true;
                    parent.forEach(function(row){
                        if (!row.selectedRow) {
                            row.selectedRow = true;
                            widget.selectedRows.push(row);
                        }
                    });
                }
            });
        } else {
            widget.selectedRows = [];
            widget.elementsToBind.forEach(function (parent) {
                parent.selectedParent = false;
                parent.forEach(function(row){
                    row.selectedRow = false;
                });
            });
        }
    };
    
    $scope.selecionarParent = function(element, widget){
        element.selectedParent = !element.selectedParent;
        if(element.selectedParent){
            element.forEach(function(row){
                row.selectedRow = true;
                widget.selectedRows.push(row);
            });
            
            var allSelected = true;
            widget.elementsToBind.forEach(function(pai) {
                if(!pai.selectedParent){
                    allSelected = false;
                }
            });
            widget.allFieldsAreSelected = allSelected;
        }else{
            element.forEach(function(row){
                var indexOfAtr = widget.selectedRows.indexOf(row);
                if (indexOfAtr > -1) {
                    widget.selectedRows.splice(indexOfAtr, 1);
                }
                row.selectedRow = false;
            });
            widget.allFieldsAreSelected = false;
        }
    };
    
    $scope.selecionarFilho = function(row, parent, widget){
        row.selectedRow = !row.selectedRow;
        if(!row.selectedRow){
            var indexOfAtr = widget.selectedRows.indexOf(row);
            if (indexOfAtr > -1) {
                widget.selectedRows.splice(indexOfAtr, 1);
            }
            widget.allFieldsAreSelected = false;
            parent.selectedParent = false;
        }else{
            widget.selectedRows.push(row);
            var allSonSelected = true;
            parent.forEach(function(element){
                if(!element.selectedRow){
                    allSonSelected = false;
                }
            });
            parent.selectedParent = allSonSelected;
            if(allSonSelected){
                var allSelected = true;
                widget.elementsToBind.forEach(function(pai){
                    if(!pai.selectedParent){
                        allSelected = false;
                    }
                });
                widget.allFieldsAreSelected = allSelected;
            }else{
                widget.allFieldsAreSelected = false;
            }
        }
    };
}

// FILE: ./tecdiet_templates/js/TecDietStripe.js
function TecDietStripe($scope, $timeout, eventAggregator) {
 
    this.filterList = [];
    
    var widget = $scope.widget;
    var fieldList    = widget.fields;
    var mainWidget   = $scope.widget.container.getWidget('WidgetMain');
    var stripeWidget = $scope.widget.container.getWidget('WidgetStripe');
    var hallField    = widget.getField('hallName');
    var bedField     = widget.getField('bedName');
    var patientField = widget.getField('patientType');
    stripeWidget.choosedOptions = {patientType: 'TodosOsPacientes'};
    widget.isHospitalized = true;
    
    $scope.changeFilterList = function (field) {
        var choosedOption = stripeWidget.choosedOptions[field.name];
        var fieldName = field.name;
        var choosedOptionJson = stringToJson(choosedOption);
        
        if(choosedOptionJson){
            choosedOption = choosedOptionJson;
            setFilter(choosedOption, field.filterBy, field);
        }else{
            switch (choosedOption){
                case "TodosOsLeitos":
                    allBeds(fieldName);
                break; 
                
                case "TodasAsAlas":
                    allHalls(fieldName);
                break;
                
                case "TodosOsAndares":
                    allStoreys();
                    reset();
                    widget.choosedOptions[patientField.name] = 'Internados';
                    mainWidget.dataSourceFilter = makeDataSourceFilter('situation', 'INTERNADO');
                    mainWidget.reload();
                break;
            }
        }
        if(!hasPatientInFilter()){
            mainWidget.dataSourceFilter.push({"name": "situation", "operator": "=", "value": "INTERNADO"});
            mainWidget.reload();
        }
        removeTextFilter();
    };
    
    function allStoreys(){
        widget.choosedOptions = {
            storeyName:   "TodosOsAndares",
            hallName:     "TodasAsAlas",
            bedName:      "TodosOsLeitos",
            patientType:  widget.choosedOptions[patientField.name]
        };
        hallField.dataSource.name = "/HallsDistincts";
        hallField.reload();
        reloadFields(widget);
    }
    function reset(){
        var defaultFilter  = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
        for(var i=1; i<fieldList.length; i++){
            fieldList[i].dataSourceFilter = defaultFilter;
            fieldList[i].reload();
        }
    }
    function allBeds(fieldName){
        if(stripeWidget.choosedOptions['hallName']){
            var value = valueJson('hallName');
            mainWidget.dataSourceFilter = makeDataSourceFilter('hall', value);
            
            value = valueJson('storeyName');
            mainWidget.dataSourceFilter.push({ "name": "storey", "operator": "=", "value": value });
            
            stripeWidget.getField(fieldName).reload();
            
        }else{
            if(stripeWidget.choosedOptions['storeyName']){
                value = valueJson('storeyName');
                mainWidget.dataSourceFilter = makeDataSourceFilter('storey', value);
            }else{
                mainWidget.dataSourceFilter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
            }
        }
        stripeWidget.choosedOptions[fieldName] = 'TodosOsLeitos';
        mainWidget.reload();
    }
    
    function allHalls(fieldName){
        if(stripeWidget.choosedOptions['storeyName'] !== "TodosOsAndares" && stripeWidget.choosedOptions['storeyName']){
            var value = valueJson('storeyName');
            mainWidget.dataSourceFilter = makeDataSourceFilter('storey', value);
            bedField.dataSourceFilter = makeDataSourceFilter('storey', value);
        }else{
            mainWidget.dataSourceFilter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
            mainWidget.dataSourceFilter.push({"name": "situation", "operator": "=", "value": "INTERNADO"});
            bedField.dataSourceFilter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
        }
        
        stripeWidget.choosedOptions[fieldName] = 'TodasAsAlas';
        stripeWidget.choosedOptions['bedName'] = 'TodosOsLeitos';
        bedField.reload();
        mainWidget.reload();
    }
    
    function valueJson(name){
        var option = stringToJson(stripeWidget.choosedOptions[name]);
        if(typeof option === "object"){
            switch (name) {
                case 'hallName':
                    return option.hall;
                break;
                case 'storeyName':
                    return option.storey;
                break;
                case 'bedName':
                    return option.bed;
                break;
            }
        }else{
            return stripeWidget.choosedOptions[name];
        }
    }
    
    
    function stringToJson(string){
        try{
            return JSON.parse(string);
        }catch (e){
            return false;
        } 
    }
    
    function setFilter(referenceValue, referenceName, referenceField){
        if (referenceName === 'storey') {
            var filter = [{name: referenceField.filterBy, operator: "=", value: referenceValue.storey}];
            
            for(var i=1; i<fieldList.length; i++){
                fieldList[i].dataSourceFilter = filter;
                fieldList[i].reload();
            }
            stripeWidget.choosedOptions['hallName'] = 'TodasAsAlas';
            stripeWidget.choosedOptions['bedName'] = 'TodosOsLeitos';
            mainWidget.dataSourceFilter = makeDataSourceFilter(referenceField.filterBy, referenceValue.storey);
            hallField.dataSource.name = "/Halls";
            hallField.reload();
        }else if(referenceName === 'hall'){
            stripeWidget.choosedOptions['bedName'] = 'TodosOsLeitos';
            mainWidget.dataSourceFilter = makeDataSourceFilter(referenceField.filterBy, referenceValue.hall);
            bedField.dataSourceFilter = makeDataSourceFilter(referenceField.filterBy, referenceValue.hall);
            
            if(referenceValue.storey){
                mainWidget.dataSourceFilter.push({"name": "storey", "operator": "=", "value": referenceValue.storey});
                bedField.dataSourceFilter.push({"name": "storey", "operator": "=", "value": referenceValue.storey});
            }else{
                stripeWidget.choosedOptions['storeyName'] = 'TodosOsAndares';
            }
            
            bedField.reload();
        }else{
            stripeWidget.choosedOptions['storeyName'] = referenceValue.storey;
            stripeWidget.choosedOptions['hallName'] = referenceValue.hall;
            hallField.dataSource.name = "/Halls";
            hallField.dataSourceFilter = makeDataSourceFilter('storey', referenceValue.storey);
            hallField.reload();
            mainWidget.dataSourceFilter = makeDataSourceFilter(referenceName, referenceValue.bed);
        }
        stripeWidget.choosedOptions['patientType'] = 'Internados';
        mainWidget.reload();
    }
    

    eventAggregator.subscribe('reloadScreenAndClearFilters', function (widget) {
        var detailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        detailsPatient.isVisible = false;
        widget.typeSearch = '';
        detailsPatient.container.showMenu = true;
        stripeWidget.isHospitalized = true;
        hallField.dataSource.name = "/HallsDistincts";
        reloadFields(widget);
        clearFilters();
        removeCss();
        mainWidget.activate();
    });
    
    function reloadFields(widget){
        fieldList.forEach(function(stripeField){
            stripeField.reload();
        });
    }
    
    function makeDataSourceFilter(name, value){
        var dataSourceFilter = [ {
            "name": name,
            "operator": "=",
            "value": value
        }];
        return dataSourceFilter;
    }
    
    function clearFilters() {
        var filter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
        
        fieldList.forEach(function (sf) {
            sf.dataSourceFilter = filter;
            sf.reload();
            stripeWidget.choosedOptions[sf.name] = sf.filterAll;
        });
        mainWidget.dataSourceFilter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
        mainWidget.dataSourceFilter.push(
            {
                "name": [
                    {
                        "name": "formattedName",
                        "direction": "ASC"
                    }
                ],
                "operator": "ORDER_BY",
                "value": ""
            }
        );
        mainWidget.reload();
    }
    
    $scope.PatientTypeChange = function(field){
        removeTextFilter();
        var typeFilter = field.widget.choosedOptions[field.name];
        if(typeFilter === 'NaoInternados' || typeFilter === 'TodosOsPacientes'){
            allStoreys();
            widget.isHospitalized = true;
            clearFilters();
            removeCss();
            if(typeFilter === 'NaoInternados'){
                mainWidget.dataSourceFilter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
                mainWidget.dataSourceFilter.push({ "name": "situation", "operator": "NOT_IN", "value": ["INTERNADO", "1"] });
                field.widget.choosedOptions[field.name] = 'NaoInternados';
            }else{
                mainWidget.dataSourceFilter = makeDataSourceFilter('unit', getLocalVar('__CDFILIAL'));
                field.widget.choosedOptions[field.name] = 'TodosOsPacientes';
            }
        }else{
            addCss();
            widget.isHospitalized = false;
            if(hasPatientInFilter()){
                mainWidget.dataSourceFilter.forEach(function(filtro){
                   if(filtro.name === "situation"){
                       filtro.operator = "=";
                       filtro.value = 'INTERNADO';
                   }
                });
            }else{
                if(!mainWidget.dataSourceFilter){
                    mainWidget.dataSourceFilter = makeDataSourceFilter("situation", "INTERNADO");
                }else{
                    mainWidget.dataSourceFilter.push({"name": "situation", "operator": "=", "value": "INTERNADO"});
                }
            }
        }
        mainWidget.reload();
    };
    function removeCss(){
        for(var i =0; i<3; i++){
            $('#idFilter_'+fieldList[i].name).removeClass('alterado');
            widget.choosedOptions[fieldList[i].name] = '';
        }
    }
    function addCss(){
        for(var i =0; i<3; i++){
            if(!$('#idFilter_'+fieldList[i].name).hasClass('alterado')){
                $('#idFilter_'+fieldList[i].name).addClass('alterado');
                widget.choosedOptions[fieldList[i].name] = fieldList[i].filterAll;
            }
        }
    }
    
    function hasPatientInFilter(){
        var has = false;
        if(mainWidget.dataSourceFilter){
            mainWidget.dataSourceFilter.forEach(function(filtro){
               if(filtro.name === "situation"){
                   has = true;
               } 
            });
        }
        return has;
    }
    
    function removeTextFilter(){
        mainWidget.typeSearch = '';
        mainWidget.dataSourceFilter.splice(
            mainWidget.dataSourceFilter.indexOf(
                mainWidget.dataSourceFilter.filter(function(index){
                    index.name === "formattedName";
                })[0]),
        1);
    }
}

// FILE: ./tecdiet_templates/js/WelcomeScreenController.js
function WelcomeScreenController($scope, $timeout, ScreenService, eventAggregator, localStorageEngine) {

    this.sayHelloTo = function(title) {
        return  ScreenService.i18n(title) + getLocalVar('__NMOPERADOR');
    };

    this.openNextWindow = function(nextWindow) {
        if (nextWindow === 'Synchronize') {
            localStorageEngine.setLocalVar('__ONLINE', true);
            nextWindow = 'MapNutritionist';
            eventAggregator.publish("wscCallUpdate", null);
        }else{
            localStorageEngine.setLocalVar('__ONLINE', false);
        }
        ScreenService.openWindow(nextWindow);
    };
}
